@@ -17,10 +17,10 @@
 # history and logs, available at http://subversion.tigris.org/.
 # ====================================================================
 
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/contrib/client-side/svnmerge/svnmerge-migrate-history.py $
-# $LastChangedDate: 2008-05-14 14:44:27 +0000 (Wed, 14 May 2008) $
+# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.5.x/contrib/client-side/svnmerge/svnmerge-migrate-history.py $
+# $LastChangedDate: 2009-02-25 13:00:35 +0000 (Wed, 25 Feb 2009) $
 # $LastChangedBy: hwright $
-# $LastChangedRevision: 31164 $
+# $LastChangedRevision: 876197 $
 
 import warnings
 warnings.filterwarnings('ignore', '.*', DeprecationWarning)
@@ -256,6 +256,13 @@ class Migrator:
 
       # Manipulate the merge history.
       if new_mergeinfo_prop_val != mergeinfo_prop_val:
+        # Run the final version of the new svn:mergeinfo through the
+        # parser to ensure it is in canonical form, e.g. no overlapping
+        # or unordered rangelists, see
+        # http://subversion.tigris.org/issues/show_bug.cgi?id=3302.
+        mergeinfo = svn.core.svn_mergeinfo_parse(new_mergeinfo_prop_val)
+        new_mergeinfo_prop_val = mergeinfo2str(mergeinfo)
+        
         self.log("Queuing change of %s to '%s'"
                  % (svn.core.SVN_PROP_MERGEINFO,
                     self.flatten_prop(new_mergeinfo_prop_val)))
