@@ -22,11 +22,12 @@
 
 /*** Includes. ***/
 
+#include "svn_types.h"
 #include "svn_wc.h"
 #include "svn_client.h"
 #include "svn_error.h"
 #include "client.h"
-
+#include "private/svn_wc_private.h"
 
 
 /*** Code. ***/
@@ -36,15 +37,29 @@ svn_client_resolved(const char *path,
                     svn_boolean_t recursive,
                     svn_client_ctx_t *ctx,
                     apr_pool_t *pool)
+{
+  svn_depth_t depth = (recursive ? svn_depth_infinity : svn_depth_empty);
+  return svn_client_resolve(path, depth,
+                            svn_wc_conflict_choose_merged, ctx, pool);
+}
+
+svn_error_t *
+svn_client_resolve(const char *path,
+                   svn_depth_t depth,
+                   svn_wc_conflict_choice_t conflict_choice,
+                   svn_client_ctx_t *ctx,
+                   apr_pool_t *pool)
 {
   svn_wc_adm_access_t *adm_access;
+  int adm_lock_level = SVN_WC__LEVELS_TO_LOCK_FROM_DEPTH(depth);
 
   SVN_ERR(svn_wc_adm_probe_open3(&adm_access, NULL, path, TRUE,
-                                 recursive ? -1 : 0,
+                                 adm_lock_level,
                                  ctx->cancel_func, ctx->cancel_baton,
                                  pool));
 
-  SVN_ERR(svn_wc_resolved_conflict2(path, adm_access, TRUE, TRUE, recursive,
+  SVN_ERR(svn_wc_resolved_conflict3(path, adm_access, TRUE, TRUE, depth,
+                                    conflict_choice,
                                     ctx->notify_func2, ctx->notify_baton2,
                                     ctx->cancel_func, ctx->cancel_baton,
                                     pool));
