@@ -1,6 +1,6 @@
-/* 
+/*
  * text-delta.c -- Internal text delta representation
- * 
+ *
  * ====================================================================
  * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
@@ -55,7 +55,7 @@ struct txdelta_baton {
   /* Calculated digest from MD5 operations.
      NOTE:  This is only valid after this stream has returned the NULL
      (final) window.  */
-  unsigned char digest[APR_MD5_DIGESTSIZE]; 
+  unsigned char digest[APR_MD5_DIGESTSIZE];
 };
 
 
@@ -151,7 +151,7 @@ compute_window(const char *data, apr_size_t source_len, apr_size_t target_len,
     svn_txdelta__vdelta(&build_baton, data, source_len, target_len, pool);
   else
     svn_txdelta__xdelta(&build_baton, data, source_len, target_len, pool);
-  
+
   /* Create and return the delta window. */
   window = svn_txdelta__make_window(&build_baton, pool);
   window->sview_offset = source_offset;
@@ -349,7 +349,7 @@ txdelta_next_window(svn_txdelta_window_t **window,
 
   *window = compute_window(b->buf, source_len, target_len,
                            b->pos - source_len, pool);
-  
+
   /* That's it. */
   return SVN_NO_ERROR;
 }
@@ -498,7 +498,7 @@ svn_txdelta_target_push(svn_txdelta_window_handler_t handler,
 /* Functions for applying deltas.  */
 
 /* Ensure that BUF has enough space for VIEW_LEN bytes.  */
-static APR_INLINE void
+static APR_INLINE svn_error_t *
 size_buffer(char **buf, apr_size_t *buf_size,
             apr_size_t view_len, apr_pool_t *pool)
 {
@@ -507,8 +507,13 @@ size_buffer(char **buf, apr_size_t *buf_size,
       *buf_size *= 2;
       if (*buf_size < view_len)
         *buf_size = view_len;
+      if (APR_ALIGN_DEFAULT(*buf_size) < *buf_size)
+        return svn_error_create(SVN_ERR_SVNDIFF_INVALID_OPS, NULL,
+                                "Diff stream resulted in invalid buffer size.");
       *buf = apr_palloc(pool, *buf_size);
     }
+
+  return SVN_NO_ERROR;
 }
 
 
@@ -609,7 +614,7 @@ apply_window(svn_txdelta_window_t *window, void *baton)
                  >= ab->sbuf_offset + ab->sbuf_len)));
 
   /* Make sure there's enough room in the target buffer.  */
-  size_buffer(&ab->tbuf, &ab->tbuf_size, window->tview_len, ab->pool);
+  SVN_ERR(size_buffer(&ab->tbuf, &ab->tbuf_size, window->tview_len, ab->pool));
 
   /* Prepare the source buffer for reading from the input stream.  */
   if (window->sview_offset != ab->sbuf_offset
@@ -618,7 +623,8 @@ apply_window(svn_txdelta_window_t *window, void *baton)
       char *old_sbuf = ab->sbuf;
 
       /* Make sure there's enough room.  */
-      size_buffer(&ab->sbuf, &ab->sbuf_size, window->sview_len, ab->pool);
+      SVN_ERR(size_buffer(&ab->sbuf, &ab->sbuf_size, window->sview_len,
+              ab->pool));
 
       /* If the existing view overlaps with the new view, copy the
        * overlap to the beginning of the new buffer.  */
@@ -709,7 +715,7 @@ svn_txdelta_apply(svn_stream_t *source,
 
 /* Convenience routines */
 
-svn_error_t * 
+svn_error_t *
 svn_txdelta_send_string(const svn_string_t *string,
                         svn_txdelta_window_handler_t handler,
                         void *handler_baton,
@@ -731,10 +737,10 @@ svn_txdelta_send_string(const svn_string_t *string,
 
   /* Push the one window at the handler. */
   SVN_ERR((*handler)(&window, handler_baton));
-  
+
   /* Push a NULL at the handler, because we're done. */
   SVN_ERR((*handler)(NULL, handler_baton));
-  
+
   return SVN_NO_ERROR;
 }
 
