@@ -1,7 +1,7 @@
 /* dag.h : DAG-like interface filesystem, private to libsvn_fs
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -57,7 +57,7 @@ extern "C" {
 
 /* Given a filesystem FS, which contains all the necessary tables,
    create the initial revision 0, and the initial root directory.  */
-svn_error_t *svn_fs_base__dag_init_fs (svn_fs_t *fs);
+svn_error_t *svn_fs_base__dag_init_fs(svn_fs_t *fs);
 
 
 
@@ -68,66 +68,66 @@ typedef struct dag_node_t dag_node_t;
 
 /* Fill *NODE with a dag_node_t representing node revision ID in FS,
    allocating in POOL.  */
-svn_error_t *svn_fs_base__dag_get_node (dag_node_t **node,
-                                        svn_fs_t *fs,
-                                        const svn_fs_id_t *id,
-                                        trail_t *trail,
-                                        apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_node(dag_node_t **node,
+                                       svn_fs_t *fs,
+                                       const svn_fs_id_t *id,
+                                       trail_t *trail,
+                                       apr_pool_t *pool);
 
 
 /* Return a new dag_node_t object referring to the same node as NODE,
    allocated in POOL.  */
-dag_node_t *svn_fs_base__dag_dup (dag_node_t *node,
-                                  apr_pool_t *pool);
+dag_node_t *svn_fs_base__dag_dup(dag_node_t *node,
+                                 apr_pool_t *pool);
 
 
 /* Return the filesystem containing NODE.  */
-svn_fs_t *svn_fs_base__dag_get_fs (dag_node_t *node);
+svn_fs_t *svn_fs_base__dag_get_fs(dag_node_t *node);
 
 
 /* Set *REV to NODE's revision number, as part of TRAIL.  If NODE has
    never been committed as part of a revision, set *REV to
    SVN_INVALID_REVNUM.  */
-svn_error_t *svn_fs_base__dag_get_revision (svn_revnum_t *rev,
-                                            dag_node_t *node,
-                                            trail_t *trail,
-                                            apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_revision(svn_revnum_t *rev,
+                                           dag_node_t *node,
+                                           trail_t *trail,
+                                           apr_pool_t *pool);
 
 
 /* Return the node revision ID of NODE.  The value returned is shared
    with NODE, and will be deallocated when NODE is.  */
-const svn_fs_id_t *svn_fs_base__dag_get_id (dag_node_t *node);
+const svn_fs_id_t *svn_fs_base__dag_get_id(dag_node_t *node);
 
 
 /* Return the created path of NODE.  The value returned is shared
    with NODE, and will be deallocated when NODE is.  */
-const char *svn_fs_base__dag_get_created_path (dag_node_t *node);
+const char *svn_fs_base__dag_get_created_path(dag_node_t *node);
 
 
 /* Set *ID_P to the node revision ID of NODE's immediate predecessor,
    or NULL if NODE has no predecessor, as part of TRAIL.  The returned
    ID will be allocated in POOL.  */
-svn_error_t *svn_fs_base__dag_get_predecessor_id (const svn_fs_id_t **id_p,
-                                                  dag_node_t *node,
-                                                  trail_t *trail,
-                                                  apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_predecessor_id(const svn_fs_id_t **id_p,
+                                                 dag_node_t *node,
+                                                 trail_t *trail,
+                                                 apr_pool_t *pool);
 
 
 /* Set *COUNT to the number of predecessors NODE has (recursively), or
    -1 if not known, as part of TRAIL.  */
-svn_error_t *svn_fs_base__dag_get_predecessor_count (int *count,
-                                                     dag_node_t *node,
-                                                     trail_t *trail,
-                                                     apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_predecessor_count(int *count,
+                                                    dag_node_t *node,
+                                                    trail_t *trail,
+                                                    apr_pool_t *pool);
 
 
 /* Return non-zero IFF NODE is currently mutable under Subversion
    transaction TXN_ID.  */
-svn_boolean_t svn_fs_base__dag_check_mutable (dag_node_t *node,
-                                              const char *txn_id);
+svn_boolean_t svn_fs_base__dag_check_mutable(dag_node_t *node,
+                                             const char *txn_id);
 
 /* Return the node kind of NODE. */
-svn_node_kind_t svn_fs_base__dag_node_kind (dag_node_t *node);
+svn_node_kind_t svn_fs_base__dag_node_kind(dag_node_t *node);
 
 /* Set *PROPLIST_P to a PROPLIST hash representing the entire property
    list of NODE, as part of TRAIL.  The hash has const char * names
@@ -136,32 +136,90 @@ svn_node_kind_t svn_fs_base__dag_node_kind (dag_node_t *node);
    If properties do not exist on NODE, *PROPLIST_P will be set to NULL.
 
    The returned property list is allocated in POOL.  */
-svn_error_t *svn_fs_base__dag_get_proplist (apr_hash_t **proplist_p,
-                                            dag_node_t *node,
-                                            trail_t *trail,
-                                            apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_proplist(apr_hash_t **proplist_p,
+                                           dag_node_t *node,
+                                           trail_t *trail,
+                                           apr_pool_t *pool);
 
 /* Set the property list of NODE to PROPLIST, as part of TRAIL.  The
    node being changed must be mutable.  TXN_ID is the Subversion
    transaction under which this occurs.  */
-svn_error_t *svn_fs_base__dag_set_proplist (dag_node_t *node,
-                                            apr_hash_t *proplist,
-                                            const char *txn_id,
-                                            trail_t *trail,
-                                            apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_set_proplist(dag_node_t *node,
+                                           apr_hash_t *proplist,
+                                           const char *txn_id,
+                                           trail_t *trail,
+                                           apr_pool_t *pool);
 
 
+
+/* Mergeinfo tracking stuff. */
+
+/* If HAS_MERGEINFO is not null, set *HAS_MERGEINFO to TRUE iff NODE
+   records that its property list contains merge tracking information.
+
+   If COUNT is not null, set *COUNT to the number of nodes --
+   including NODE itself -- in the subtree rooted at NODE which claim
+   to carry merge tracking information.
+
+   Do this as part of TRAIL, and use POOL for necessary allocations.
+
+   NOTE:  No validation against NODE's actual property list is
+   performed. */
+svn_error_t *svn_fs_base__dag_get_mergeinfo_stats(svn_boolean_t *has_mergeinfo,
+                                                  apr_int64_t *count,
+                                                  dag_node_t *node,
+                                                  trail_t *trail,
+                                                  apr_pool_t *pool);
+
+/* If HAS_MERGEINFO is set, record on NODE that its property list
+   carries merge tracking information.  Otherwise, record on NODE its
+   property list does *not* carry merge tracking information.  NODE
+   must be mutable under TXN_ID (the Subversion transaction under
+   which this operation occurs).  Set *HAD_MERGEINFO to the previous
+   state of this record.
+
+   Update the mergeinfo count on NODE as necessary.
+
+   Do all of this as part of TRAIL, and use POOL for necessary
+   allocations.
+
+   NOTE:  No validation against NODE's actual property list is
+   performed. */
+svn_error_t *svn_fs_base__dag_set_has_mergeinfo(dag_node_t *node,
+                                                svn_boolean_t has_mergeinfo,
+                                                svn_boolean_t *had_mergeinfo,
+                                                const char *txn_id,
+                                                trail_t *trail,
+                                                apr_pool_t *pool);
+
+/* Record on NODE a change of COUNT_DELTA nodes -- including NODE
+   itself -- in the subtree rooted at NODE claim to carry merge
+   tracking information.  That is, add COUNT_DELTA to NODE's current
+   mergeinfo count (regardless of whether COUNT_DELTA is a positive or
+   negative integer).
+
+   NODE must be mutable under TXN_ID (the Subversion transaction under
+   which this operation occurs).  Do this as part of TRAIL, and use
+   POOL for necessary allocations.
+
+   NOTE:  No validation of these claims is performed. */
+svn_error_t *svn_fs_base__dag_adjust_mergeinfo_count(dag_node_t *node,
+                                                     apr_int64_t count_delta,
+                                                     const char *txn_id,
+                                                     trail_t *trail,
+                                                     apr_pool_t *pool);
+
 
 /* Revision and transaction roots.  */
 
 
 /* Open the root of revision REV of filesystem FS, as part of TRAIL.
    Set *NODE_P to the new node.  Allocate the node in POOL.  */
-svn_error_t *svn_fs_base__dag_revision_root (dag_node_t **node_p,
-                                             svn_fs_t *fs,
-                                             svn_revnum_t rev,
-                                             trail_t *trail,
-                                             apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_revision_root(dag_node_t **node_p,
+                                            svn_fs_t *fs,
+                                            svn_revnum_t rev,
+                                            trail_t *trail,
+                                            apr_pool_t *pool);
 
 
 /* Set *NODE_P to the root of transaction TXN_ID in FS, as part
@@ -171,20 +229,20 @@ svn_error_t *svn_fs_base__dag_revision_root (dag_node_t **node_p,
    changes have been made in the transaction, then it may share its
    root directory with its base revision.  To get a mutable root node
    for a transaction, call svn_fs_base__dag_clone_root.  */
-svn_error_t *svn_fs_base__dag_txn_root (dag_node_t **node_p,
-                                        svn_fs_t *fs,
-                                        const char *txn_id,
-                                        trail_t *trail,
-                                        apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_txn_root(dag_node_t **node_p,
+                                       svn_fs_t *fs,
+                                       const char *txn_id,
+                                       trail_t *trail,
+                                       apr_pool_t *pool);
 
 
 /* Set *NODE_P to the base root of transaction TXN_ID in FS, as part
    of TRAIL.  Allocate the node in POOL.  */
-svn_error_t *svn_fs_base__dag_txn_base_root (dag_node_t **node_p,
-                                             svn_fs_t *fs,
-                                             const char *txn_id,
-                                             trail_t *trail,
-                                             apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_txn_base_root(dag_node_t **node_p,
+                                            svn_fs_t *fs,
+                                            const char *txn_id,
+                                            trail_t *trail,
+                                            apr_pool_t *pool);
 
 
 /* Clone the root directory of TXN_ID in FS, and update the
@@ -192,32 +250,31 @@ svn_error_t *svn_fs_base__dag_txn_base_root (dag_node_t **node_p,
    done already.  In either case, set *ROOT_P to a reference to the
    root directory clone.  Do all this as part of TRAIL, and allocate
    *ROOT_P in POOL.  */
-svn_error_t *svn_fs_base__dag_clone_root (dag_node_t **root_p,
-                                          svn_fs_t *fs,
-                                          const char *txn_id,
-                                          trail_t *trail,
-                                          apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_clone_root(dag_node_t **root_p,
+                                         svn_fs_t *fs,
+                                         const char *txn_id,
+                                         trail_t *trail,
+                                         apr_pool_t *pool);
 
 
-/* Commit the transaction TXN_ID in FS, as part of TRAIL.  Store the
+/* Commit the transaction TXN->id in TXN->FS, as part of TRAIL.  Store the
    new revision number in *NEW_REV.  This entails:
-   - marking the tree of mutable nodes at TXN_ID's root as immutable,
+   - marking the tree of mutable nodes at TXN->id's root as immutable,
      and marking all their contents as stable
-   - creating a new revision, with TXN_ID's root as its root directory
-   - promoting TXN_ID to a "committed" transaction.
+   - creating a new revision, with TXN->id's root as its root directory
+   - promoting TXN->id to a "committed" transaction.
 
-   Beware!  This does not make sure that TXN_ID is based on the very
-   latest revision in FS.  If the caller doesn't take care of this,
+   Beware!  This does not make sure that TXN->id is based on the very
+   latest revision in TXN->FS.  If the caller doesn't take care of this,
    you may lose people's work!
 
    Do any necessary temporary allocation in a subpool of POOL.
    Consume temporary space at most proportional to the maximum depth
    of SVN_TXN's tree of mutable nodes.  */
-svn_error_t *svn_fs_base__dag_commit_txn (svn_revnum_t *new_rev,
-                                          svn_fs_t *fs,
-                                          const char *txn_id,
-                                          trail_t *trail,
-                                          apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_commit_txn(svn_revnum_t *new_rev,
+                                         svn_fs_txn_t *txn,
+                                         trail_t *trail,
+                                         apr_pool_t *pool);
 
 
 
@@ -228,11 +285,11 @@ svn_error_t *svn_fs_base__dag_commit_txn (svn_revnum_t *new_rev,
    Set *CHILD_P to the new node, allocated in POOL.  NAME must be a
    single path component; it cannot be a slash-separated directory
    path.  */
-svn_error_t *svn_fs_base__dag_open (dag_node_t **child_p,
-                                    dag_node_t *parent,
-                                    const char *name,
-                                    trail_t *trail,
-                                    apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_open(dag_node_t **child_p,
+                                   dag_node_t *parent,
+                                   const char *name,
+                                   trail_t *trail,
+                                   apr_pool_t *pool);
 
 
 /* Set *ENTRIES_P to a hash table of NODE's entries, as part of TRAIL,
@@ -245,22 +302,22 @@ svn_error_t *svn_fs_base__dag_open (dag_node_t **child_p,
    svn_node_unknown by this function -- callers that need in
    interesting value in these slots should fill them in using a new
    TRAIL, since the list of entries can be arbitrarily large.  */
-svn_error_t *svn_fs_base__dag_dir_entries (apr_hash_t **entries_p,
-                                           dag_node_t *node,
-                                           trail_t *trail,
-                                           apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_dir_entries(apr_hash_t **entries_p,
+                                          dag_node_t *node,
+                                          trail_t *trail,
+                                          apr_pool_t *pool);
 
 
 /* Set ENTRY_NAME in NODE to point to ID, as part of TRAIL.  NODE must
    be a mutable directory.  ID can refer to a mutable or immutable
    node.  If ENTRY_NAME does not exist, it will be created.  TXN_ID is
    the Subversion transaction under which this occurs.*/
-svn_error_t *svn_fs_base__dag_set_entry (dag_node_t *node,
-                                         const char *entry_name,
-                                         const svn_fs_id_t *id,
-                                         const char *txn_id,
-                                         trail_t *trail,
-                                         apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_set_entry(dag_node_t *node,
+                                        const char *entry_name,
+                                        const svn_fs_id_t *id,
+                                        const char *txn_id,
+                                        trail_t *trail,
+                                        apr_pool_t *pool);
 
 
 /* Make a new mutable clone of the node named NAME in PARENT, and
@@ -280,14 +337,14 @@ svn_error_t *svn_fs_base__dag_set_entry (dag_node_t *node,
    created.
 
    TXN_ID is the Subversion transaction under which this occurs.  */
-svn_error_t *svn_fs_base__dag_clone_child (dag_node_t **child_p,
-                                           dag_node_t *parent,
-                                           const char *parent_path,
-                                           const char *name,
-                                           const char *copy_id,
-                                           const char *txn_id,
-                                           trail_t *trail,
-                                           apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_clone_child(dag_node_t **child_p,
+                                          dag_node_t *parent,
+                                          const char *parent_path,
+                                          const char *name,
+                                          const char *copy_id,
+                                          const char *txn_id,
+                                          trail_t *trail,
+                                          apr_pool_t *pool);
 
 
 /* Delete the directory entry named NAME from PARENT, as part of
@@ -299,11 +356,11 @@ svn_error_t *svn_fs_base__dag_clone_child (dag_node_t **child_p,
 
    If return SVN_ERR_FS_NO_SUCH_ENTRY, then there is no entry NAME in
    PARENT.  */
-svn_error_t *svn_fs_base__dag_delete (dag_node_t *parent,
-                                      const char *name,
-                                      const char *txn_id,
-                                      trail_t *trail,
-                                      apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_delete(dag_node_t *parent,
+                                     const char *name,
+                                     const char *txn_id,
+                                     trail_t *trail,
+                                     apr_pool_t *pool);
 
 
 /* Delete the node revision assigned to node ID from FS's `nodes'
@@ -316,11 +373,11 @@ svn_error_t *svn_fs_base__dag_delete (dag_node_t *parent,
    children, you risk orphaning those children by leaving them
    dangling, disconnected from all DAG trees.  It is assumed that
    callers of this interface know what in the world they are doing.  */
-svn_error_t *svn_fs_base__dag_remove_node (svn_fs_t *fs,
-                                           const svn_fs_id_t *id,
-                                           const char *txn_id,
-                                           trail_t *trail,
-                                           apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_remove_node(svn_fs_t *fs,
+                                          const svn_fs_id_t *id,
+                                          const char *txn_id,
+                                          trail_t *trail,
+                                          apr_pool_t *pool);
 
 
 /* Delete all mutable node revisions reachable from node ID, including
@@ -329,11 +386,11 @@ svn_error_t *svn_fs_base__dag_remove_node (svn_fs_t *fs,
    revision.  ID may refer to a file or directory, which may be
    mutable or immutable.  TXN_ID is the Subversion transaction under
    which this occurs.  */
-svn_error_t *svn_fs_base__dag_delete_if_mutable (svn_fs_t *fs,
-                                                 const svn_fs_id_t *id,
-                                                 const char *txn_id,
-                                                 trail_t *trail,
-                                                 apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_delete_if_mutable(svn_fs_t *fs,
+                                                const svn_fs_id_t *id,
+                                                const char *txn_id,
+                                                trail_t *trail,
+                                                apr_pool_t *pool);
 
 
 /* Create a new mutable directory named NAME in PARENT, as part of
@@ -345,13 +402,13 @@ svn_error_t *svn_fs_base__dag_delete_if_mutable (svn_fs_t *fs,
    must not currently have an entry named NAME.  Do any temporary
    allocation in POOL.  TXN_ID is the Subversion transaction
    under which this occurs.  */
-svn_error_t *svn_fs_base__dag_make_dir (dag_node_t **child_p,
-                                        dag_node_t *parent,
-                                        const char *parent_path,
-                                        const char *name,
-                                        const char *txn_id,
-                                        trail_t *trail,
-                                        apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_make_dir(dag_node_t **child_p,
+                                       dag_node_t *parent,
+                                       const char *parent_path,
+                                       const char *name,
+                                       const char *txn_id,
+                                       trail_t *trail,
+                                       apr_pool_t *pool);
 
 
 
@@ -361,10 +418,10 @@ svn_error_t *svn_fs_base__dag_make_dir (dag_node_t **child_p,
 /* Set *CONTENTS to a readable generic stream which yields the
    contents of FILE, as part of TRAIL.  Allocate the stream in POOL.
    If FILE is not a file, return SVN_ERR_FS_NOT_FILE.  */
-svn_error_t *svn_fs_base__dag_get_contents (svn_stream_t **contents,
-                                            dag_node_t *file,
-                                            trail_t *trail,
-                                            apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_contents(svn_stream_t **contents,
+                                           dag_node_t *file,
+                                           trail_t *trail,
+                                           apr_pool_t *pool);
 
 
 /* Return a generic writable stream in *CONTENTS with which to set the
@@ -372,11 +429,11 @@ svn_error_t *svn_fs_base__dag_get_contents (svn_stream_t **contents,
    TXN_ID is the Subversion transaction under which this occurs.  Any
    previous edits on the file will be deleted, and a new edit stream
    will be constructed.  */
-svn_error_t *svn_fs_base__dag_get_edit_stream (svn_stream_t **contents,
-                                               dag_node_t *file,
-                                               const char *txn_id,
-                                               trail_t *trail,
-                                               apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_get_edit_stream(svn_stream_t **contents,
+                                              dag_node_t *file,
+                                              const char *txn_id,
+                                              trail_t *trail,
+                                              apr_pool_t *pool);
 
 
 /* Signify the completion of edits to FILE made using the stream
@@ -388,18 +445,18 @@ svn_error_t *svn_fs_base__dag_get_edit_stream (svn_stream_t **contents,
    used), else the error SVN_ERR_CHECKSUM_MISMATCH is returned.
 
    This operation is a no-op if no edits are present.  */
-svn_error_t *svn_fs_base__dag_finalize_edits (dag_node_t *file,
-                                              const char *checksum,
-                                              const char *txn_id,
-                                              trail_t *trail,
-                                              apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_finalize_edits(dag_node_t *file,
+                                             const char *checksum,
+                                             const char *txn_id,
+                                             trail_t *trail,
+                                             apr_pool_t *pool);
 
 
 /* Set *LENGTH to the length of the contents of FILE, as part of TRAIL. */
-svn_error_t *svn_fs_base__dag_file_length (svn_filesize_t *length,
-                                           dag_node_t *file,
-                                           trail_t *trail,
-                                           apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_file_length(svn_filesize_t *length,
+                                          dag_node_t *file,
+                                          trail_t *trail,
+                                          apr_pool_t *pool);
 
 /* Put the recorded MD5 checksum of FILE into DIGEST, as part of
  * TRAIL.  DIGEST must point to APR_MD5_DIGESTSIZE bytes of storage.
@@ -408,10 +465,10 @@ svn_error_t *svn_fs_base__dag_file_length (svn_filesize_t *length,
  * just put all 0's into DIGEST.
  */
 svn_error_t *
-svn_fs_base__dag_file_checksum (unsigned char digest[],
-                                dag_node_t *file,
-                                trail_t *trail,
-                                apr_pool_t *pool);
+svn_fs_base__dag_file_checksum(unsigned char digest[],
+                               dag_node_t *file,
+                               trail_t *trail,
+                               apr_pool_t *pool);
 
 /* Create a new mutable file named NAME in PARENT, as part of TRAIL.
    Set *CHILD_P to a reference to the new node, allocated in
@@ -421,13 +478,13 @@ svn_fs_base__dag_file_checksum (unsigned char digest[],
    PARENT_PATH must be the canonicalized absolute path of the parent
    directory.  TXN_ID is the Subversion transaction under which this
    occurs.  */
-svn_error_t *svn_fs_base__dag_make_file (dag_node_t **child_p,
-                                         dag_node_t *parent,
-                                         const char *parent_path,
-                                         const char *name,
-                                         const char *txn_id,
-                                         trail_t *trail,
-                                         apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_make_file(dag_node_t **child_p,
+                                        dag_node_t *parent,
+                                        const char *parent_path,
+                                        const char *name,
+                                        const char *txn_id,
+                                        trail_t *trail,
+                                        apr_pool_t *pool);
 
 
 
@@ -443,15 +500,15 @@ svn_error_t *svn_fs_base__dag_make_file (dag_node_t **child_p,
    checked.
 
    If PRESERVE_HISTORY is false, FROM_PATH and FROM_REV are ignored.  */
-svn_error_t *svn_fs_base__dag_copy (dag_node_t *to_node,
-                                    const char *entry,
-                                    dag_node_t *from_node,
-                                    svn_boolean_t preserve_history,
-                                    svn_revnum_t from_rev,
-                                    const char *from_path,
-                                    const char *txn_id,
-                                    trail_t *trail,
-                                    apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_copy(dag_node_t *to_node,
+                                   const char *entry,
+                                   dag_node_t *from_node,
+                                   svn_boolean_t preserve_history,
+                                   svn_revnum_t from_rev,
+                                   const char *from_path,
+                                   const char *txn_id,
+                                   trail_t *trail,
+                                   apr_pool_t *pool);
 
 
 
@@ -466,11 +523,11 @@ svn_error_t *svn_fs_base__dag_copy (dag_node_t *to_node,
    node.  Things will go *very* sour if you deltify TARGET against a
    node that might just disappear from the filesystem in the (near)
    future.  */
-svn_error_t *svn_fs_base__dag_deltify (dag_node_t *target,
-                                       dag_node_t *source,
-                                       svn_boolean_t props_only,
-                                       trail_t *trail,
-                                       apr_pool_t *pool);
+svn_error_t *svn_fs_base__dag_deltify(dag_node_t *target,
+                                      dag_node_t *source,
+                                      svn_boolean_t props_only,
+                                      trail_t *trail,
+                                      apr_pool_t *pool);
 
 
 /* Comparison */
@@ -491,12 +548,12 @@ svn_error_t *svn_fs_base__dag_deltify (dag_node_t *target,
    may leave us with a slight chance of a false positive, though I
    don't really see how that would happen in practice.  Nevertheless,
    it should probably be fixed.  */
-svn_error_t *svn_fs_base__things_different (svn_boolean_t *props_changed,
-                                            svn_boolean_t *contents_changed,
-                                            dag_node_t *node1,
-                                            dag_node_t *node2,
-                                            trail_t *trail,
-                                            apr_pool_t *pool);
+svn_error_t *svn_fs_base__things_different(svn_boolean_t *props_changed,
+                                           svn_boolean_t *contents_changed,
+                                           dag_node_t *node1,
+                                           dag_node_t *node2,
+                                           trail_t *trail,
+                                           apr_pool_t *pool);
 
 
 #ifdef __cplusplus
