@@ -2,7 +2,7 @@
  * util.c : serf utility routines for ra_serf
  *
  * ====================================================================
- * Copyright (c) 2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2006-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,14 +19,17 @@
 
 
 #define APR_WANT_STRFUNC
+#include <apr.h>
 #include <apr_want.h>
-#include <apr_base64.h>
+#include <apr_fnmatch.h>
 
 #include <serf.h>
 #include <serf_bucket_types.h>
 
 #include "svn_path.h"
 #include "svn_private_config.h"
+#include "svn_xml.h"
+#include "private/svn_dep_compat.h"
 
 #include "ra_serf.h"
 
@@ -39,6 +42,174 @@
 #define XML_STATUS_ERROR 0
 #endif
 
+#if SERF_VERSION_AT_LEAST(0,1,3)
+static const apr_uint32_t serf_failure_map[][2] =
+{
+  { SERF_SSL_CERT_NOTYETVALID,   SVN_AUTH_SSL_NOTYETVALID },
+  { SERF_SSL_CERT_EXPIRED,       SVN_AUTH_SSL_EXPIRED },
+  { SERF_SSL_CERT_SELF_SIGNED,   SVN_AUTH_SSL_UNKNOWNCA },
+  { SERF_SSL_CERT_UNKNOWNCA,     SVN_AUTH_SSL_UNKNOWNCA }
+};
+
+/* Convert serf's SSL failure mask to our own failure mask. */
+static apr_uint32_t
+ssl_convert_serf_failures(int failures)
+{
+  apr_uint32_t svn_failures = 0;
+  apr_size_t i;
+
+  for (i = 0; i < sizeof(serf_failure_map) / (2 * sizeof(apr_uint32_t)); ++i)
+    {
+      if (failures & serf_failure_map[i][0])
+        {
+          svn_failures |= serf_failure_map[i][1];
+          failures &= ~serf_failure_map[i][0];
+        }
+    }
+
+  /* Map any remaining failure bits to our OTHER bit. */
+  if (failures)
+    {
+      svn_failures |= SVN_AUTH_SSL_OTHER;
+    }
+
+  return svn_failures;
+}
+
+static char *
+convert_organisation_to_str(apr_hash_t *org, apr_pool_t *pool)
+{
+  char *str;
+
+  str = apr_psprintf(pool, "%s, %s, %s, %s, %s (%s)",
+                     (char*)apr_hash_get(org, "OU", APR_HASH_KEY_STRING),
+                     (char*)apr_hash_get(org, "O", APR_HASH_KEY_STRING),
+                     (char*)apr_hash_get(org, "L", APR_HASH_KEY_STRING),
+                     (char*)apr_hash_get(org, "ST", APR_HASH_KEY_STRING),
+                     (char*)apr_hash_get(org, "C", APR_HASH_KEY_STRING),
+                     (char*)apr_hash_get(org, "E", APR_HASH_KEY_STRING));
+
+  return str;
+}
+
+static apr_status_t
+ssl_server_cert(void *baton, int failures, 
+                const serf_ssl_certificate_t *cert)
+{
+  svn_ra_serf__connection_t *conn = baton;
+  apr_pool_t *subpool;
+  svn_auth_ssl_server_cert_info_t cert_info;
+  svn_auth_cred_ssl_server_trust_t *server_creds = NULL;
+  svn_auth_iterstate_t *state;
+  const char *realmstring;
+  apr_uint32_t svn_failures;
+  svn_error_t *err;
+  apr_hash_t *issuer, *subject, *serf_cert;
+  void *creds;
+
+  apr_pool_create(&subpool, conn->session->pool);
+
+  /* Construct the realmstring, e.g. https://svn.collab.net:443 */
+  realmstring = apr_uri_unparse(subpool,
+                                &conn->session->repos_url, 
+                                APR_URI_UNP_OMITPATHINFO);
+
+  /* Extract the info from the certificate */
+  subject = serf_ssl_cert_subject(cert, subpool);
+  issuer = serf_ssl_cert_issuer(cert, subpool);
+  serf_cert = serf_ssl_cert_certificate(cert, subpool);
+
+  cert_info.hostname = apr_hash_get(subject, "CN", APR_HASH_KEY_STRING);
+  cert_info.fingerprint = apr_hash_get(serf_cert, "sha1", APR_HASH_KEY_STRING);
+  if (! cert_info.fingerprint)
+    cert_info.fingerprint = apr_pstrdup(subpool, "<unknown>");
+  cert_info.valid_from = apr_hash_get(serf_cert, "notBefore",
+                         APR_HASH_KEY_STRING);
+  if (! cert_info.valid_from)
+    cert_info.valid_from = apr_pstrdup(subpool, "[invalid date]");
+  cert_info.valid_until = apr_hash_get(serf_cert, "notAfter",
+                          APR_HASH_KEY_STRING);
+  if (! cert_info.valid_until)
+    cert_info.valid_until = apr_pstrdup(subpool, "[invalid date]");
+  cert_info.issuer_dname = convert_organisation_to_str(issuer, subpool);
+  cert_info.ascii_cert = "ce"; //ascii_cert;
+
+  svn_failures = ssl_convert_serf_failures(failures);
+
+  /* Match server certificate CN with the hostname of the server */
+  if (cert_info.hostname)
+    {
+      if (apr_fnmatch(cert_info.hostname, conn->hostinfo,
+                      APR_FNM_PERIOD) == APR_FNM_NOMATCH)
+        {
+          svn_failures |= SVN_AUTH_SSL_CNMISMATCH;
+        }
+    }
+
+  svn_auth_set_parameter(conn->session->wc_callbacks->auth_baton,
+                         SVN_AUTH_PARAM_SSL_SERVER_FAILURES,
+                         &svn_failures);
+
+  svn_auth_set_parameter(conn->session->wc_callbacks->auth_baton,
+                         SVN_AUTH_PARAM_SSL_SERVER_CERT_INFO,
+                         &cert_info);
+
+  err = svn_auth_first_credentials(&creds, &state,
+                                   SVN_AUTH_CRED_SSL_SERVER_TRUST,
+                                   realmstring,
+                                   conn->session->wc_callbacks->auth_baton,
+                                   subpool);
+  if (err || ! creds)
+    {
+      svn_error_clear(err);
+    }
+  else
+    {
+      server_creds = creds;
+      err = svn_auth_save_credentials(state, subpool);
+      if (err)
+        {
+          /* It would be nice to show the error to the user somehow... */
+          svn_error_clear(err);
+        }
+    }
+
+  svn_auth_set_parameter(conn->session->wc_callbacks->auth_baton,
+                         SVN_AUTH_PARAM_SSL_SERVER_CERT_INFO, NULL);
+
+  svn_pool_destroy(subpool);
+
+  return server_creds ? APR_SUCCESS : SVN_ERR_RA_SERF_SSL_CERT_UNTRUSTED;
+}
+
+static svn_error_t *
+load_authorities(svn_ra_serf__connection_t *conn, const char *authorities,
+                 apr_pool_t *pool)
+{
+  char *files, *file, *last;
+  files = apr_pstrdup(pool, authorities);
+
+  while ((file = apr_strtok(files, ";", &last)) != NULL)
+    {
+      serf_ssl_certificate_t *ca_cert;
+      apr_status_t status = serf_ssl_load_cert_file(&ca_cert, file, pool);
+      if (status == APR_SUCCESS)
+        status = serf_ssl_trust_cert(conn->ssl_context, ca_cert);
+
+      if (status != APR_SUCCESS)
+        {
+          return svn_error_createf
+            (SVN_ERR_RA_DAV_INVALID_CONFIG_VALUE, NULL,
+             _("Invalid config: unable to load certificate file '%s'"),
+             svn_path_local_style(file, pool));
+        }
+      files = NULL;
+    }
+
+  return SVN_NO_ERROR;
+}
+#endif
+
 serf_bucket_t *
 svn_ra_serf__conn_setup(apr_socket_t *sock,
                         void *baton,
@@ -47,7 +218,13 @@ svn_ra_serf__conn_setup(apr_socket_t *sock,
   serf_bucket_t *bucket;
   svn_ra_serf__connection_t *conn = baton;
 
+#if SERF_VERSION_AT_LEAST(0,1,3)
+  bucket = serf_context_bucket_socket_create(conn->session->context, 
+                                             sock, conn->bkt_alloc);
+#else
   bucket = serf_bucket_socket_create(sock, conn->bkt_alloc);
+#endif
+
   if (conn->using_ssl)
     {
       bucket = serf_bucket_ssl_decrypt_create(bucket, conn->ssl_context,
@@ -55,6 +232,37 @@ svn_ra_serf__conn_setup(apr_socket_t *sock,
       if (!conn->ssl_context)
         {
           conn->ssl_context = serf_bucket_ssl_decrypt_context_get(bucket);
+#if SERF_VERSION_AT_LEAST(0,1,1)
+          serf_ssl_client_cert_provider_set(conn->ssl_context,
+                                            svn_ra_serf__handle_client_cert,
+                                            conn, conn->session->pool);
+          serf_ssl_client_cert_password_set(conn->ssl_context,
+                                            svn_ra_serf__handle_client_cert_pw,
+                                            conn, conn->session->pool);
+#endif
+#if SERF_VERSION_AT_LEAST(0,1,3)
+          serf_ssl_server_cert_callback_set(conn->ssl_context,
+                                            ssl_server_cert,
+                                            conn);
+          /* See if the user wants us to trust "default" openssl CAs. */
+          if (conn->session->trust_default_ca)
+            {
+              serf_ssl_use_default_certificates(conn->ssl_context);
+            }
+          /* Are there custom CAs to load? */
+          if (conn->session->ssl_authorities)
+            {
+              svn_error_t *err;
+              err = load_authorities(conn, conn->session->ssl_authorities,
+                                     conn->session->pool);
+              if (err) 
+                {
+                  /* TODO: we need a way to pass this error back to the 
+                     caller */
+                  svn_error_clear(err);
+                }
+            }
+#endif
         }
     }
 
@@ -110,6 +318,13 @@ svn_ra_serf__conn_closed(serf_connection_t *conn,
     {
       our_conn->ssl_context = NULL;
     }
+  /* Restart the authentication phase on this new connection. */
+  if (our_conn->session->auth_protocol)
+    {
+      our_conn->session->auth_protocol->init_conn_func(our_conn->session,
+                                                       our_conn,
+                                                       our_conn->session->pool);
+    }
 }
 
 apr_status_t
@@ -138,6 +353,107 @@ svn_ra_serf__cleanup_serf_session(void *data)
   return APR_SUCCESS;
 }
 
+apr_status_t svn_ra_serf__handle_client_cert(void *data,
+                                             const char **cert_path)
+{
+    svn_ra_serf__connection_t *conn = data;
+    svn_ra_serf__session_t *session = conn->session;
+    const char *realm;
+    apr_port_t port;
+    svn_error_t *err;
+    void *creds;
+
+    *cert_path = NULL;
+
+    if (session->repos_url.port_str)
+      {
+        port = session->repos_url.port;
+      }
+    else
+      {
+        port = apr_uri_port_of_scheme(session->repos_url.scheme);
+      }
+
+    realm = apr_psprintf(session->pool, "%s://%s:%d",
+                         session->repos_url.scheme,
+                         session->repos_url.hostname,
+                         port);
+
+    if (!conn->ssl_client_auth_state)
+      {
+        err = svn_auth_first_credentials(&creds,
+                                         &conn->ssl_client_auth_state,
+                                         SVN_AUTH_CRED_SSL_CLIENT_CERT,
+                                         realm,
+                                         session->wc_callbacks->auth_baton,
+                                         session->pool);
+      }
+    else
+      {
+        err = svn_auth_next_credentials(&creds,
+                                        conn->ssl_client_auth_state,
+                                        session->pool);
+      }
+
+    if (err)
+      {
+        session->pending_error = err;
+        return err->apr_err;
+      }
+
+    if (creds)
+      {
+        svn_auth_cred_ssl_client_cert_t *client_creds;
+        client_creds = creds;
+        *cert_path = client_creds->cert_file;
+      }
+
+    return APR_SUCCESS;
+}
+
+apr_status_t svn_ra_serf__handle_client_cert_pw(void *data,
+                                                const char *cert_path,
+                                                const char **password)
+{
+    svn_ra_serf__connection_t *conn = data;
+    svn_ra_serf__session_t *session = conn->session;
+    svn_error_t *err;
+    void *creds;
+
+    *password = NULL;
+
+    if (!conn->ssl_client_pw_auth_state)
+      {
+        err = svn_auth_first_credentials(&creds,
+                                         &conn->ssl_client_pw_auth_state,
+                                         SVN_AUTH_CRED_SSL_CLIENT_CERT_PW,
+                                         cert_path,
+                                         session->wc_callbacks->auth_baton,
+                                         session->pool);
+      }
+    else
+      {
+        err = svn_auth_next_credentials(&creds,
+                                        conn->ssl_client_pw_auth_state,
+                                        session->pool);
+      }
+
+    if (err)
+      {
+        session->pending_error = err;
+        return err->apr_err;
+      }
+
+    if (creds)
+      {
+        svn_auth_cred_ssl_client_cert_pw_t *pw_creds;
+        pw_creds = creds;
+        *password = pw_creds->password;
+      }
+
+    return APR_SUCCESS;
+}
+
 void
 svn_ra_serf__setup_serf_req(serf_request_t *request,
                             serf_bucket_t **req_bkt,
@@ -153,15 +469,27 @@ svn_ra_serf__setup_serf_req(serf_request_t *request,
 
   hdrs_bkt = serf_bucket_request_get_headers(*req_bkt);
   serf_bucket_headers_setn(hdrs_bkt, "Host", conn->hostinfo);
-  serf_bucket_headers_setn(hdrs_bkt, "User-Agent", "svn/ra_serf");
+  serf_bucket_headers_setn(hdrs_bkt, "User-Agent", conn->useragent);
+
   if (content_type)
     {
       serf_bucket_headers_setn(hdrs_bkt, "Content-Type", content_type);
     }
-  if (conn->auth_header && conn->auth_value)
-    {
-      serf_bucket_headers_setn(hdrs_bkt, conn->auth_header, conn->auth_value);
-    }
+
+  /* These headers need to be sent with every request; see issue #3255
+     ("mod_dav_svn does not pass client capabilities to start-commit
+     hooks") for why. */
+  serf_bucket_headers_set(hdrs_bkt, "DAV", SVN_DAV_NS_DAV_SVN_DEPTH);
+  serf_bucket_headers_set(hdrs_bkt, "DAV", SVN_DAV_NS_DAV_SVN_MERGEINFO);
+  serf_bucket_headers_set(hdrs_bkt, "DAV", SVN_DAV_NS_DAV_SVN_LOG_REVPROPS);
+
+  /* Setup server authorization headers */
+  if (conn->session->auth_protocol)
+    conn->session->auth_protocol->setup_request_func(conn, hdrs_bkt);
+
+  /* Setup proxy authorization headers */
+  if (conn->session->proxy_auth_protocol)
+    conn->session->proxy_auth_protocol->setup_request_func(conn, hdrs_bkt);
 
   /* Set up SSL if we need to */
   if (conn->using_ssl)
@@ -171,9 +499,28 @@ svn_ra_serf__setup_serf_req(serf_request_t *request,
       if (!conn->ssl_context)
         {
           conn->ssl_context = serf_bucket_ssl_encrypt_context_get(*req_bkt);
+#if SERF_VERSION_AT_LEAST(0,1,1)
+          serf_ssl_client_cert_provider_set(conn->ssl_context,
+                                            svn_ra_serf__handle_client_cert,
+                                            conn, conn->session->pool);
+          serf_ssl_client_cert_password_set(conn->ssl_context,
+                                            svn_ra_serf__handle_client_cert_pw,
+                                            conn, conn->session->pool);
+#endif
         }
     }
 
+  /* Set up Proxy settings */
+#if SERF_VERSION_AT_LEAST(0,1,3)
+  if (conn->session->using_proxy)
+    {
+      char *root = apr_uri_unparse(conn->session->pool,
+                                   &conn->session->repos_url, 
+                                   APR_URI_UNP_OMITPATHINFO);
+      serf_bucket_request_set_root(*req_bkt, root);
+    }
+#endif
+
   if (ret_hdrs_bkt)
     {
       *ret_hdrs_bkt = hdrs_bkt;
@@ -193,17 +540,21 @@ svn_ra_serf__context_run_wait(svn_boolean_t *done,
     {
       int i;
 
+      if (sess->wc_callbacks &&
+          sess->wc_callbacks->cancel_func)
+        SVN_ERR((sess->wc_callbacks->cancel_func)(sess->wc_callback_baton));
+
       status = serf_context_run(sess->context, SERF_DURATION_FOREVER, pool);
       if (APR_STATUS_IS_TIMEUP(status))
         {
           continue;
         }
+      if (sess->pending_error)
+        {
+          return sess->pending_error;
+        }
       if (status)
         {
-          if (sess->pending_error)
-            { 
-              return sess->pending_error;
-            }
           return svn_error_wrap_apr(status, "Error running context");
         }
       /* Debugging purposes only! */
@@ -217,6 +568,7 @@ svn_ra_serf__context_run_wait(svn_boolean_t *done,
   return SVN_NO_ERROR;
 }
 
+#if ! SERF_VERSION_AT_LEAST(0, 1, 3)
 apr_status_t
 svn_ra_serf__is_conn_closing(serf_bucket_t *response)
 {
@@ -225,13 +577,14 @@ svn_ra_serf__is_conn_closing(serf_bucket_t *response)
 
   hdrs = serf_bucket_response_get_headers(response);
   val = serf_bucket_headers_get(hdrs, "Connection");
-  if (val && strcasecmp("close", val) == 0)
+  if (val && svn_cstring_casecmp("close", val) == 0)
     {
       return SERF_ERROR_CLOSING;
     }
 
   return APR_EOF;
 }
+#endif
 
 /*
  * Expat callback invoked on a start element tag for an error response.
@@ -244,7 +597,7 @@ start_error(svn_ra_serf__xml_parser_t *parser,
 {
   svn_ra_serf__server_error_t *ctx = userData;
 
-  if (!ctx->in_error && 
+  if (!ctx->in_error &&
       strcmp(name.namespace, "DAV:") == 0 &&
       strcmp(name.name, "error") == 0)
     {
@@ -254,7 +607,7 @@ start_error(svn_ra_serf__xml_parser_t *parser,
     {
       const char *err_code;
 
-      err_code = svn_ra_serf__find_attr(attrs, "errcode");
+      err_code = svn_xml_get_attr_value("errcode", attrs);
       if (err_code)
         {
           ctx->error->apr_err = apr_atoi64(err_code);
@@ -263,7 +616,10 @@ start_error(svn_ra_serf__xml_parser_t *parser,
         {
           ctx->error->apr_err = APR_EGENERAL;
         }
-      ctx->collect_message = TRUE;
+
+      /* Start collecting cdata. */
+      svn_stringbuf_setempty(ctx->cdata);
+      ctx->collect_cdata = TRUE;
     }
 
   return SVN_NO_ERROR;
@@ -287,7 +643,19 @@ end_error(svn_ra_serf__xml_parser_t *parser,
     }
   if (ctx->in_error && strcmp(name.name, "human-readable") == 0)
     {
-      ctx->collect_message = FALSE;
+      /* On the server dav_error_response_tag() will add a leading
+         and trailing newline if DEBUG_CR is defined in mod_dav.h,
+         so remove any such characters here. */
+      apr_size_t len;
+      const char *cd = ctx->cdata->data;
+      if (*cd == '\n')
+        ++cd;
+      len = strlen(cd);
+      if (len > 0 && cd[len-1] == '\n')
+        --len;
+
+      ctx->error->message = apr_pstrmemdup(ctx->error->pool, cd, len);
+      ctx->collect_cdata = FALSE;
     }
 
   return SVN_NO_ERROR;
@@ -306,11 +674,9 @@ cdata_error(svn_ra_serf__xml_parser_t *parser,
 {
   svn_ra_serf__server_error_t *ctx = userData;
 
-  /* Skip blank lines in the human-readable error responses. */
-  if (ctx->collect_message && (len != 1 || data[0] != '\n'))
+  if (ctx->collect_cdata)
     {
-      svn_ra_serf__expand_string(&ctx->error->message, &ctx->message_len,
-                                 data, len, ctx->error->pool);
+      svn_stringbuf_appendbytes(ctx->cdata, data, len);
     }
 
   return SVN_NO_ERROR;
@@ -331,7 +697,7 @@ svn_ra_serf__handle_discard_body(serf_request_t *request,
         {
           serf_bucket_t *hdrs;
           const char *val;
-          
+
           server_err->init = TRUE;
           hdrs = serf_bucket_response_get_headers(response);
           val = serf_bucket_headers_get(hdrs, "Content-Type");
@@ -339,6 +705,8 @@ svn_ra_serf__handle_discard_body(serf_request_t *request,
             {
               server_err->error = svn_error_create(APR_SUCCESS, NULL, NULL);
               server_err->has_xml_response = TRUE;
+              server_err->cdata = svn_stringbuf_create("", pool);
+              server_err->collect_cdata = FALSE;
               server_err->parser.pool = server_err->error->pool;
               server_err->parser.user_data = server_err;
               server_err->parser.start = start_error;
@@ -358,7 +726,7 @@ svn_ra_serf__handle_discard_body(serf_request_t *request,
           status = svn_ra_serf__handle_xml_parser(request, response,
                                                   &server_err->parser, pool);
 
-          if (server_err->done && server_err->error->apr_err == APR_SUCCESS) 
+          if (server_err->done && server_err->error->apr_err == APR_SUCCESS)
             {
               svn_error_clear(server_err->error);
               server_err->error = SVN_NO_ERROR;
@@ -366,7 +734,7 @@ svn_ra_serf__handle_discard_body(serf_request_t *request,
 
           return status;
         }
-      
+
     }
 
   /* Just loop through and discard the body. */
@@ -404,7 +772,7 @@ svn_ra_serf__handle_status_only(serf_request_t *request,
       apr_status_t rv;
 
       rv = serf_bucket_response_status(response, &sl);
-      
+
       ctx->status = sl.code;
       ctx->reason = sl.reason;
 
@@ -414,142 +782,144 @@ svn_ra_serf__handle_status_only(serf_request_t *request,
   return status;
 }
 
-static apr_status_t
-handle_auth(svn_ra_serf__session_t *session,
-            svn_ra_serf__connection_t *conn,
-            serf_request_t *request,
-            serf_bucket_t *response,
-            apr_pool_t *pool)
+/*
+ * Expat callback invoked on a start element tag for a 207 response.
+ */
+static svn_error_t *
+start_207(svn_ra_serf__xml_parser_t *parser,
+          void *userData,
+          svn_ra_serf__dav_props_t name,
+          const char **attrs)
 {
-  void *creds;
-  svn_auth_cred_simple_t *simple_creds;
-  const char *tmp;
-  apr_size_t tmp_len, encoded_len;
-  svn_error_t *error;
-  int i;
+  svn_ra_serf__server_error_t *ctx = userData;
 
-  if (!session->realm)
+  if (!ctx->in_error &&
+      strcmp(name.namespace, "DAV:") == 0 &&
+      strcmp(name.name, "multistatus") == 0)
+    {
+      ctx->in_error = TRUE;
+    }
+  else if (ctx->in_error && strcmp(name.name, "responsedescription") == 0)
+    {
+      /* Start collecting cdata. */
+      svn_stringbuf_setempty(ctx->cdata);
+      ctx->collect_cdata = TRUE;
+    }
+
+  return SVN_NO_ERROR;
+}
+
+/*
+ * Expat callback invoked on an end element tag for a 207 response.
+ */
+static svn_error_t *
+end_207(svn_ra_serf__xml_parser_t *parser,
+        void *userData,
+        svn_ra_serf__dav_props_t name)
+{
+  svn_ra_serf__server_error_t *ctx = userData;
+
+  if (ctx->in_error &&
+      strcmp(name.namespace, "DAV:") == 0 &&
+      strcmp(name.name, "multistatus") == 0)
+    {
+      ctx->in_error = FALSE;
+    }
+  if (ctx->in_error && strcmp(name.name, "responsedescription") == 0)
+    {
+      ctx->collect_cdata = FALSE;
+      ctx->error->message = apr_pstrmemdup(ctx->error->pool, ctx->cdata->data,
+                                           ctx->cdata->len);
+      ctx->error->apr_err = SVN_ERR_RA_DAV_REQUEST_FAILED;
+    }
+
+  return SVN_NO_ERROR;
+}
+
+/*
+ * Expat callback invoked on CDATA elements in a 207 response.
+ *
+ * This callback can be called multiple times.
+ */
+static svn_error_t *
+cdata_207(svn_ra_serf__xml_parser_t *parser,
+          void *userData,
+          const char *data,
+          apr_size_t len)
+{
+  svn_ra_serf__server_error_t *ctx = userData;
+
+  if (ctx->collect_cdata)
+    {
+      svn_stringbuf_appendbytes(ctx->cdata, data, len);
+    }
+
+  return SVN_NO_ERROR;
+}
+
+apr_status_t
+svn_ra_serf__handle_multistatus_only(serf_request_t *request,
+                                     serf_bucket_t *response,
+                                     void *baton,
+                                     apr_pool_t *pool)
+{
+  apr_status_t status;
+  svn_ra_serf__simple_request_context_t *ctx = baton;
+  svn_ra_serf__server_error_t *server_err = &ctx->server_error;
+
+  if (server_err && !server_err->init)
     {
       serf_bucket_t *hdrs;
-      char *cur, *last, *auth_hdr, *realm_name;
-      apr_port_t port;
+      const char *val;
 
+      server_err->init = TRUE;
       hdrs = serf_bucket_response_get_headers(response);
-      auth_hdr = (char*)serf_bucket_headers_get(hdrs, "WWW-Authenticate");
-
-      if (!auth_hdr)
+      val = serf_bucket_headers_get(hdrs, "Content-Type");
+      if (val && strncasecmp(val, "text/xml", sizeof("text/xml") - 1) == 0)
         {
-          abort();
-        }
+          server_err->error = svn_error_create(APR_SUCCESS, NULL, NULL);
+          server_err->has_xml_response = TRUE;
+          server_err->cdata = svn_stringbuf_create("", pool);
+          server_err->collect_cdata = FALSE;
+          server_err->parser.pool = server_err->error->pool;
+          server_err->parser.user_data = server_err;
+          server_err->parser.start = start_207;
+          server_err->parser.end = end_207;
+          server_err->parser.cdata = cdata_207;
+          server_err->parser.done = &ctx->done;
+          server_err->parser.ignore_errors = TRUE;
 
-      cur = apr_strtok(auth_hdr, " ", &last);
-      while (cur)
-        {
-          if (strcmp(cur, "Basic") == 0)
+          status = svn_ra_serf__handle_xml_parser(request, response,
+                                                  &server_err->parser, pool);
+
+          if (ctx->done && server_err->error->apr_err == APR_SUCCESS)
             {
-              char *attr;
-
-              attr = apr_strtok(NULL, "=", &last);
-              if (strcmp(attr, "realm") == 0)
-                {
-                  realm_name = apr_strtok(NULL, "=", &last);
-                  if (realm_name[0] == '\"') 
-                    {
-                      apr_size_t realm_len;
-
-                      realm_len = strlen(realm_name);
-                      if (realm_name[realm_len - 1] == '\"')
-                        {
-                          realm_name[realm_len - 1] = '\0';
-                          realm_name++;
-                        }
-                    }
-                }
-              else
-                {
-                  abort();
-                }
+              svn_error_clear(server_err->error);
+              server_err->error = SVN_NO_ERROR;
             }
-          else
-            {
-              /* Support more authentication mechanisms. */
-              abort();
-            }
-          cur = apr_strtok(NULL, " ", &last);
-        }
-
-      if (!realm_name)
-        {
-          abort();
-        }
-
-      if (session->repos_url.port_str)
-        {
-          port = session->repos_url.port;
         }
       else
         {
-          port = apr_uri_port_of_scheme(session->repos_url.scheme);
+          ctx->done = TRUE;
+          server_err->error = SVN_NO_ERROR;
         }
-
-      session->realm = apr_psprintf(session->pool, "<%s://%s:%d> %s",
-                                    session->repos_url.scheme,
-                                    session->repos_url.hostname,
-                                    port,
-                                    realm_name);
-
-      error = svn_auth_first_credentials(&creds,
-                                         &session->auth_state,
-                                         SVN_AUTH_CRED_SIMPLE,
-                                         session->realm,
-                                         session->wc_callbacks->auth_baton,
-                                         session->pool);
     }
-  else
+
+  status = svn_ra_serf__handle_discard_body(request, response,
+                                            NULL, pool);
+
+  if (APR_STATUS_IS_EOF(status))
     {
-      error = svn_auth_next_credentials(&creds,
-                                        session->auth_state,
-                                        session->pool);
-    }
-  
-  session->auth_attempts++;
+      serf_status_line sl;
+      apr_status_t rv;
 
-  if (error)
-    {
-      abort();
+      rv = serf_bucket_response_status(response, &sl);
+
+      ctx->status = sl.code;
+      ctx->reason = sl.reason;
     }
 
-  if (!creds || session->auth_attempts > 4)
-    {
-      /* No more credentials. */
-      printf("No more credentials or we tried too many times.  Sorry.\n");
-      return APR_EGENERAL;
-    }
-
-  simple_creds = creds;
-
-  tmp = apr_pstrcat(session->pool,
-                    simple_creds->username, ":", simple_creds->password, NULL);
-  tmp_len = strlen(tmp);
-
-  encoded_len = apr_base64_encode_len(tmp_len);
-
-  session->auth_value = apr_palloc(session->pool, encoded_len + 6);
-
-  apr_cpystrn(session->auth_value, "Basic ", 7);
-
-  apr_base64_encode(&session->auth_value[6], tmp, tmp_len);
-
-  session->auth_header = "Authorization";
-
-  /* FIXME Come up with a cleaner way of changing the connection auth. */
-  for (i = 0; i < session->num_conns; i++)
-    {
-      session->conns[i]->auth_header = session->auth_header;
-      session->conns[i]->auth_value = session->auth_value;
-    }
-
-  return APR_SUCCESS;
+  return status;
 }
 
 static void
@@ -637,6 +1007,7 @@ svn_ra_serf__handle_xml_parser(serf_request_t *request,
               *ctx->done_list = ctx->done_item;
             }
         }
+      ctx->error = svn_ra_serf__handle_server_error(request, response, pool);
       return svn_ra_serf__handle_discard_body(request, response, NULL, pool);
     }
 
@@ -688,11 +1059,7 @@ svn_ra_serf__handle_xml_parser(serf_request_t *request,
       if (ctx->error && ctx->ignore_errors == FALSE)
         {
           XML_ParserFree(ctx->xmlp);
-          status = ctx->error->apr_err;
-
-          svn_error_clear(ctx->error);
-
-          return status;
+          return ctx->error->apr_err;
         }
 
       if (APR_STATUS_IS_EAGAIN(status))
@@ -703,13 +1070,12 @@ svn_ra_serf__handle_xml_parser(serf_request_t *request,
       if (APR_STATUS_IS_EOF(status))
         {
           xml_status = XML_Parse(ctx->xmlp, NULL, 0, 1);
+          XML_ParserFree(ctx->xmlp);
           if (xml_status == XML_STATUS_ERROR && ctx->ignore_errors == FALSE)
             {
-              abort();
+              svn_error_clear(ctx->error);
             }
 
-          XML_ParserFree(ctx->xmlp);
-
           *ctx->done = TRUE;
           if (ctx->done_list)
             {
@@ -737,6 +1103,7 @@ svn_ra_serf__handle_server_error(serf_request_t *request,
   status = svn_ra_serf__handle_discard_body(request, response,
                                             &server_err, pool);
 
+#if ! SERF_VERSION_AT_LEAST(0, 1, 3)
   if (APR_STATUS_IS_EOF(status))
     {
       status = svn_ra_serf__is_conn_closing(response);
@@ -745,10 +1112,9 @@ svn_ra_serf__handle_server_error(serf_request_t *request,
           serf_connection_reset(serf_request_get_conn(request));
         }
     }
+#endif
 
-  SVN_ERR(server_err.error);
-
-  return svn_error_create(APR_EGENERAL, NULL, _("Unspecified error message"));
+  return server_err.error;
 }
 
 /* Implements the serf_response_handler_t interface.  Wait for HTTP
@@ -788,6 +1154,11 @@ handle_response(serf_request_t *request,
     {
       return status;
     }
+  if (!sl.version && (APR_STATUS_IS_EOF(status) ||
+                      APR_STATUS_IS_EAGAIN(status)))
+    {
+      return status;
+    }
 
   status = serf_bucket_response_wait_for_headers(response);
   if (status)
@@ -810,7 +1181,7 @@ handle_response(serf_request_t *request,
           ctx->session->pending_error =
               svn_error_create(SVN_ERR_RA_DAV_MALFORMED_DATA, NULL,
                                _("Premature EOF seen from server"));
-          return ctx->session->pending_error->apr_err;
+          return status;
         }
     }
 
@@ -819,21 +1190,50 @@ handle_response(serf_request_t *request,
       svn_auth_save_credentials(ctx->session->auth_state, ctx->session->pool);
       ctx->session->auth_attempts = 0;
       ctx->session->auth_state = NULL;
+      ctx->session->realm = NULL;
     }
 
   ctx->conn->last_status_code = sl.code;
 
-  if (sl.code == 401)
+  if (sl.code == 401 || sl.code == 407)
     {
-      handle_auth(ctx->session, ctx->conn, request, response, pool);
-      svn_ra_serf__request_create(ctx);
-      status = svn_ra_serf__handle_discard_body(request, response, NULL, pool);
+      /* 401 Authorization or 407 Proxy-Authentication required */
+      svn_error_t *err;
+
+      err = svn_ra_serf__handle_auth(sl.code, ctx->session, ctx->conn,
+                                     request, response, pool);
+      if (err)
+        {
+          ctx->session->pending_error = err;
+          svn_ra_serf__handle_discard_body(request, response, NULL, pool);
+          return ctx->session->pending_error->apr_err;
+        }
+      else
+        {
+          status = svn_ra_serf__handle_discard_body(request, response, NULL,
+                                                    pool);
+          /* At this time we might not have received the whole response from
+             the server. If that's the case, don't setup a new request now
+             but wait till we retry the request later. */
+          if (! APR_STATUS_IS_EAGAIN(status))
+            {
+              svn_ra_serf__priority_request_create(ctx);
+            }
+        }
     }
-  else if (sl.code >= 500)
+  else if (sl.code == 409 || sl.code >= 500)
     {
+      /* 409 Conflict: can indicate a hook error.
+         5xx (Internal) Server error. */
       ctx->session->pending_error =
           svn_ra_serf__handle_server_error(request, response, pool);
-      return ctx->session->pending_error->apr_err;
+      if (!ctx->session->pending_error)
+        {
+          ctx->session->pending_error =
+              svn_error_create(APR_EGENERAL, NULL,
+                               _("Unspecified error message"));
+        }
+      return APR_EGENERAL;
     }
   else
     {
@@ -841,10 +1241,12 @@ handle_response(serf_request_t *request,
                                      pool);
     }
 
+#if ! SERF_VERSION_AT_LEAST(0, 1, 3)
   if (APR_STATUS_IS_EOF(status))
     {
       status = svn_ra_serf__is_conn_closing(response);
     }
+#endif
 
   return status;
 }
@@ -893,7 +1295,7 @@ setup_request(serf_request_t *request,
         {
           *acceptor = accept_head;
         }
- 
+
       if (ctx->body_delegate)
         {
           ctx->body_buckets =
@@ -925,6 +1327,21 @@ svn_ra_serf__request_create(svn_ra_serf__handler_t *handler)
                                         setup_request, handler);
 }
 
+serf_request_t *
+svn_ra_serf__priority_request_create(svn_ra_serf__handler_t *handler)
+{
+#if SERF_VERSION_AT_LEAST(0,1,3)
+  return serf_connection_priority_request_create(handler->conn->conn,
+                                                 setup_request, handler);
+#else
+  /* Fall back to the adding the new request at the end of the queue. While
+     this will make certain auth. protocols fail, at least basic authentication
+     will still work. */
+  return serf_connection_request_create(handler->conn->conn,
+                                        setup_request, handler);
+#endif
+}
+
 svn_error_t *
 svn_ra_serf__discover_root(const char **vcc_url,
                            const char **rel_path,
@@ -934,7 +1351,7 @@ svn_ra_serf__discover_root(const char **vcc_url,
                            apr_pool_t *pool)
 {
   apr_hash_t *props;
-  const char *path, *relative_path, *present_path = "";
+  const char *path, *relative_path, *present_path = "", *uuid;
 
   /* If we're only interested in our VCC, just return it. */
   if (session->vcc_url && !rel_path)
@@ -946,39 +1363,58 @@ svn_ra_serf__discover_root(const char **vcc_url,
   props = apr_hash_make(pool);
   path = orig_path;
   *vcc_url = NULL;
+  uuid = NULL;
 
   do
     {
-      SVN_ERR(svn_ra_serf__retrieve_props(props, session, conn,
-                                          path, SVN_INVALID_REVNUM,
-                                          "0", base_props, pool));
-      *vcc_url =
-          svn_ra_serf__get_ver_prop(props, path,
-                                    SVN_INVALID_REVNUM,
-                                    "DAV:",
-                                    "version-controlled-configuration");
-
-      if (*vcc_url)
+      svn_error_t *err = svn_ra_serf__retrieve_props(props, session, conn,
+                                                     path, SVN_INVALID_REVNUM,
+                                                     "0", base_props, pool);
+      if (! err)
         {
+          *vcc_url =
+              svn_ra_serf__get_ver_prop(props, path,
+                                        SVN_INVALID_REVNUM,
+                                        "DAV:",
+                                        "version-controlled-configuration");
+
           relative_path = svn_ra_serf__get_ver_prop(props, path,
                                                     SVN_INVALID_REVNUM,
                                                     SVN_DAV_PROP_NS_DAV,
                                                     "baseline-relative-path");
+
+          uuid = svn_ra_serf__get_ver_prop(props, path,
+                                           SVN_INVALID_REVNUM,
+                                           SVN_DAV_PROP_NS_DAV,
+                                           "repository-uuid");
           break;
         }
+      else
+        {
+          if (err->apr_err != SVN_ERR_FS_NOT_FOUND)
+            {
+              return err;  /* found a _real_ error */
+            }
+          else
+            {
+              /* This happens when the file is missing in HEAD. */
+              svn_error_clear(err);
 
-      /* This happens when the file is missing in HEAD. */
-
-      /* Okay, strip off. */
-      present_path = svn_path_join(svn_path_basename(path, pool),
-                                   present_path, pool);
-      path = svn_path_dirname(path, pool);
+              /* Okay, strip off. */
+              present_path = svn_path_join(svn_path_basename(path, pool),
+                                           present_path, pool);
+              path = svn_path_dirname(path, pool);
+            }
+        }
     }
   while (!svn_path_is_empty(path));
 
   if (!*vcc_url)
     {
-      abort();
+      return svn_error_create(SVN_ERR_RA_DAV_OPTIONS_REQ_FAILED, NULL,
+                              _("The OPTIONS response did not include the "
+                                "requested version-controlled-configuration "
+                                "value"));
     }
 
   /* Store our VCC in our cache. */
@@ -1000,16 +1436,27 @@ svn_ra_serf__discover_root(const char **vcc_url,
       /* Now recreate the root_url. */
       session->repos_root = session->repos_url;
       session->repos_root.path = apr_pstrdup(session->pool, url_buf->data);
-      session->repos_root_str = apr_uri_unparse(session->pool,
-                                                &session->repos_root, 0);
+      session->repos_root_str = 
+        svn_path_canonicalize(apr_uri_unparse(session->pool,
+                                              &session->repos_root, 0),
+                              session->pool);
+    }
+
+  /* Store the repository UUID in the cache. */
+  if (!session->uuid)
+    {
+      session->uuid = apr_pstrdup(session->pool, uuid);
     }
 
   if (rel_path)
     {
       if (present_path[0] != '\0')
         {
-          *rel_path = svn_path_url_add_component(relative_path,
-                                                 present_path, pool);
+          /* The relative path is supposed to be URI decoded, so decode
+             present_path before joining both together. */
+          *rel_path = svn_path_join(relative_path,
+                                    svn_path_uri_decode(present_path, pool),
+                                    pool);
         }
       else
         {
@@ -1019,3 +1466,16 @@ svn_ra_serf__discover_root(const char **vcc_url,
 
   return SVN_NO_ERROR;
 }
+
+svn_error_t *
+svn_ra_serf__error_on_status(int status_code, const char *path)
+{
+  switch(status_code) 
+    {
+      case 404:
+        return svn_error_createf(SVN_ERR_FS_NOT_FOUND, NULL,
+                                 _("'%s' path not found"), path);
+    }
+
+  return SVN_NO_ERROR;
+}
