@@ -1,7 +1,7 @@
 /* hooks.c : running repository hooks
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -39,12 +39,129 @@
 
 /*** Hook drivers. ***/
 
-/* NAME, CMD and ARGS are the name, path to and arguments for the hook
-   program that is to be run.  If READ_ERRSTREAM is TRUE then the hook's
-   exit status will be checked, and if an error occurred the hook's stderr
-   output will be added to the returned error.
+#ifndef AS400
+/* Helper function for run_hook_cmd().  Wait for a hook to finish
+   executing and return either SVN_NO_ERROR if the hook script completed
+   without error, or an error describing the reason for failure.
 
-   If READ_ERRSTREAM is FALSE the hook's stderr output will be discarded.
+   NAME and CMD are the name and path of the hook program, CMD_PROC
+   is a pointer to the structure representing the running process,
+   and READ_ERRHANDLE is an open handle to the hook's stderr.
+
+   Hooks are considered to have failed if we are unable to wait for the
+   process, if we are unable to read from the hook's stderr, if the
+   process has failed to exit cleanly (due to a coredump, for example),
+   or if the process returned a non-zero return code.
+
+   Any error output returned by the hook's stderr will be included in an
+   error message, though the presence of output on stderr is not itself
+   a reason to fail a hook. */
+static svn_error_t *
+check_hook_result(const char *name, const char *cmd, apr_proc_t *cmd_proc,
+                  apr_file_t *read_errhandle, apr_pool_t *pool)
+{
+  svn_error_t *err, *err2;
+  svn_stringbuf_t *native_stderr, *failure_message;
+  const char *utf8_stderr;
+  int exitcode;
+  apr_exit_why_e exitwhy;
+
+  err2 = svn_stringbuf_from_aprfile(&native_stderr, read_errhandle, pool);
+
+  err = svn_io_wait_for_cmd(cmd_proc, cmd, &exitcode, &exitwhy, pool);
+  if (err)
+    {
+      svn_error_clear(err2);
+      return err;
+    }
+
+  if (APR_PROC_CHECK_EXIT(exitwhy) && exitcode == 0)
+    {
+      /* The hook exited cleanly.  However, if we got an error reading
+         the hook's stderr, fail the hook anyway, because this might be
+         symptomatic of a more important problem. */
+      if (err2)
+        {
+          return svn_error_createf
+            (SVN_ERR_REPOS_HOOK_FAILURE, err2,
+             _("'%s' hook succeeded, but error output could not be read"),
+             name);
+        }
+
+      return SVN_NO_ERROR;
+    }
+
+  /* The hook script failed. */
+
+  /* If we got the stderr output okay, try to translate it into UTF-8.
+     Ensure there is something sensible in the UTF-8 string regardless. */
+  if (!err2)
+    {
+      err2 = svn_utf_cstring_to_utf8(&utf8_stderr, native_stderr->data, pool);
+      if (err2)
+        utf8_stderr = _("[Error output could not be translated from the "
+                        "native locale to UTF-8.]");
+    }
+  else
+    {
+      utf8_stderr = _("[Error output could not be read.]");
+    }
+  /*### It would be nice to include the text of any translation or read
+        error in the messages above before we clear it here. */
+  svn_error_clear(err2);
+
+  if (!APR_PROC_CHECK_EXIT(exitwhy))
+    {
+      failure_message = svn_stringbuf_createf(pool,
+        _("'%s' hook failed (did not exit cleanly: "
+          "apr_exit_why_e was %d, exitcode was %d).  "),
+        name, exitwhy, exitcode);
+    }
+  else
+    {
+      const char *action;
+      if (strcmp(name, "start-commit") == 0
+          || strcmp(name, "pre-commit") == 0)
+        action = _("Commit");
+      else if (strcmp(name, "pre-revprop-change") == 0)
+        action = _("Revprop change");
+      else if (strcmp(name, "pre-lock") == 0)
+        action = _("Lock");
+      else if (strcmp(name, "pre-unlock") == 0)
+        action = _("Unlock");
+      else
+        action = NULL;
+      if (action == NULL)
+        failure_message = svn_stringbuf_createf(
+            pool, _("%s hook failed (exit code %d)"),
+            name, exitcode);
+      else
+        failure_message = svn_stringbuf_createf(
+            pool, _("%s blocked by %s hook (exit code %d)"),
+            action, name, exitcode);
+    }
+
+  if (utf8_stderr[0])
+    {
+      svn_stringbuf_appendcstr(failure_message,
+                               _(" with output:\n"));
+      svn_stringbuf_appendcstr(failure_message, utf8_stderr);
+    }
+  else
+    {
+      svn_stringbuf_appendcstr(failure_message,
+                               _(" with no output."));
+    }
+
+  return svn_error_create(SVN_ERR_REPOS_HOOK_FAILURE, err,
+                          failure_message->data);
+}
+#endif /* AS400 */
+
+/* NAME, CMD and ARGS are the name, path to and arguments for the hook
+   program that is to be run.  The hook's exit status will be checked,
+   and if an error occurred the hook's stderr output will be added to
+   the returned error.
 
    If STDIN_HANDLE is non-null, pass it as the hook's stdin, else pass
    no stdin to the hook. */
@@ -52,7 +169,6 @@ static svn_error_t *
 run_hook_cmd(const char *name,
              const char *cmd,
              const char **args,
-             svn_boolean_t read_errstream,
              apr_file_t *stdin_handle,
              apr_pool_t *pool)
 #ifndef AS400
@@ -60,8 +176,6 @@ run_hook_cmd(const char *name,
   apr_file_t *read_errhandle, *write_errhandle, *null_handle;
   apr_status_t apr_err;
   svn_error_t *err;
-  int exitcode;
-  apr_exit_why_e exitwhy;
   apr_proc_t cmd_proc;
 
   /* Create a pipe to access stderr of the child. */
@@ -116,42 +230,7 @@ run_hook_cmd(const char *name,
     }
   else
     {
-      svn_stringbuf_t *native_error;
-      const char *error;
-      svn_error_t *err2;
-
-      err2 = svn_stringbuf_from_aprfile(&native_error, read_errhandle, pool);
-
-      err = svn_io_wait_for_cmd(&cmd_proc, cmd, &exitcode, &exitwhy, pool);
-      if (! err)
-        {
-          if (! APR_PROC_CHECK_EXIT(exitwhy) || exitcode != 0)
-            {
-              if (read_errstream && ! err2)
-                {
-                  err2 = svn_utf_cstring_to_utf8(&error, native_error->data,
-                                                 pool);
-                  if (! err2)
-                    err = svn_error_createf
-                      (SVN_ERR_REPOS_HOOK_FAILURE, err,
-                       _("'%s' hook failed with error output:\n%s"),
-                       name, error);
-                }
-              else
-                {
-                  err = svn_error_createf
-                    (SVN_ERR_REPOS_HOOK_FAILURE, err,
-                     _("'%s' hook failed; no error output available"), name);
-                }
-            }
-        }
-      if (err2)
-        {
-          if (err)
-            svn_error_clear(err2);
-          else
-            err = err2;
-        }
+      err = check_hook_result(name, cmd, &cmd_proc, read_errhandle, pool);
     }
 
   /* Hooks are fallible, and so hook failure is "expected" to occur at
@@ -171,10 +250,9 @@ run_hook_cmd(const char *name,
 #else /* Run hooks with spawn() on OS400. */
 #define AS400_BUFFER_SIZE 256
 {
-  const char *script_stderr_utf8 = "";
   const char **native_args;
   int fd_map[3], stderr_pipe[2], exitcode;
-  svn_stringbuf_t *script_output = svn_stringbuf_create("", pool);
+  svn_stringbuf_t *script_output;
   pid_t child_pid, wait_rv;
   apr_size_t args_arr_size = 0, i;
   struct inheritance xmp_inherit = {0};
@@ -211,7 +289,7 @@ run_hook_cmd(const char *name,
       /* Get OS400 file descriptor of APR stdin file and map it. */
       if (apr_os_file_get(&fd_map[0], stdin_handle))
         {
-          return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+          return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                    "Error converting APR file to OS400 "
                                    "type for hook script '%s'", cmd);
         }
@@ -221,7 +299,7 @@ run_hook_cmd(const char *name,
       fd_map[0] = open(dev_null_ebcdic, O_RDONLY);
       if (fd_map[0] == -1)
 
-        return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+        return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                  "Error opening /dev/null for hook "
                                  "script '%s'", cmd);
     }
@@ -230,56 +308,46 @@ run_hook_cmd(const char *name,
   /* Map stdout. */
   fd_map[1] = open(dev_null_ebcdic, O_WRONLY);
   if (fd_map[1] == -1)
-    return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+    return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                              "Error opening /dev/null for hook script '%s'",
                              cmd);
 
   /* Map stderr. */
-  if (read_errstream)
+  /* Get pipe for hook's stderr. */
+  if (pipe(stderr_pipe) != 0)
     {
-      /* Get pipe for hook's stderr. */
-      if (pipe(stderr_pipe) != 0)
-        {
-          return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
-                                   "Can't create stderr pipe for "
-                                   "hook '%s'", cmd);
-        }
-      fd_map[2] = stderr_pipe[1];
-    }
-  else
-    {
-      /* Just dump stderr to /dev/null if we don't want it. */
-      fd_map[2] = open(dev_null_ebcdic, O_WRONLY);
-      if (fd_map[2] == -1)
-        return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
-                                 "Error opening /dev/null for hook "
-                                 "script '%s'", cmd);
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
+                               "Can't create stderr pipe for "
+                               "hook '%s'", cmd);
     }
+  fd_map[2] = stderr_pipe[1];
 
   /* Spawn the hook command. */
   child_pid = spawn(native_args[0], 3, fd_map, &xmp_inherit, native_args,
                     xmp_envp);
   if (child_pid == -1)
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Error spawning process for hook script '%s'",
                                cmd);
     }
 
   /* Close the stdout file descriptor. */
   if (close(fd_map[1]) == -1)
-    return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+    return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                              "Error closing write end of stdout pipe to "
                              "hook script '%s'", cmd);
 
   /* Close the write end of the stderr pipe so any subsequent reads
-   * don't hang. */  
+   * don't hang. */
   if (close(fd_map[2]) == -1)
-    return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+    return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                              "Error closing write end of stderr pipe to "
                              "hook script '%s'", cmd);
 
-  while (read_errstream)
+  script_output = svn_stringbuf_create("", pool);
+
+  while (1)
     {
       int rc;
 
@@ -292,7 +360,7 @@ run_hook_cmd(const char *name,
 
       if (rc == -1)
         {
-          return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+          return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                    "Error reading stderr of hook "
                                    "script '%s'", cmd);
         }
@@ -303,15 +371,14 @@ run_hook_cmd(const char *name,
        * stderr. */
       if (rc == 0)
         {
-          /* Null terminate the stringbuf. */
           script_output->data[script_output->len] = '\0';
           break;
         }
     }
 
   /* Close the read end of the stderr pipe. */
-  if (read_errstream && close(stderr_pipe[0]) == -1)
-    return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+  if (close(stderr_pipe[0]) == -1)
+    return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                              "Error closing read end of stderr "
                              "pipe to hook script '%s'", cmd);
 
@@ -319,36 +386,52 @@ run_hook_cmd(const char *name,
   wait_rv = waitpid(child_pid, &exitcode, 0);
   if (wait_rv == -1)
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Error waiting for process completion of "
                                "hook script '%s'", cmd);
     }
 
-  if (!svn_stringbuf_isempty(script_output))
-    {
-      /* OS400 scripts produce EBCDIC stderr, so convert it. */
-      SVN_ERR(svn_utf_cstring_to_utf8_ex2(&script_stderr_utf8,
-                                          script_output->data,
-                                          (const char*)0, pool));
-    }
 
   if (WIFEXITED(exitcode))
     {
       if (WEXITSTATUS(exitcode))
         {
-          if (read_errstream)
+          svn_error_t *err;
+          const char *utf8_stderr = NULL;
+          svn_stringbuf_t *failure_message = svn_stringbuf_createf(
+            pool, "'%s' hook failed (exited with a non-zero exitcode "
+            "of %d).  ", name, exitcode);
+
+          if (!svn_stringbuf_isempty(script_output))
             {
-              return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
-                                       "'%s' hook failed with error "
-                                       "output:\n%s", name,
-                                       script_stderr_utf8);
+              /* OS400 scripts produce EBCDIC stderr, so convert it. */
+              err = svn_utf_cstring_to_utf8_ex2(&utf8_stderr,
+                                                script_output->data,
+                                                (const char*)0, pool);
+              if (err)
+                {
+                  utf8_stderr = "[Error output could not be translated from "
+                                "the native locale to UTF-8.]";
+                  svn_error_clear(err);
+
+                }
+            }
+
+          if (utf8_stderr)
+            {
+              svn_stringbuf_appendcstr(failure_message,
+                                       "The following error output was "
+                                       "produced by the hook:\n");
+              svn_stringbuf_appendcstr(failure_message, utf8_stderr);
             }
           else
             {
-              return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
-                                       "'%s' hook failed; no error output "
-                                       "available", name);
+              svn_stringbuf_appendcstr(failure_message,
+                                       "No error output was produced by "
+                                       "the hook.");
             }
+          return svn_error_create(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
+                                  failure_message->data);
         }
       else
         /* Success! */
@@ -356,26 +439,26 @@ run_hook_cmd(const char *name,
     }
   else if (WIFSIGNALED(exitcode))
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Process '%s' failed because of an "
                                "uncaught terminating signal", cmd);
     }
   else if (WIFEXCEPTION(exitcode))
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Process '%s' failed unexpectedly with "
                                "OS400 exception %d", cmd,
                                WEXCEPTNUMBER(exitcode));
     }
   else if (WIFSTOPPED(exitcode))
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Process '%s' stopped unexpectedly by "
                                "signal %d", cmd, WSTOPSIG(exitcode));
     }
   else
     {
-      return svn_error_createf(SVN_ERR_EXTERNAL_PROGRAM, NULL,
+      return svn_error_createf(SVN_ERR_REPOS_HOOK_FAILURE, NULL,
                                "Process '%s' failed unexpectedly", cmd);
     }
 }
@@ -402,7 +485,7 @@ create_temp_file(apr_file_t **f, const svn_string_t *value, apr_pool_t *pool)
 
 
 /* Check if the HOOK program exists and is a file or a symbolic link, using
-   POOL for temporary allocations. 
+   POOL for temporary allocations.
 
    If the hook exists but is a broken symbolic link, set *BROKEN_LINK
    to TRUE, else if the hook program exists set *BROKEN_LINK to FALSE.
@@ -434,7 +517,7 @@ check_hook_cmd(const char *hook, svn_boolean_t *broken_link, apr_pool_t *pool)
     {
       const char *const hook_path =
         (**extn ? apr_pstrcat(pool, hook, *extn, 0) : hook);
-      
+
       svn_node_kind_t kind;
       if (!(err = svn_io_check_resolved_path(hook_path, &kind, pool))
           && kind == svn_node_file)
@@ -468,25 +551,42 @@ hook_symlink_error(const char *hook)
 svn_error_t *
 svn_repos__hooks_start_commit(svn_repos_t *repos,
                               const char *user,
+                              apr_array_header_t *capabilities,
                               apr_pool_t *pool)
 {
   const char *hook = svn_repos_start_commit_hook(repos, pool);
   svn_boolean_t broken_link;
-  
+
   if ((hook = check_hook_cmd(hook, &broken_link, pool)) && broken_link)
     {
       return hook_symlink_error(hook);
     }
   else if (hook)
     {
-      const char *args[4];
+      const char *args[5];
+      char *capabilities_string;
+
+      if (capabilities)
+        {
+          capabilities_string = svn_cstring_join(capabilities, ":", pool);
+
+          /* Get rid of that annoying final colon. */
+          if (capabilities_string[0])
+            capabilities_string[strlen(capabilities_string) - 1] = '\0';
+        }
+      else
+        {
+          capabilities_string = apr_pstrdup(pool, "");
+        }
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = user ? user : "";
-      args[3] = NULL;
+      args[3] = capabilities_string;
+      args[4] = NULL;
 
-      SVN_ERR(run_hook_cmd("start-commit", hook, args, TRUE, NULL, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_START_COMMIT, hook, args, NULL,
+                           pool));
     }
 
   return SVN_NO_ERROR;
@@ -510,11 +610,12 @@ svn_repos__hooks_pre_commit(svn_repos_t *repos,
       const char *args[4];
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = txn_name;
       args[3] = NULL;
 
-      SVN_ERR(run_hook_cmd("pre-commit", hook, args, TRUE, NULL, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_PRE_COMMIT, hook, args, NULL,
+                           pool));
     }
 
   return SVN_NO_ERROR;
@@ -538,11 +639,12 @@ svn_repos__hooks_post_commit(svn_repos_t *repos,
       const char *args[4];
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = apr_psprintf(pool, "%ld", rev);
       args[3] = NULL;
 
-      SVN_ERR(run_hook_cmd("post-commit", hook, args, TRUE, NULL, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_POST_COMMIT, hook, args, NULL,
+                           pool));
     }
 
   return SVN_NO_ERROR;
@@ -582,14 +684,14 @@ svn_repos__hooks_pre_revprop_change(svn_repos_t *repos,
       action_string[1] = '\0';
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = apr_psprintf(pool, "%ld", rev);
       args[3] = author ? author : "";
       args[4] = name;
       args[5] = action_string;
       args[6] = NULL;
 
-      SVN_ERR(run_hook_cmd("pre-revprop-change", hook, args, TRUE,
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_PRE_REVPROP_CHANGE, hook, args,
                            stdin_handle, pool));
 
       SVN_ERR(svn_io_file_close(stdin_handle, pool));
@@ -600,8 +702,8 @@ svn_repos__hooks_pre_revprop_change(svn_repos_t *repos,
          MASSIVE PARANOIA.  Changing revision properties is a lossy
          operation; so unless the repository admininstrator has
          *deliberately* created the pre-hook, disallow all changes. */
-      return 
-        svn_error_create 
+      return
+        svn_error_create
         (SVN_ERR_REPOS_DISABLED_FEATURE, NULL,
          _("Repository has not been enabled to accept revision propchanges;\n"
            "ask the administrator to create a pre-revprop-change hook"));
@@ -622,7 +724,7 @@ svn_repos__hooks_post_revprop_change(svn_repos_t *repos,
 {
   const char *hook = svn_repos_post_revprop_change_hook(repos, pool);
   svn_boolean_t broken_link;
-  
+
   if ((hook = check_hook_cmd(hook, &broken_link, pool)) && broken_link)
     {
       return hook_symlink_error(hook);
@@ -644,16 +746,16 @@ svn_repos__hooks_post_revprop_change(svn_repos_t *repos,
       action_string[1] = '\0';
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = apr_psprintf(pool, "%ld", rev);
       args[3] = author ? author : "";
       args[4] = name;
       args[5] = action_string;
       args[6] = NULL;
 
-      SVN_ERR(run_hook_cmd("post-revprop-change", hook, args, FALSE,
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_POST_REVPROP_CHANGE, hook, args,
                            stdin_handle, pool));
-      
+
       SVN_ERR(svn_io_file_close(stdin_handle, pool));
     }
 
@@ -680,12 +782,12 @@ svn_repos__hooks_pre_lock(svn_repos_t *repos,
       const char *args[5];
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = path;
       args[3] = username;
       args[4] = NULL;
 
-      SVN_ERR(run_hook_cmd("pre-lock", hook, args, TRUE, NULL, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_PRE_LOCK, hook, args, NULL, pool));
     }
 
   return SVN_NO_ERROR;
@@ -700,7 +802,7 @@ svn_repos__hooks_post_lock(svn_repos_t *repos,
 {
   const char *hook = svn_repos_post_lock_hook(repos, pool);
   svn_boolean_t broken_link;
-  
+
   if ((hook = check_hook_cmd(hook, &broken_link, pool)) && broken_link)
     {
       return hook_symlink_error(hook);
@@ -710,19 +812,19 @@ svn_repos__hooks_post_lock(svn_repos_t *repos,
       const char *args[5];
       apr_file_t *stdin_handle = NULL;
       svn_string_t *paths_str = svn_string_create(svn_cstring_join
-                                                  (paths, "\n", pool), 
+                                                  (paths, "\n", pool),
                                                   pool);
 
       SVN_ERR(create_temp_file(&stdin_handle, paths_str, pool));
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = username;
       args[3] = NULL;
       args[4] = NULL;
 
-      SVN_ERR(run_hook_cmd("post-lock", hook, args, FALSE, 
-                           stdin_handle, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_POST_LOCK, hook, args, stdin_handle,
+                           pool));
 
       SVN_ERR(svn_io_file_close(stdin_handle, pool));
     }
@@ -749,12 +851,13 @@ svn_repos__hooks_pre_unlock(svn_repos_t *repos,
       const char *args[5];
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = path;
       args[3] = username ? username : "";
       args[4] = NULL;
 
-      SVN_ERR(run_hook_cmd("pre-unlock", hook, args, TRUE, NULL, pool));
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_PRE_UNLOCK, hook, args, NULL,
+                           pool));
     }
 
   return SVN_NO_ERROR;
@@ -769,7 +872,7 @@ svn_repos__hooks_post_unlock(svn_repos_t *repos,
 {
   const char *hook = svn_repos_post_unlock_hook(repos, pool);
   svn_boolean_t broken_link;
-  
+
   if ((hook = check_hook_cmd(hook, &broken_link, pool)) && broken_link)
     {
       return hook_symlink_error(hook);
@@ -779,18 +882,18 @@ svn_repos__hooks_post_unlock(svn_repos_t *repos,
       const char *args[5];
       apr_file_t *stdin_handle = NULL;
       svn_string_t *paths_str = svn_string_create(svn_cstring_join
-                                                  (paths, "\n", pool), 
+                                                  (paths, "\n", pool),
                                                   pool);
 
       SVN_ERR(create_temp_file(&stdin_handle, paths_str, pool));
 
       args[0] = hook;
-      args[1] = svn_repos_path(repos, pool);
+      args[1] = svn_path_local_style(svn_repos_path(repos, pool), pool);
       args[2] = username ? username : "";
       args[3] = NULL;
       args[4] = NULL;
 
-      SVN_ERR(run_hook_cmd("post-unlock", hook, args, FALSE, 
+      SVN_ERR(run_hook_cmd(SVN_REPOS__HOOK_POST_UNLOCK, hook, args,
                            stdin_handle, pool));
 
       SVN_ERR(svn_io_file_close(stdin_handle, pool));
@@ -801,8 +904,8 @@ svn_repos__hooks_post_unlock(svn_repos_t *repos,
 
 
 
-/* 
- * vim:ts=4:sw=4:expandtab:tw=80:fo=tcroq 
- * vim:isk=a-z,A-Z,48-57,_,.,-,> 
+/*
+ * vim:ts=4:sw=4:expandtab:tw=80:fo=tcroq
+ * vim:isk=a-z,A-Z,48-57,_,.,-,>
  * vim:cino=>1s,e0,n0,f0,{.5s,}0,^-.5s,=.5s,t0,+1s,c3,(0,u0,\:0
  */
