@@ -117,7 +117,7 @@ svn_opt_get_option_from_code2(int code,
               }
         return &(option_table[i]);
       }
-  
+
   return NULL;
 }
 
@@ -131,31 +131,46 @@ svn_opt_get_option_from_code(int code,
   for (i = 0; option_table[i].optch; i++)
     if (option_table[i].optch == code)
       return &(option_table[i]);
-  
+
   return NULL;
 }
 
 
 svn_boolean_t
-svn_opt_subcommand_takes_option2(const svn_opt_subcommand_desc2_t *command,
-                                 int option_code)
+svn_opt_subcommand_takes_option3(const svn_opt_subcommand_desc2_t *command,
+                                 int option_code,
+                                 const int *global_options)
 {
   apr_size_t i;
-  
+
   for (i = 0; i < SVN_OPT_MAX_OPTIONS; i++)
     if (command->valid_options[i] == option_code)
       return TRUE;
 
+  if (global_options)
+    for (i = 0; global_options[i]; i++)
+      if (global_options[i] == option_code)
+        return TRUE;
+
   return FALSE;
 }
 
+svn_boolean_t
+svn_opt_subcommand_takes_option2(const svn_opt_subcommand_desc2_t *command,
+                                 int option_code)
+{
+  return svn_opt_subcommand_takes_option3(command,
+                                          option_code,
+                                          NULL);
+}
+
 
 svn_boolean_t
 svn_opt_subcommand_takes_option(const svn_opt_subcommand_desc_t *command,
                                 int option_code)
 {
   apr_size_t i;
-  
+
   for (i = 0; i < SVN_OPT_MAX_OPTIONS; i++)
     if (command->valid_options[i] == option_code)
       return TRUE;
@@ -170,7 +185,8 @@ svn_opt_subcommand_takes_option(const svn_opt_subcommand_desc_t *command,
 static svn_error_t *
 print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
                     const apr_getopt_option_t *options_table,
-                    svn_boolean_t help, 
+                    const int *global_options,
+                    svn_boolean_t help,
                     apr_pool_t *pool,
                     FILE *stream)
 {
@@ -182,7 +198,7 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
 
   /* Print the list of aliases. */
   first_time = TRUE;
-  for (i = 0; i < SVN_OPT_MAX_ALIASES; i++) 
+  for (i = 0; i < SVN_OPT_MAX_ALIASES; i++)
     {
       if (cmd->aliases[i] == NULL)
         break;
@@ -193,13 +209,13 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
       }
       else
         SVN_ERR(svn_cmdline_fputs(", ", stream, pool));
-      
+
       SVN_ERR(svn_cmdline_fputs(cmd->aliases[i], stream, pool));
     }
 
   if (! first_time)
     SVN_ERR(svn_cmdline_fputs(")", stream, pool));
-  
+
   if (help)
     {
       const apr_getopt_option_t *option;
@@ -220,13 +236,13 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
                 }
 
               /* convert each option code into an option */
-              option = 
+              option =
                 svn_opt_get_option_from_code2(cmd->valid_options[i],
                                               options_table,
                                               cmd, pool);
 
               /* print the option's docstring */
-              if (option)
+              if (option && option->description)
                 {
                   const char *optstr;
                   svn_opt_format_option(&optstr, option, TRUE, pool);
@@ -235,7 +251,33 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
                 }
             }
         }
+      /* And global options too */
+      if (global_options && *global_options)
+        {
+          SVN_ERR(svn_cmdline_fputs(_("\nGlobal options:\n"),
+                                    stream, pool));
+          have_options = TRUE;
 
+          for (i = 0; global_options[i]; i++)
+            {
+              
+              /* convert each option code into an option */
+              option =
+                svn_opt_get_option_from_code2(global_options[i],
+                                              options_table,
+                                              cmd, pool);
+              
+              /* print the option's docstring */
+              if (option && option->description)
+                {
+                  const char *optstr;
+                  svn_opt_format_option(&optstr, option, TRUE, pool);
+                  SVN_ERR(svn_cmdline_fprintf(stream, pool, "  %s\n",
+                                              optstr));
+                }
+            }
+        }
+          
       if (have_options)
         SVN_ERR(svn_cmdline_fprintf(stream, pool, "\n"));
     }
@@ -248,7 +290,7 @@ print_command_info2(const svn_opt_subcommand_desc2_t *cmd,
 static svn_error_t *
 print_command_info(const svn_opt_subcommand_desc_t *cmd,
                    const apr_getopt_option_t *options_table,
-                   svn_boolean_t help, 
+                   svn_boolean_t help,
                    apr_pool_t *pool,
                    FILE *stream)
 {
@@ -260,7 +302,7 @@ print_command_info(const svn_opt_subcommand_desc_t *cmd,
 
   /* Print the list of aliases. */
   first_time = TRUE;
-  for (i = 0; i < SVN_OPT_MAX_ALIASES; i++) 
+  for (i = 0; i < SVN_OPT_MAX_ALIASES; i++)
     {
       if (cmd->aliases[i] == NULL)
         break;
@@ -271,13 +313,13 @@ print_command_info(const svn_opt_subcommand_desc_t *cmd,
       }
       else
         SVN_ERR(svn_cmdline_fputs(", ", stream, pool));
-      
+
       SVN_ERR(svn_cmdline_fputs(cmd->aliases[i], stream, pool));
     }
 
   if (! first_time)
     SVN_ERR(svn_cmdline_fputs(")", stream, pool));
-  
+
   if (help)
     {
       const apr_getopt_option_t *option;
@@ -298,12 +340,12 @@ print_command_info(const svn_opt_subcommand_desc_t *cmd,
                 }
 
               /* convert each option code into an option */
-              option = 
+              option =
                 svn_opt_get_option_from_code(cmd->valid_options[i],
                                              options_table);
 
               /* print the option's docstring */
-              if (option)
+              if (option && option->description)
                 {
                   const char *optstr;
                   svn_opt_format_option(&optstr, option, TRUE, pool);
@@ -334,11 +376,12 @@ svn_opt_print_generic_help2(const char *header,
   if (header)
     if ((err = svn_cmdline_fputs(header, stream, pool)))
       goto print_error;
-  
-  while (cmd_table[i].name) 
+
+  while (cmd_table[i].name)
     {
       if ((err = svn_cmdline_fputs("   ", stream, pool))
-          || (err = print_command_info2(cmd_table + i, opt_table, FALSE,
+          || (err = print_command_info2(cmd_table + i, opt_table,
+                                        NULL, FALSE,
                                         pool, stream))
           || (err = svn_cmdline_fputs("\n", stream, pool)))
         goto print_error;
@@ -373,8 +416,8 @@ svn_opt_print_generic_help(const char *header,
   if (header)
     if ((err = svn_cmdline_fputs(header, stream, pool)))
       goto print_error;
-  
-  while (cmd_table[i].name) 
+
+  while (cmd_table[i].name)
     {
       if ((err = svn_cmdline_fputs("   ", stream, pool))
           || (err = print_command_info(cmd_table + i, opt_table, FALSE,
@@ -415,13 +458,13 @@ svn_opt_format_option(const char **string,
 
   /* We have a valid option which may or may not have a "short
      name" (a single-character alias for the long option). */
-  if (opt->optch <= 255)  
+  if (opt->optch <= 255)
     opts = apr_psprintf(pool, "-%c [--%s]", opt->optch, opt->name);
   else
     opts = apr_psprintf(pool, "--%s", opt->name);
 
   if (opt->has_arg)
-    opts = apr_pstrcat(pool, opts, _(" arg"), NULL);
+    opts = apr_pstrcat(pool, opts, _(" ARG"), NULL);
 
   if (doc)
     opts = apr_psprintf(pool, "%-24s : %s", opts, _(opt->description));
@@ -431,27 +474,39 @@ svn_opt_format_option(const char **string,
 
 
 void
-svn_opt_subcommand_help2(const char *subcommand,
+svn_opt_subcommand_help3(const char *subcommand,
                          const svn_opt_subcommand_desc2_t *table,
                          const apr_getopt_option_t *options_table,
+                         const int *global_options,
                          apr_pool_t *pool)
 {
   const svn_opt_subcommand_desc2_t *cmd =
     svn_opt_get_canonical_subcommand2(table, subcommand);
   svn_error_t *err;
-    
+
   if (cmd)
-    err = print_command_info2(cmd, options_table, TRUE, pool, stdout);
+    err = print_command_info2(cmd, options_table, global_options,
+                              TRUE, pool, stdout);
   else
     err = svn_cmdline_fprintf(stderr, pool,
                               _("\"%s\": unknown command.\n\n"), subcommand);
-  
+
   if (err) {
     svn_handle_error2(err, stderr, FALSE, "svn: ");
     svn_error_clear(err);
   }
 }
 
+void
+svn_opt_subcommand_help2(const char *subcommand,
+                         const svn_opt_subcommand_desc2_t *table,
+                         const apr_getopt_option_t *options_table,
+                         apr_pool_t *pool)
+{
+  svn_opt_subcommand_help3(subcommand, table, options_table,
+                           NULL, pool);
+}
+
 
 void
 svn_opt_subcommand_help(const char *subcommand,
@@ -462,13 +517,13 @@ svn_opt_subcommand_help(const char *subcommand,
   const svn_opt_subcommand_desc_t *cmd =
     svn_opt_get_canonical_subcommand(table, subcommand);
   svn_error_t *err;
-    
+
   if (cmd)
     err = print_command_info(cmd, options_table, TRUE, pool, stdout);
   else
     err = svn_cmdline_fprintf(stderr, pool,
                               _("\"%s\": unknown command.\n\n"), subcommand);
-  
+
   if (err) {
     svn_handle_error2(err, stderr, FALSE, "svn: ");
     svn_error_clear(err);
@@ -498,19 +553,19 @@ svn_opt_subcommand_help(const char *subcommand,
 static int
 revision_from_word(svn_opt_revision_t *revision, const char *word)
 {
-  if (strcasecmp(word, "head") == 0)
+  if (svn_cstring_casecmp(word, "head") == 0)
     {
       revision->kind = svn_opt_revision_head;
     }
-  else if (strcasecmp(word, "prev") == 0)
+  else if (svn_cstring_casecmp(word, "prev") == 0)
     {
       revision->kind = svn_opt_revision_previous;
     }
-  else if (strcasecmp(word, "base") == 0)
+  else if (svn_cstring_casecmp(word, "base") == 0)
     {
       revision->kind = svn_opt_revision_base;
     }
-  else if (strcasecmp(word, "committed") == 0)
+  else if (svn_cstring_casecmp(word, "committed") == 0)
     {
       revision->kind = svn_opt_revision_committed;
     }
@@ -530,6 +585,20 @@ static char *parse_one_rev(svn_opt_revision_t *revision, char *str,
 {
   char *end, save;
 
+  /* Allow any number of 'r's to prefix a revision number, because
+     that way if a script pastes svn output into another svn command
+     (like "svn log -r${REV_COPIED_FROM_OUTPUT}"), it'll Just Work,
+     even when compounded.
+
+     As it happens, none of our special revision words begins with
+     "r".  If any ever do, then this code will have to get smarter.
+
+     Incidentally, this allows "r{DATE}".  We could avoid that with
+     some trivial code rearrangement, but it's not clear what would
+     be gained by doing so. */
+  while (*str == 'r')
+    str++;
+
   if (*str == '{')
     {
       svn_boolean_t matched;
@@ -549,7 +618,7 @@ static char *parse_one_rev(svn_opt_revision_t *revision, char *str,
           return NULL;
         }
       if (!matched)
-        return NULL; 
+        return NULL;
       revision->kind = svn_opt_revision_date;
       revision->value.date = tm;
       return end + 1;
@@ -610,13 +679,59 @@ svn_opt_parse_revision(svn_opt_revision_t *start_revision,
 }
 
 
+int
+svn_opt_parse_revision_to_range(apr_array_header_t *opt_ranges,
+                                const char *arg,
+                                apr_pool_t *pool)
+{
+  svn_opt_revision_range_t *range = apr_palloc(pool, sizeof(*range));
+
+  range->start.kind = svn_opt_revision_unspecified;
+  range->end.kind = svn_opt_revision_unspecified;
+
+  if (svn_opt_parse_revision(&(range->start), &(range->end),
+                             arg, pool) == -1)
+    return -1;
+
+  APR_ARRAY_PUSH(opt_ranges, svn_opt_revision_range_t *) = range;
+  return 0;
+}
+
+svn_error_t *
+svn_opt_resolve_revisions(svn_opt_revision_t *peg_rev,
+                          svn_opt_revision_t *op_rev,
+                          svn_boolean_t is_url,
+                          svn_boolean_t notice_local_mods,
+                          apr_pool_t *pool)
+{
+  if (peg_rev->kind == svn_opt_revision_unspecified)
+    {
+      if (is_url)
+        {
+          peg_rev->kind = svn_opt_revision_head;
+        }
+      else
+        {
+          if (notice_local_mods)
+            peg_rev->kind = svn_opt_revision_working;
+          else
+            peg_rev->kind = svn_opt_revision_base;
+        }
+    }
+
+  if (op_rev->kind == svn_opt_revision_unspecified)
+    *op_rev = *peg_rev;
+
+  return SVN_NO_ERROR;
+}
+
 
 /*** Parsing arguments. ***/
 #define DEFAULT_ARRAY_SIZE 5
 
 
 /* Copy STR into POOL and push the copy onto ARRAY. */
-static void 
+static void
 array_push_str(apr_array_header_t *array,
                const char *str,
                apr_pool_t *pool)
@@ -627,12 +742,12 @@ array_push_str(apr_array_header_t *array,
      transfer str's lifetime to pool.  Is that something callers are
      depending on? */
 
-  (*((const char **) apr_array_push(array))) = apr_pstrdup(pool, str);
+  APR_ARRAY_PUSH(array, const char *) = apr_pstrdup(pool, str);
 }
 
 
 void
-svn_opt_push_implicit_dot_target(apr_array_header_t *targets, 
+svn_opt_push_implicit_dot_target(apr_array_header_t *targets,
                                  apr_pool_t *pool)
 {
   if (targets->nelts == 0)
@@ -648,7 +763,7 @@ svn_opt_parse_num_args(apr_array_header_t **args_p,
                        apr_pool_t *pool)
 {
   int i;
-  apr_array_header_t *args 
+  apr_array_header_t *args
     = apr_array_make(pool, DEFAULT_ARRAY_SIZE, sizeof(const char *));
 
   /* loop for num_args and add each arg to the args array */
@@ -670,7 +785,7 @@ svn_opt_parse_all_args(apr_array_header_t **args_p,
                        apr_getopt_t *os,
                        apr_pool_t *pool)
 {
-  apr_array_header_t *args 
+  apr_array_header_t *args
     = apr_array_make(pool, DEFAULT_ARRAY_SIZE, sizeof(const char *));
 
   if (os->ind > os->argc)
@@ -706,45 +821,34 @@ svn_opt_parse_path(svn_opt_revision_t *rev,
 
       if (path[i] == '@')
         {
-          svn_boolean_t is_url;
           int ret;
           svn_opt_revision_t start_revision, end_revision;
 
           end_revision.kind = svn_opt_revision_unspecified;
 
-          /* URLs get treated differently from wc paths. */
-          is_url = svn_path_is_url(path);
-
           if (path[i + 1] == '\0')  /* looking at empty peg revision */
             {
-              if (is_url)
-                {
-                  ret = svn_opt_parse_revision(&start_revision, &end_revision,
-                                               "head", pool);
-                }
-              else
-                {
-                  ret = svn_opt_parse_revision(&start_revision, &end_revision,
-                                               "base", pool);
-                }
+              ret = 0;
+              start_revision.kind = svn_opt_revision_unspecified;
             }
           else  /* looking at non-empty peg revision */
             {
               const char *rev_str = path + i + 1;
 
-              if (is_url)
+              /* URLs get treated differently from wc paths. */
+              if (svn_path_is_url(path))
                 {
                   /* URLs are URI-encoded, so we look for dates with
                      URI-encoded delimeters.  */
                   int rev_len = strlen(rev_str);
                   if (rev_len > 6
-                      && rev_str[0] == '%' 
-                      && rev_str[1] == '7' 
-                      && (rev_str[2] == 'B' 
+                      && rev_str[0] == '%'
+                      && rev_str[1] == '7'
+                      && (rev_str[2] == 'B'
                           || rev_str[2] == 'b')
-                      && rev_str[rev_len-3] == '%' 
-                      && rev_str[rev_len-2] == '7' 
-                      && (rev_str[rev_len-1] == 'D' 
+                      && rev_str[rev_len-3] == '%'
+                      && rev_str[rev_len-2] == '7'
+                      && (rev_str[rev_len-1] == 'D'
                           || rev_str[rev_len-1] == 'd'))
                     {
                       rev_str = svn_path_uri_decode(rev_str, pool);
@@ -760,8 +864,7 @@ svn_opt_parse_path(svn_opt_revision_t *rev,
                                      _("Syntax error parsing revision '%s'"),
                                      path + i + 1);
 
-          *truepath = svn_path_canonicalize(apr_pstrndup(pool, path, i),
-                                            pool);
+          *truepath = apr_pstrmemdup(pool, path, i);
           rev->kind = start_revision.kind;
           rev->value = start_revision.value;
 
@@ -770,7 +873,7 @@ svn_opt_parse_path(svn_opt_revision_t *rev,
     }
 
   /* Didn't find an @-sign. */
-  *truepath = svn_path_canonicalize(path, pool);
+  *truepath = path;
   rev->kind = svn_opt_revision_unspecified;
 
   return SVN_NO_ERROR;
@@ -778,12 +881,32 @@ svn_opt_parse_path(svn_opt_revision_t *rev,
 
 
 svn_error_t *
-svn_opt_args_to_target_array2(apr_array_header_t **targets_p, 
+svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
+                              apr_getopt_t *os,
+                              apr_array_header_t *known_targets,
+                              apr_pool_t *pool)
+{
+  svn_error_t *err = svn_opt_args_to_target_array3(targets_p, os, 
+                                                   known_targets, pool);
+
+  if (err && err->apr_err == SVN_ERR_RESERVED_FILENAME_SPECIFIED)
+    {
+      svn_error_clear(err);
+      return SVN_NO_ERROR;
+    }
+
+  return err;
+}
+
+
+svn_error_t *
+svn_opt_args_to_target_array3(apr_array_header_t **targets_p,
                               apr_getopt_t *os,
                               apr_array_header_t *known_targets,
                               apr_pool_t *pool)
 {
   int i;
+  svn_error_t *err = SVN_NO_ERROR;
   apr_array_header_t *input_targets =
     apr_array_make(pool, DEFAULT_ARRAY_SIZE, sizeof(const char *));
   apr_array_header_t *output_targets =
@@ -810,7 +933,7 @@ svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
              because we needed to split up the list with svn_cstring_split. */
           const char *utf8_target = APR_ARRAY_IDX(known_targets,
                                                   i, const char *);
-          (*((const char **) apr_array_push(input_targets))) = utf8_target;
+          APR_ARRAY_PUSH(input_targets, const char *) = utf8_target;
         }
     }
 
@@ -819,7 +942,39 @@ svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
   for (i = 0; i < input_targets->nelts; i++)
     {
       const char *utf8_target = APR_ARRAY_IDX(input_targets, i, const char *);
+      const char *peg_start = NULL; /* pointer to the peg revision, if any */
       const char *target;      /* after all processing is finished */
+      int j;
+
+      /* Remove a peg revision, if any, in the target so that it can
+         be properly canonicalized, otherwise the canonicalization
+         does not treat a ".@BASE" as a "." with a BASE peg revision,
+         and it is not canonicalized to "@BASE".  If any peg revision
+         exists, it is appended to the final canonicalized path or
+         URL.  Do not use svn_opt_parse_path() because the resulting
+         peg revision is a structure that would have to be converted
+         back into a string.  Converting from a string date to the
+         apr_time_t field in the svn_opt_revision_value_t and back to
+         a string would not necessarily preserve the exact bytes of
+         the input date, so its easier just to keep it in string
+         form. */
+      for (j = (strlen(utf8_target) - 1); j >= 0; --j)
+        {
+          /* If we hit a path separator, stop looking.  This is OK
+              only because our revision specifiers can't contain
+              '/'. */
+          if (utf8_target[j] == '/')
+            break;
+          if (utf8_target[j] == '@')
+            {
+              peg_start = utf8_target + j;
+              break;
+            }
+        }
+      if (peg_start)
+        utf8_target = apr_pstrmemdup(pool,
+                                     utf8_target,
+                                     peg_start - utf8_target);
 
       /* URLs and wc-paths get treated differently. */
       if (svn_path_is_url(utf8_target))
@@ -842,7 +997,7 @@ svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
             return svn_error_createf(SVN_ERR_BAD_URL, 0,
                                      _("URL '%s' contains a '..' element"),
                                      utf8_target);
-          
+
           /* strip any trailing '/' */
           target = svn_path_canonicalize(target, pool);
         }
@@ -889,23 +1044,34 @@ svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
              synchronized! */
           if (0 == strcmp(base_name, ".svn")
               || 0 == strcmp(base_name, "_svn"))
-            continue;
+            {
+              err = svn_error_createf(SVN_ERR_RESERVED_FILENAME_SPECIFIED,
+                                      err, _("'%s' ends in a reserved name"),
+                                      target);
+              continue;
+            }
         }
 
-      (*((const char **) apr_array_push(output_targets))) = target;
+      /* Append the peg revision back to the canonicalized target if
+         there was a peg revision. */
+      if (peg_start)
+        target = apr_pstrcat(pool, target, peg_start, NULL);
+
+      APR_ARRAY_PUSH(output_targets, const char *) = target;
     }
 
 
   /* kff todo: need to remove redundancies from targets before
      passing it to the cmd_func. */
-  
+
   *targets_p = output_targets;
-  return SVN_NO_ERROR;
+
+  return err;
 }
 
 
 svn_error_t *
-svn_opt_args_to_target_array(apr_array_header_t **targets_p, 
+svn_opt_args_to_target_array(apr_array_header_t **targets_p,
                              apr_getopt_t *os,
                              apr_array_header_t *known_targets,
                              svn_opt_revision_t *start_revision,
@@ -925,28 +1091,28 @@ svn_opt_args_to_target_array(apr_array_header_t **targets_p,
 
       if (output_targets->nelts > 0)
         {
-          path = ((const char **) (output_targets->elts))[0];
+          path = APR_ARRAY_IDX(output_targets, 0, const char *);
           SVN_ERR(svn_opt_parse_path(&temprev, &path, path, pool));
           if (temprev.kind != svn_opt_revision_unspecified)
             {
-              ((const char **) (output_targets->elts))[0] = path;
+              APR_ARRAY_IDX(output_targets, 0, const char *) = path;
               start_revision->kind = temprev.kind;
               start_revision->value = temprev.value;
             }
         }
       if (output_targets->nelts > 1)
         {
-          path = ((const char **) (output_targets->elts))[1];
+          path = APR_ARRAY_IDX(output_targets, 1, const char *);
           SVN_ERR(svn_opt_parse_path(&temprev, &path, path, pool));
           if (temprev.kind != svn_opt_revision_unspecified)
             {
-              ((const char **) (output_targets->elts))[1] = path;
+              APR_ARRAY_IDX(output_targets, 1, const char *) = path;
               end_revision->kind = temprev.kind;
               end_revision->value = temprev.value;
             }
         }
     }
-  
+
   *targets_p = output_targets;
   return SVN_NO_ERROR;
 }
@@ -974,7 +1140,7 @@ print_version_info(const char *pgm_name,
   SVN_ERR(svn_cmdline_printf(pool, _("%s, version %s\n"
                                      "   compiled %s, %s\n\n"), pgm_name,
                              SVN_VERSION, __DATE__, __TIME__));
-  SVN_ERR(svn_cmdline_fputs(_("Copyright (C) 2000-2007 CollabNet.\n"
+  SVN_ERR(svn_cmdline_fputs(_("Copyright (C) 2000-2008 CollabNet.\n"
                               "Subversion is open source software, see"
                               " http://subversion.tigris.org/\n"
                               "This product includes software developed by "
@@ -990,6 +1156,49 @@ print_version_info(const char *pgm_name,
 }
 
 
+svn_error_t *
+svn_opt_print_help3(apr_getopt_t *os,
+                    const char *pgm_name,
+                    svn_boolean_t print_version,
+                    svn_boolean_t quiet,
+                    const char *version_footer,
+                    const char *header,
+                    const svn_opt_subcommand_desc2_t *cmd_table,
+                    const apr_getopt_option_t *option_table,
+                    const int *global_options,
+                    const char *footer,
+                    apr_pool_t *pool)
+{
+  apr_array_header_t *targets = NULL;
+  int i;
+
+  if (os)
+    SVN_ERR(svn_opt_parse_all_args(&targets, os, pool));
+
+  if (os && targets->nelts)  /* help on subcommand(s) requested */
+    for (i = 0; i < targets->nelts; i++)
+      {
+        svn_opt_subcommand_help3(APR_ARRAY_IDX(targets, i, const char *),
+                                 cmd_table, option_table,
+                                 global_options, pool);
+      }
+  else if (print_version)   /* just --version */
+    SVN_ERR(print_version_info(pgm_name, version_footer, quiet, pool));
+  else if (os && !targets->nelts)            /* `-h', `--help', or `help' */
+    svn_opt_print_generic_help2(header,
+                                cmd_table,
+                                option_table,
+                                footer,
+                                pool,
+                                stdout);
+  else                                       /* unknown option or cmd */
+    SVN_ERR(svn_cmdline_fprintf(stderr, pool,
+                                _("Type '%s help' for usage.\n"), pgm_name));
+
+  return SVN_NO_ERROR;
+}
+
+
 svn_error_t *
 svn_opt_print_help2(apr_getopt_t *os,
                     const char *pgm_name,
@@ -1002,34 +1211,17 @@ svn_opt_print_help2(apr_getopt_t *os,
                     const char *footer,
                     apr_pool_t *pool)
 {
-  apr_array_header_t *targets = NULL;
-  int i;
-
-  if (os)
-    SVN_ERR(svn_opt_parse_all_args(&targets, os, pool));
-
-  if (cmd_table
-      && targets
-      && targets->nelts)  /* help on subcommand(s) requested */
-    for (i = 0; i < targets->nelts; i++)
-      {
-        svn_opt_subcommand_help2(((const char **) (targets->elts))[i],
-                                 cmd_table, option_table, pool);
-      }
-  else if (print_version)   /* just --version */
-    SVN_ERR(print_version_info(pgm_name, version_footer, quiet, pool));
-  else if (os && !targets->nelts)            /* `-h', `--help', or `help' */
-    svn_opt_print_generic_help2(header,
-                                cmd_table,
-                                option_table,
-                                footer,
-                                pool,
-                                stdout);
-  else                                       /* unknown option or cmd */
-    SVN_ERR(svn_cmdline_fprintf(stderr, pool,
-                                _("Type '%s help' for usage.\n"), pgm_name));
-
-  return SVN_NO_ERROR;
+  return svn_opt_print_help3(os,
+                             pgm_name,
+                             print_version,
+                             quiet,
+                             version_footer,
+                             header,
+                             cmd_table,
+                             option_table,
+                             NULL,
+                             footer,
+                             pool);
 }
 
 
@@ -1051,12 +1243,10 @@ svn_opt_print_help(apr_getopt_t *os,
   if (os)
     SVN_ERR(svn_opt_parse_all_args(&targets, os, pool));
 
-  if (cmd_table
-      && targets
-      && targets->nelts)  /* help on subcommand(s) requested */
+  if (os && targets->nelts)  /* help on subcommand(s) requested */
     for (i = 0; i < targets->nelts; i++)
       {
-        svn_opt_subcommand_help(((const char **) (targets->elts))[i],
+        svn_opt_subcommand_help(APR_ARRAY_IDX(targets, i, const char *),
                                 cmd_table, option_table, pool);
       }
   else if (print_version)   /* just --version */
