@@ -1,10 +1,10 @@
 /*
  * svn_string.h:  routines to manipulate counted-length strings
  *                (svn_stringbuf_t and svn_string_t) and C strings.
- *                
+ *
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -25,13 +25,14 @@
 #include <apr_md5.h>
 #include <apr_fnmatch.h>
 #include "svn_string.h"  /* loads "svn_types.h" and <apr_pools.h> */
+#include "svn_ctype.h"
 
 
 
 /* Our own realloc, since APR doesn't have one.  Note: this is a
    generic realloc for memory pools, *not* for strings. */
 static void *
-my__realloc(char *data, apr_size_t oldsize, apr_size_t request, 
+my__realloc(char *data, apr_size_t oldsize, apr_size_t request,
             apr_pool_t *pool)
 {
   void *new_area;
@@ -49,7 +50,7 @@ my__realloc(char *data, apr_size_t oldsize, apr_size_t request,
   memcpy(new_area, data, oldsize);
 
   /* I'm NOT freeing old area here -- cuz we're using pools, ugh. */
-  
+
   /* return new area */
   return new_area;
 }
@@ -111,7 +112,7 @@ create_string(const char *data, apr_size_t size,
 {
   svn_string_t *new_string;
 
-  new_string = apr_palloc(pool, sizeof(*new_string)); 
+  new_string = apr_palloc(pool, sizeof(*new_string));
 
   new_string->data = data;
   new_string->len = size;
@@ -223,7 +224,7 @@ create_stringbuf(char *data, apr_size_t size, apr_pool_t *pool)
 {
   svn_stringbuf_t *new_string;
 
-  new_string = apr_palloc(pool, sizeof(*new_string)); 
+  new_string = apr_palloc(pool, sizeof(*new_string));
 
   new_string->data = data;
   new_string->len = size;
@@ -289,7 +290,7 @@ svn_stringbuf_createf(apr_pool_t *pool, const char *fmt, ...)
 }
 
 
-void 
+void
 svn_stringbuf_fillchar(svn_stringbuf_t *str, unsigned char c)
 {
   memset(str->data, c, str->len);
@@ -356,16 +357,18 @@ svn_stringbuf_ensure(svn_stringbuf_t *str, apr_size_t minimum_size)
               }
           }
 
-      str->data = (char *) my__realloc(str->data, 
-                                       str->len,
+      str->data = (char *) my__realloc(str->data,
+                                       str->len + 1,
+                                       /* We need to maintain (and thus copy)
+                                          the trailing nul */
                                        str->blocksize,
-                                       str->pool); 
+                                       str->pool);
     }
 }
 
 
 void
-svn_stringbuf_appendbytes(svn_stringbuf_t *str, const char *bytes, 
+svn_stringbuf_appendbytes(svn_stringbuf_t *str, const char *bytes,
                           apr_size_t count)
 {
   apr_size_t total_len;
@@ -389,7 +392,7 @@ svn_stringbuf_appendbytes(svn_stringbuf_t *str, const char *bytes,
 
 
 void
-svn_stringbuf_appendstr(svn_stringbuf_t *targetstr, 
+svn_stringbuf_appendstr(svn_stringbuf_t *targetstr,
                         const svn_stringbuf_t *appendstr)
 {
   svn_stringbuf_appendbytes(targetstr, appendstr->data, appendstr->len);
@@ -415,7 +418,7 @@ svn_stringbuf_dup(const svn_stringbuf_t *original_string, apr_pool_t *pool)
 
 
 svn_boolean_t
-svn_stringbuf_compare(const svn_stringbuf_t *str1, 
+svn_stringbuf_compare(const svn_stringbuf_t *str1,
                       const svn_stringbuf_t *str2)
 {
   return string_compare(str1->data, str2->data, str1->len, str2->len);
@@ -479,14 +482,14 @@ svn_cstring_split_append(apr_array_header_t *array,
 
   pats = apr_pstrdup(pool, input);  /* strtok wants non-const data */
   p = apr_strtok(pats, sep_chars, &last);
-  
+
   while (p)
     {
       if (chop_whitespace)
         {
           while (apr_isspace(*p))
             p++;
-          
+
           {
             char *e = p + (strlen(p) - 1);
             while ((e >= p) && (apr_isspace(*e)))
@@ -496,7 +499,7 @@ svn_cstring_split_append(apr_array_header_t *array,
         }
 
       if (p[0] != '\0')
-        (*((const char **) apr_array_push(array))) = p;
+        APR_ARRAY_PUSH(array, const char *) = p;
 
       p = apr_strtok(NULL, sep_chars, &last);
     }
@@ -565,12 +568,25 @@ svn_cstring_join(apr_array_header_t *strings,
   svn_stringbuf_t *new_str = svn_stringbuf_create("", pool);
   int sep_len = strlen(separator);
   int i;
-  
+
   for (i = 0; i < strings->nelts; i++)
     {
-      const char *string = ((const char **) (strings->elts))[i];
+      const char *string = APR_ARRAY_IDX(strings, i, const char *);
       svn_stringbuf_appendbytes(new_str, string, strlen(string));
       svn_stringbuf_appendbytes(new_str, separator, sep_len);
     }
   return new_str->data;
 }
+
+int
+svn_cstring_casecmp(const char *str1, const char *str2)
+{
+  for (;;)
+    {
+      const int a = *str1++;
+      const int b = *str2++;
+      const int cmp = svn_ctype_casecmp(a, b);
+      if (cmp || !a || !b)
+        return cmp;
+    }
+}
