@@ -2,7 +2,7 @@
  * entries.h :  manipulating entries
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -37,7 +37,10 @@ extern "C" {
 #define SVN_WC__ENTRIES_ATTR_DIR_STR    "dir"
 
 
-/* The names of the XML attributes for storing entries' information.
+/* The names of the fields used for storing entries' information.
+   Used for the names of the XML attributes in XML entries files
+   (format 6 and below), for the names of attributes in wc logs,
+   and for error reporting when reading a non-XML entries file.
    ### If you add or remove items here, you probably want to make sure
    to do the same for the SVN_WC__ENTRY_MODIFY_* #defines as well. */
 #define SVN_WC__ENTRY_ATTR_NAME               "name"
@@ -67,6 +70,13 @@ extern "C" {
 #define SVN_WC__ENTRY_ATTR_LOCK_OWNER         "lock-owner"
 #define SVN_WC__ENTRY_ATTR_LOCK_COMMENT       "lock-comment"
 #define SVN_WC__ENTRY_ATTR_LOCK_CREATION_DATE "lock-creation-date"
+#define SVN_WC__ENTRY_ATTR_HAS_PROPS          "has-props"
+#define SVN_WC__ENTRY_ATTR_HAS_PROP_MODS      "has-prop-mods"
+#define SVN_WC__ENTRY_ATTR_CACHABLE_PROPS     "cachable-props"
+#define SVN_WC__ENTRY_ATTR_PRESENT_PROPS      "present-props"
+#define SVN_WC__ENTRY_ATTR_CHANGELIST         "changelist"
+#define SVN_WC__ENTRY_ATTR_KEEP_LOCAL         "keep-local"
+#define SVN_WC__ENTRY_ATTR_WORKING_SIZE       "working-size"
 
 /* Attribute values for 'schedule' */
 #define SVN_WC__ENTRY_VALUE_ADD        "add"
@@ -79,93 +89,95 @@ extern "C" {
    area for PATH.  The adm area must not already have an entries
    file.  UUID is the repository UUID, and may be NULL.  REPOS is the
    repository root URL and, if not NULL, must be a prefix of URL.
+   DEPTH is the initial depth of the working copy, it must be a
+   definite depth, not svn_depth_unknown.
 
    If initial rev is valid and non-zero, then mark the 'this_dir'
    entry as being incomplete.
 */
-svn_error_t *svn_wc__entries_init (const char *path,
-                                   const char *uuid,
-                                   const char *url,
-                                   const char *repos,
-                                   svn_revnum_t initial_rev,
-                                   apr_pool_t *pool);
+svn_error_t *svn_wc__entries_init(const char *path,
+                                  const char *uuid,
+                                  const char *url,
+                                  const char *repos,
+                                  svn_revnum_t initial_rev,
+                                  svn_depth_t depth,
+                                  apr_pool_t *pool);
 
 
 /* Create or overwrite an `entries' file for ADM_ACCESS using the contents
    of ENTRIES.  See also svn_wc_entries_read() in the public api. */
-svn_error_t *svn_wc__entries_write (apr_hash_t *entries,
-                                    svn_wc_adm_access_t *adm_access,
-                                    apr_pool_t *pool);
+svn_error_t *svn_wc__entries_write(apr_hash_t *entries,
+                                   svn_wc_adm_access_t *adm_access,
+                                   apr_pool_t *pool);
 
 
 /* Set *NEW_ENTRY to a new entry, taking attributes from ATTS, whose
-   keys and values are both char *.  Allocate the entry itself in
-   POOL, but don't copy attributes into POOL.  Instead,
-   (*NEW_ENTRY)->attributes and any allocated members in *NEW_ENTRY
-   will refer directly to ATTS and its values.
+   keys and values are both char *.  Allocate the entry and copy
+   attributes into POOL as needed.
 
    Set MODIFY_FLAGS to reflect the fields that were present in ATTS. */
-svn_error_t *svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
-                                    apr_uint32_t *modify_flags,
-                                    apr_hash_t *atts,
-                                    apr_pool_t *pool);
+svn_error_t *svn_wc__atts_to_entry(svn_wc_entry_t **new_entry,
+                                   apr_uint64_t *modify_flags,
+                                   apr_hash_t *atts,
+                                   apr_pool_t *pool);
 
 
 /* The MODIFY_FLAGS that tell svn_wc__entry_modify which parameters to
    pay attention to.  ### These should track the changes made to the
    SVN_WC__ENTRY_ATTR_* #defines! */
-#define SVN_WC__ENTRY_MODIFY_REVISION           0x00000001
-#define SVN_WC__ENTRY_MODIFY_URL                0x00000002
-#define SVN_WC__ENTRY_MODIFY_REPOS              0x00000004
-#define SVN_WC__ENTRY_MODIFY_KIND               0x00000008
-#define SVN_WC__ENTRY_MODIFY_TEXT_TIME          0x00000010
-#define SVN_WC__ENTRY_MODIFY_PROP_TIME          0x00000020
-#define SVN_WC__ENTRY_MODIFY_CHECKSUM           0x00000040
-#define SVN_WC__ENTRY_MODIFY_SCHEDULE           0x00000080
-#define SVN_WC__ENTRY_MODIFY_COPIED             0x00000100
-#define SVN_WC__ENTRY_MODIFY_DELETED            0x00000200
-#define SVN_WC__ENTRY_MODIFY_COPYFROM_URL       0x00000400
-#define SVN_WC__ENTRY_MODIFY_COPYFROM_REV       0x00000800
-#define SVN_WC__ENTRY_MODIFY_CONFLICT_OLD       0x00001000
-#define SVN_WC__ENTRY_MODIFY_CONFLICT_NEW       0x00002000
-#define SVN_WC__ENTRY_MODIFY_CONFLICT_WRK       0x00004000
-#define SVN_WC__ENTRY_MODIFY_PREJFILE           0x00008000
-#define SVN_WC__ENTRY_MODIFY_CMT_REV            0x00010000
-#define SVN_WC__ENTRY_MODIFY_CMT_DATE           0x00020000
-#define SVN_WC__ENTRY_MODIFY_CMT_AUTHOR         0x00040000
-#define SVN_WC__ENTRY_MODIFY_UUID               0x00080000
-#define SVN_WC__ENTRY_MODIFY_INCOMPLETE         0x00100000
-#define SVN_WC__ENTRY_MODIFY_ABSENT             0x00200000
-#define SVN_WC__ENTRY_MODIFY_LOCK_TOKEN         0x00400000
-#define SVN_WC__ENTRY_MODIFY_LOCK_OWNER         0x00800000
-#define SVN_WC__ENTRY_MODIFY_LOCK_COMMENT       0x01000000
-#define SVN_WC__ENTRY_MODIFY_LOCK_CREATION_DATE 0x02000000
-
-
-/* ...or perhaps this to mean all of those above... */
-#define SVN_WC__ENTRY_MODIFY_ALL           0x7FFFFFFF
+/* Note: we use APR_INT64_C because APR 0.9 lacks APR_UINT64_C */
+#define SVN_WC__ENTRY_MODIFY_REVISION           APR_INT64_C(0x0000000000000001)
+#define SVN_WC__ENTRY_MODIFY_URL                APR_INT64_C(0x0000000000000002)
+#define SVN_WC__ENTRY_MODIFY_REPOS              APR_INT64_C(0x0000000000000004)
+#define SVN_WC__ENTRY_MODIFY_KIND               APR_INT64_C(0x0000000000000008)
+#define SVN_WC__ENTRY_MODIFY_TEXT_TIME          APR_INT64_C(0x0000000000000010)
+#define SVN_WC__ENTRY_MODIFY_PROP_TIME          APR_INT64_C(0x0000000000000020)
+#define SVN_WC__ENTRY_MODIFY_CHECKSUM           APR_INT64_C(0x0000000000000040)
+#define SVN_WC__ENTRY_MODIFY_SCHEDULE           APR_INT64_C(0x0000000000000080)
+#define SVN_WC__ENTRY_MODIFY_COPIED             APR_INT64_C(0x0000000000000100)
+#define SVN_WC__ENTRY_MODIFY_DELETED            APR_INT64_C(0x0000000000000200)
+#define SVN_WC__ENTRY_MODIFY_COPYFROM_URL       APR_INT64_C(0x0000000000000400)
+#define SVN_WC__ENTRY_MODIFY_COPYFROM_REV       APR_INT64_C(0x0000000000000800)
+#define SVN_WC__ENTRY_MODIFY_CONFLICT_OLD       APR_INT64_C(0x0000000000001000)
+#define SVN_WC__ENTRY_MODIFY_CONFLICT_NEW       APR_INT64_C(0x0000000000002000)
+#define SVN_WC__ENTRY_MODIFY_CONFLICT_WRK       APR_INT64_C(0x0000000000004000)
+#define SVN_WC__ENTRY_MODIFY_PREJFILE           APR_INT64_C(0x0000000000008000)
+#define SVN_WC__ENTRY_MODIFY_CMT_REV            APR_INT64_C(0x0000000000010000)
+#define SVN_WC__ENTRY_MODIFY_CMT_DATE           APR_INT64_C(0x0000000000020000)
+#define SVN_WC__ENTRY_MODIFY_CMT_AUTHOR         APR_INT64_C(0x0000000000040000)
+#define SVN_WC__ENTRY_MODIFY_UUID               APR_INT64_C(0x0000000000080000)
+#define SVN_WC__ENTRY_MODIFY_INCOMPLETE         APR_INT64_C(0x0000000000100000)
+#define SVN_WC__ENTRY_MODIFY_ABSENT             APR_INT64_C(0x0000000000200000)
+#define SVN_WC__ENTRY_MODIFY_LOCK_TOKEN         APR_INT64_C(0x0000000000400000)
+#define SVN_WC__ENTRY_MODIFY_LOCK_OWNER         APR_INT64_C(0x0000000000800000)
+#define SVN_WC__ENTRY_MODIFY_LOCK_COMMENT       APR_INT64_C(0x0000000001000000)
+#define SVN_WC__ENTRY_MODIFY_LOCK_CREATION_DATE APR_INT64_C(0x0000000002000000)
+#define SVN_WC__ENTRY_MODIFY_HAS_PROPS          APR_INT64_C(0x0000000004000000)
+#define SVN_WC__ENTRY_MODIFY_HAS_PROP_MODS      APR_INT64_C(0x0000000008000000)
+#define SVN_WC__ENTRY_MODIFY_CACHABLE_PROPS     APR_INT64_C(0x0000000010000000)
+#define SVN_WC__ENTRY_MODIFY_PRESENT_PROPS      APR_INT64_C(0x0000000020000000)
+#define SVN_WC__ENTRY_MODIFY_CHANGELIST         APR_INT64_C(0x0000000040000000)
+#define SVN_WC__ENTRY_MODIFY_KEEP_LOCAL         APR_INT64_C(0x0000000080000000)
+#define SVN_WC__ENTRY_MODIFY_WORKING_SIZE       APR_INT64_C(0x0000000100000000)
+/* No #define for DEPTH, because it's only meaningful on this-dir anyway. */
 
 /* ...ORed together with this to mean "I really mean this, don't be
    trying to protect me from myself on this one." */
-#define SVN_WC__ENTRY_MODIFY_FORCE         0x80000000
+#define SVN_WC__ENTRY_MODIFY_FORCE              APR_INT64_C(0x4000000000000000)
 
 
-/* Modify an entry for NAME in access baton PATH by folding ("merging")
-   in changes, and sync those changes to disk.  New values for the
-   entry are pulled from their respective fields in ENTRY, and
+/* Modify an entry for NAME in access baton ADM_ACCESS by folding in
+   ("merging") changes, and sync those changes to disk.  New values
+   for the entry are pulled from their respective fields in ENTRY, and
    MODIFY_FLAGS is a bitmask to specify which of those field to pay
    attention to.  ADM_ACCESS must hold a write lock.
 
-   - ENTRY->kind specifies the node kind for this entry, and is
-     *required* to be set to one of the following valid values:
-     'svn_node_dir', 'svn_node_file'.
-
-   - NAME can be NULL to specify that the caller wishes to modify the
-     "this dir" entry in PATH.
+   NAME can be NULL to specify that the caller wishes to modify the
+   "this dir" entry in ADM_ACCESS.
 
    If DO_SYNC is FALSE then the modification will be entirely local to the
    access baton, if DO_SYNC is TRUE the modification will be written to
-   the entries file.  Be careful when setting DO_SYNC to FALSE, if there
+   the entries file.  Be careful when setting DO_SYNC to FALSE: if there
    is no subsequent svn_wc__entries_write call the modifications will be
    lost when the access baton is closed.
 
@@ -173,15 +185,15 @@ svn_error_t *svn_wc__atts_to_entry (svn_wc_entry_t **new_entry,
 
    NOTE: when you call this function, the entries file will be read,
    tweaked and finally, if DO_SYNC is TRUE, written back out.  */
-svn_error_t *svn_wc__entry_modify (svn_wc_adm_access_t *adm_access,
-                                   const char *name,
-                                   svn_wc_entry_t *entry,
-                                   apr_uint32_t modify_flags,
-                                   svn_boolean_t do_sync,
-                                   apr_pool_t *pool);
+svn_error_t *svn_wc__entry_modify(svn_wc_adm_access_t *adm_access,
+                                  const char *name,
+                                  svn_wc_entry_t *entry,
+                                  apr_uint64_t modify_flags,
+                                  svn_boolean_t do_sync,
+                                  apr_pool_t *pool);
 
 /* Remove entry NAME from ENTRIES, unconditionally. */
-void svn_wc__entry_remove (apr_hash_t *entries, const char *name);
+void svn_wc__entry_remove(apr_hash_t *entries, const char *name);
 
 
 /* Tweak the entry NAME within hash ENTRIES.  If NEW_URL is non-null,
@@ -198,17 +210,17 @@ void svn_wc__entry_remove (apr_hash_t *entries, const char *name);
  * require the entries to be written to disk, otherwise *WRITE_REQUIRED
  * will not be altered.
  *
- * (Intended as a helper to svn_wc__do_update_cleanup, which see.) 
+ * (Intended as a helper to svn_wc__do_update_cleanup, which see.)
  */
 svn_error_t *
-svn_wc__tweak_entry (apr_hash_t *entries,
-                     const char *name,
-                     const char *new_url,
-                     const char *repos,
-                     svn_revnum_t new_rev,
-                     svn_boolean_t allow_removal,
-                     svn_boolean_t *write_required,
-                     apr_pool_t *pool);
+svn_wc__tweak_entry(apr_hash_t *entries,
+                    const char *name,
+                    const char *new_url,
+                    const char *repos,
+                    svn_revnum_t new_rev,
+                    svn_boolean_t allow_removal,
+                    svn_boolean_t *write_required,
+                    apr_pool_t *pool);
 
 
 #ifdef __cplusplus
