@@ -2,7 +2,7 @@
  * translate.c :  wc-specific eol/keyword substitution
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -21,7 +21,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include <apr_general.h>  /* for strcasecmp() */
 #include <apr_pools.h>
 #include <apr_file_io.h>
 #include <apr_strings.h>
@@ -32,7 +31,6 @@
 #include "svn_subst.h"
 #include "svn_io.h"
 #include "svn_props.h"
-#include "svn_wc.h"
 
 #include "wc.h"
 #include "adm_files.h"
@@ -40,6 +38,79 @@
 #include "props.h"
 
 #include "svn_private_config.h"
+#include "private/svn_wc_private.h"
+
+
+
+static svn_error_t *
+read_handler_unsupported(void *baton, char *buffer, apr_size_t *len)
+{
+  abort();
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *
+write_handler_unsupported(void *baton, const char *buffer, apr_size_t *len)
+{
+  abort();
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_wc_translated_stream(svn_stream_t **stream,
+                         const char *path,
+                         const char *versioned_file,
+                         svn_wc_adm_access_t *adm_access,
+                         apr_uint32_t flags,
+                         apr_pool_t *pool)
+{
+  svn_subst_eol_style_t style;
+  const char *eol;
+  apr_hash_t *keywords;
+  svn_boolean_t special;
+  svn_boolean_t to_nf = flags & SVN_WC_TRANSLATE_TO_NF;
+
+  SVN_ERR(svn_wc__get_eol_style(&style, &eol, versioned_file,
+                                adm_access, pool));
+  SVN_ERR(svn_wc__get_keywords(&keywords, versioned_file,
+                               adm_access, NULL, pool));
+  SVN_ERR(svn_wc__get_special(&special, versioned_file, adm_access, pool));
+
+  if (special)
+    SVN_ERR(svn_subst_stream_from_specialfile(stream, path, pool));
+  else
+    {
+      apr_file_t *file;
+      svn_boolean_t repair_forced = flags & SVN_WC_TRANSLATE_FORCE_EOL_REPAIR;
+
+      SVN_ERR(svn_io_file_open(&file, path,
+                               to_nf ? (APR_READ | APR_BUFFERED)
+                               : (APR_CREATE | APR_WRITE | APR_BUFFERED),
+                               APR_OS_DEFAULT, pool));
+
+      *stream = svn_stream_from_aprfile2(file, FALSE, pool);
+
+      if (svn_subst_translation_required(style, eol, keywords, special, TRUE))
+        {
+          if (to_nf)
+            SVN_ERR(svn_subst_stream_translated_to_normal_form
+                    (stream, *stream, style, eol, repair_forced,
+                     keywords, pool));
+          else
+            *stream = svn_subst_stream_translated
+              (*stream, eol, TRUE, keywords, TRUE, pool);
+        }
+    }
+
+  /* Enfore our contract, because a specialfile stream won't */
+  if (to_nf)
+    svn_stream_set_write(*stream, write_handler_unsupported);
+  else
+    svn_stream_set_read(*stream, read_handler_unsupported);
+
+  return SVN_NO_ERROR;
+}
+
 
 svn_error_t *
 svn_wc_translated_file2(const char **xlated_path,
@@ -180,7 +251,7 @@ svn_wc__get_keywords(apr_hash_t **keywords,
 
       SVN_ERR(svn_wc_prop_get(&propval, SVN_PROP_KEYWORDS, path, adm_access,
                               pool));
-      
+
       list = propval ? propval->data : NULL;
     }
   else
@@ -193,11 +264,7 @@ svn_wc__get_keywords(apr_hash_t **keywords,
       return SVN_NO_ERROR;
     }
 
-  SVN_ERR(svn_wc_entry(&entry, path, adm_access, FALSE, pool));
-  if (! entry)
-    return svn_error_createf(SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                             _("'%s' is not under version control"),
-                             svn_path_local_style(path, pool));
+  SVN_ERR(svn_wc__entry_versioned(&entry, path, adm_access, FALSE, pool));
 
   SVN_ERR(svn_subst_build_keywords2(keywords,
                                     list,
@@ -222,7 +289,7 @@ svn_wc__get_special(svn_boolean_t *special,
                     apr_pool_t *pool)
 {
   const svn_string_t *propval;
-  
+
   /* Get the property value. */
   SVN_ERR(svn_wc_prop_get(&propval, SVN_PROP_SPECIAL, path,
                           adm_access, pool));
@@ -271,7 +338,7 @@ svn_wc__maybe_set_read_only(svn_boolean_t *did_set,
   if (entry && entry->lock_token)
     return SVN_NO_ERROR;
 
-  SVN_ERR(svn_wc_prop_get(&needs_lock, SVN_PROP_NEEDS_LOCK, path, 
+  SVN_ERR(svn_wc_prop_get(&needs_lock, SVN_PROP_NEEDS_LOCK, path,
                           adm_access, pool));
 
   if (needs_lock != NULL)
