@@ -2,7 +2,7 @@
  * liveprops.c: mod_dav_svn live property provider functions for Subversion
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -16,63 +16,65 @@
  * ====================================================================
  */
 
-
-
-#include <httpd.h>
-#include <util_xml.h>
 #include <apr_tables.h>
 #include <apr_md5.h>
-#include <mod_dav.h>
 
-#include "dav_svn.h"
+#include <httpd.h>
+#include <http_core.h>
+#include <util_xml.h>
+#include <mod_dav.h>
 
 #include "svn_pools.h"
 #include "svn_time.h"
 #include "svn_dav.h"
 #include "svn_md5.h"
+#include "svn_props.h"
 
-/* Private header included just for 1.0.7 revprops security fix. */
-#include "../libsvn_repos/repos.h"
+#include "private/svn_dav_protocol.h"
+
+#include "dav_svn.h"
 
 
 /*
 ** The namespace URIs that we use. This list and the enumeration must
 ** stay in sync.
 */
-static const char * const dav_svn_namespace_uris[] =
+static const char * const namespace_uris[] =
 {
-    "DAV:",
-    SVN_DAV_PROP_NS_DAV,
+  "DAV:",
+  SVN_DAV_PROP_NS_DAV,
 
-    NULL        /* sentinel */
+  NULL        /* sentinel */
 };
 enum {
-    DAV_SVN_NAMESPACE_URI_DAV,  /* the DAV: namespace URI */
-    DAV_SVN_NAMESPACE_URI       /* the dav<->ra_dav namespace URI */
+  NAMESPACE_URI_DAV,  /* the DAV: namespace URI */
+  NAMESPACE_URI       /* the dav<->ra_dav namespace URI */
 };
 
 #define SVN_RO_DAV_PROP(name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 0 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 0 }
 #define SVN_RW_DAV_PROP(name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 1 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##name, 1 }
 #define SVN_RO_DAV_PROP2(sym,name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 0 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 0 }
 #define SVN_RW_DAV_PROP2(sym,name) \
-        { DAV_SVN_NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 1 }
+        { NAMESPACE_URI_DAV, #name, DAV_PROPID_##sym, 1 }
 
 #define SVN_RO_SVN_PROP(sym,name) \
-        { DAV_SVN_NAMESPACE_URI, #name, SVN_PROPID_##sym, 0 }
+        { NAMESPACE_URI, #name, SVN_PROPID_##sym, 0 }
 #define SVN_RW_SVN_PROP(sym,name) \
-        { DAV_SVN_NAMESPACE_URI, #name, SVN_PROPID_##sym, 1 }
+        { NAMESPACE_URI, #name, SVN_PROPID_##sym, 1 }
 
 
 enum {
   SVN_PROPID_baseline_relative_path = 1,
   SVN_PROPID_md5_checksum,
-  SVN_PROPID_repository_uuid
+  SVN_PROPID_repository_uuid,
+  SVN_PROPID_deadprop_count
 };
 
-static const dav_liveprop_spec dav_svn_props[] =
+
+static const dav_liveprop_spec props[] =
 {
   /* ### don't worry about these for a bit */
 #if 0
@@ -98,21 +100,174 @@ static const dav_liveprop_spec dav_svn_props[] =
   SVN_RO_SVN_PROP(baseline_relative_path, baseline-relative-path),
   SVN_RO_SVN_PROP(md5_checksum, md5-checksum),
   SVN_RO_SVN_PROP(repository_uuid, repository-uuid),
+  SVN_RO_SVN_PROP(deadprop_count, deadprop-count),
 
   { 0 } /* sentinel */
 };
 
-static const dav_liveprop_group dav_svn_liveprop_group =
+
+/* Set *PROPVAL to the value for the revision property PROPNAME on
+   COMMITTED_REV, in the repository identified by RESOURCE, if
+   RESOURCE's path is readable.  If it is not readable, set *PROPVAL
+   to NULL and return SVN_NO_ERROR.  Use POOL for temporary
+   allocations and the allocation of *PROPVAL.
+
+   Note that this function does not check the readability of the
+   revision property, but the readability of a path.  The true
+   readability of a revision property is determined by investigating
+   the readability of all changed paths in the revision.  For certain
+   revision properties (e.g. svn:author and svn:date) to be readable,
+   it is enough if at least one changed path is readable.  When we
+   already have a changed path, we can skip the check for the other
+   changed paths in the revision and save a lot of work.  This means
+   that we will make a mistake when our path is unreadable and another
+   changed path is readable, but we will at least only hide too much
+   and not leak any protected properties.
+
+   WARNING: This method of only checking the readability of a path is
+   only valid to get revision properties for which it is enough if at
+   least one changed path is readable.  Using this function to get
+   revision properties for which all changed paths must be readable
+   might leak protected information because we will only test the
+   readability of a single changed path.
+*/
+static svn_error_t *
+get_path_revprop(svn_string_t **propval,
+                 const dav_resource *resource,
+                 svn_revnum_t committed_rev,
+                 const char *propname,
+                 apr_pool_t *pool)
 {
-    dav_svn_props,
-    dav_svn_namespace_uris,
-    &dav_svn_hooks_liveprop
+  *propval = NULL;
+
+  if (! dav_svn__allow_read(resource, committed_rev, pool))
+    return SVN_NO_ERROR;
+
+  /* Get the property of the created revision. The authz is already
+     performed, so we don't need to do it here too. */
+  return svn_repos_fs_revision_prop(propval,
+                                    resource->info->repos->repos,
+                                    committed_rev,
+                                    propname,
+                                    NULL, NULL, pool);
+}
+
+
+enum time_format {
+  time_format_iso8601,
+  time_format_rfc1123
 };
 
 
-static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
-                                           int propid, dav_prop_insert what,
-                                           apr_text_header *phdr)
+/* Given a mod_dav_svn @a resource, set @a *timeval and @a *datestring
+   to the last-modified-time of the resource.  The datestring will be
+   formatted according to @a format.  Use @a pool for both
+   scratchwork, and to allocate @a *datestring.
+
+   If @a timeval or @a datestring is NULL, don't touch it.
+
+   Return zero on success, non-zero if an error occurs. */
+static int
+get_last_modified_time(const char **datestring,
+                       apr_time_t *timeval,
+                       const dav_resource *resource,
+                       enum time_format format,
+                       apr_pool_t *pool)
+{
+  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
+  svn_string_t *committed_date = NULL;
+  svn_error_t *serr;
+  apr_time_t timeval_tmp;
+
+  if ((datestring == NULL) && (timeval == NULL))
+    return 0;
+
+  if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      /* A baseline URI. */
+      committed_rev = resource->info->root.rev;
+    }
+  else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
+           || resource->type == DAV_RESOURCE_TYPE_WORKING
+           || resource->type == DAV_RESOURCE_TYPE_VERSION)
+    {
+      serr = svn_fs_node_created_rev(&committed_rev,
+                                     resource->info->root.root,
+                                     resource->info->repos_path, pool);
+      if (serr != NULL)
+        {
+          svn_error_clear(serr);
+          return 1;
+        }
+    }
+  else
+    {
+      /* unsupported resource kind -- has no mod-time */
+      return 1;
+    }
+
+  serr = get_path_revprop(&committed_date,
+                          resource,
+                          committed_rev,
+                          SVN_PROP_REVISION_DATE,
+                          pool);
+  if (serr)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (committed_date == NULL)
+    return 1;
+
+  /* return the ISO8601 date as an apr_time_t */
+  serr = svn_time_from_cstring(&timeval_tmp, committed_date->data, pool);
+  if (serr != NULL)
+    {
+      svn_error_clear(serr);
+      return 1;
+    }
+
+  if (timeval)
+    memcpy(timeval, &timeval_tmp, sizeof(*timeval));
+
+  if (! datestring)
+    return 0;
+
+  if (format == time_format_iso8601)
+    {
+      *datestring = committed_date->data;
+    }
+  else if (format == time_format_rfc1123)
+    {
+      apr_time_exp_t tms;
+      apr_status_t status;
+
+      /* convert the apr_time_t into an apr_time_exp_t */
+      status = apr_time_exp_gmt(&tms, timeval_tmp);
+      if (status != APR_SUCCESS)
+        return 1;
+
+      /* stolen from dav/fs/repos.c   :-)  */
+      *datestring = apr_psprintf(pool, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
+                                 apr_day_snames[tms.tm_wday],
+                                 tms.tm_mday, apr_month_snames[tms.tm_mon],
+                                 tms.tm_year + 1900,
+                                 tms.tm_hour, tms.tm_min, tms.tm_sec);
+    }
+  else /* unknown time format */
+    {
+      return 1;
+    }
+
+  return 0;
+}
+
+static dav_prop_insert
+insert_prop(const dav_resource *resource,
+            int propid,
+            dav_prop_insert what,
+            apr_text_header *phdr)
 {
   const char *value = NULL;
   const char *s;
@@ -120,24 +275,24 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   apr_pool_t *p = resource->info->pool;
   const dav_liveprop_spec *info;
   int global_ns;
-  dav_svn_authz_read_baton arb;
   svn_error_t *serr;
 
   /*
-  ** None of SVN provider properties are defined if the resource does not
-  ** exist. Just bail for this case.
+  ** Almost none of the SVN provider properties are defined if the
+  ** resource does not exist.  We do need to return the one VCC
+  ** property and baseline-relative-path on lock-null resources,
+  ** however, so that svn clients can run 'svn unlock' and 'svn info'
+  ** on these things.
   **
   ** Even though we state that the SVN properties are not defined, the
   ** client cannot store dead values -- we deny that thru the is_writable
   ** hook function.
   */
-  if (!resource->exists)
+  if ((! resource->exists)
+      && (propid != DAV_PROPID_version_controlled_configuration)
+      && (propid != SVN_PROPID_baseline_relative_path))
     return DAV_PROP_INSERT_NOTSUPP;
 
-  /* Build a baton for path-based authz function (dav_svn_authz_read) */
-  arb.r = resource->info->r;
-  arb.repos = resource->info->repos;
-
   /* ### we may want to respond to DAV_PROPID_resourcetype for PRIVATE
      ### resources. need to think on "proper" interaction with mod_dav */
 
@@ -157,7 +312,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
            as 'last modified date'?  */
         const char *datestring;
         apr_time_t timeval;
-        enum dav_svn_time_format format;
+        enum time_format format;
 
         /* ### for now, our global VCC has no such property. */
         if (resource->type == DAV_RESOURCE_TYPE_PRIVATE
@@ -165,20 +320,20 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
-       
+
         if (propid == DAV_PROPID_creationdate)
           {
             /* Return an ISO8601 date; this is what the svn client
                expects, and rfc2518 demands it. */
-            format = dav_svn_time_format_iso8601;
+            format = time_format_iso8601;
           }
         else /* propid == DAV_PROPID_getlastmodified */
           {
-            format = dav_svn_time_format_rfc1123;
+            format = time_format_rfc1123;
           }
 
-        if (0 != dav_svn_get_last_modified_time (&datestring, &timeval,
-                                                 resource, format, p))
+        if (0 != get_last_modified_time(&datestring, &timeval,
+                                        resource, format, p))
           {
             return DAV_PROP_INSERT_NOTDEF;
           }
@@ -188,7 +343,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       }
 
     case DAV_PROPID_creator_displayname:
-      {        
+      {
         svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
         svn_string_t *last_author = NULL;
 
@@ -216,24 +371,25 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             if (serr != NULL)
               {
                 /* ### what to do? */
+                svn_error_clear(serr);
                 value = "###error###";
                 break;
               }
-          }        
+          }
         else
           {
             return DAV_PROP_INSERT_NOTSUPP;
           }
-        
-        /* Get the date property of the created revision. */
-        serr = svn_repos__fs_revision_prop(&last_author,
-                                           resource->info->repos->repos,
-                                           committed_rev,
-                                           SVN_PROP_REVISION_AUTHOR,
-                                           dav_svn_authz_read, &arb, p);
-        if (serr != NULL)
+
+        serr = get_path_revprop(&last_author,
+                                resource,
+                                committed_rev,
+                                SVN_PROP_REVISION_AUTHOR,
+                                p);
+        if (serr)
           {
             /* ### what to do? */
+            svn_error_clear(serr);
             value = "###error###";
             break;
           }
@@ -253,7 +409,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
     case DAV_PROPID_getcontentlength:
       {
         svn_filesize_t len = 0;
-        
+
         /* our property, but not defined on collection resources */
         if (resource->collection || resource->baselined)
           return DAV_PROP_INSERT_NOTSUPP;
@@ -262,6 +418,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
                                   resource->info->repos_path, p);
         if (serr != NULL)
           {
+            svn_error_clear(serr);
             value = "0";  /* ### what to do? */
             break;
           }
@@ -276,6 +433,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
            svn:mime-type property is of type text/plain.  So it seems
            safe (and consistent) to assume the same on the server.  */
         svn_string_t *pval;
+        const char *mime_type = NULL;
 
         if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
           return DAV_PROP_INSERT_NOTSUPP;
@@ -286,39 +444,43 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
             return DAV_PROP_INSERT_NOTSUPP;
           }
 
-        serr = svn_fs_node_prop (&pval, resource->info->root.root,
-                                 resource->info->repos_path,
-                                 SVN_PROP_MIME_TYPE, p);
-
-        if ((serr != NULL) || (pval == NULL))
+        if (resource->collection) /* defaults for directories */
           {
-            if (resource->collection) /* defaults for directories */
-              {
-                if (resource->info->repos->xslt_uri)
-                  value = "text/xml";
-                else
-                  value = "text/html";
-              }
+            if (resource->info->repos->xslt_uri)
+              mime_type = "text/xml";
             else
-              {
-                value = "text/plain"; /* default for file */
-              }
-          }            
+              mime_type = "text/html; charset=UTF-8";
+          }
         else
           {
-            serr = svn_mime_type_validate (pval->data, p);
-            if (serr)
+            if ((serr = svn_fs_node_prop(&pval, resource->info->root.root,
+                                         resource->info->repos_path,
+                                         SVN_PROP_MIME_TYPE, p)))
+              {
+                svn_error_clear(serr);
+                pval = NULL;
+              }
+
+            if (pval)
+              mime_type = pval->data;
+            else if ((! resource->info->repos->is_svn_client)
+                     && resource->info->r->content_type)
+              mime_type = resource->info->r->content_type;
+            else
+              mime_type = ap_default_type(resource->info->r);
+
+            if ((serr = svn_mime_type_validate(mime_type, p)))
               {
                 /* Probably serr->apr == SVN_ERR_BAD_MIME_TYPE, but
                    there's no point even checking.  No matter what the
                    error is, we can't claim to have a mime type for
                    this resource. */
-                  return DAV_PROP_INSERT_NOTDEF;
+                svn_error_clear(serr);
+                return DAV_PROP_INSERT_NOTDEF;
               }
-            else
-              value = pval->data;
           }
 
+        value = mime_type;
         break;
       }
 
@@ -329,7 +491,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           return DAV_PROP_INSERT_NOTSUPP;
         }
 
-      value = dav_svn_getetag(resource, p);
+      value = dav_svn__getetag(resource, p);
       break;
 
     case DAV_PROPID_auto_version:
@@ -347,9 +509,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       /* ### whoops. also defined for a VCC. deal with it later. */
       if (resource->type != DAV_RESOURCE_TYPE_VERSION || !resource->baselined)
         return DAV_PROP_INSERT_NOTSUPP;
-      value = dav_svn_build_uri(resource->info->repos, DAV_SVN_BUILD_URI_BC,
-                                resource->info->root.rev, NULL,
-                                1 /* add_href */, p);
+      value = dav_svn__build_uri(resource->info->repos, DAV_SVN__BUILD_URI_BC,
+                                 resource->info->root.rev, NULL,
+                                 1 /* add_href */, p);
       break;
 
     case DAV_PROPID_checked_in:
@@ -364,13 +526,14 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
-          s = dav_svn_build_uri(resource->info->repos,
-                                DAV_SVN_BUILD_URI_BASELINE, 
-                                revnum, NULL, 0 /* add_href */, p);
-          value = apr_psprintf(p, "<D:href>%s</D:href>", 
+          s = dav_svn__build_uri(resource->info->repos,
+                                 DAV_SVN__BUILD_URI_BASELINE,
+                                 revnum, NULL, 0 /* add_href */, p);
+          value = apr_psprintf(p, "<D:href>%s</D:href>",
                                apr_xml_quote_string(p, s, 1));
         }
       else if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
@@ -381,14 +544,14 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       else
         {
           svn_revnum_t rev_to_use =
-            dav_svn_get_safe_cr(resource->info->root.root,
-                                resource->info->repos_path, p);
+            dav_svn__get_safe_cr(resource->info->root.root,
+                                 resource->info->repos_path, p);
 
-          s = dav_svn_build_uri(resource->info->repos,
-                                DAV_SVN_BUILD_URI_VERSION,
-                                rev_to_use, resource->info->repos_path,
+          s = dav_svn__build_uri(resource->info->repos,
+                                 DAV_SVN__BUILD_URI_VERSION,
+                                 rev_to_use, resource->info->repos_path,
                                 0 /* add_href */, p);
-          value = apr_psprintf(p, "<D:href>%s</D:href>", 
+          value = apr_psprintf(p, "<D:href>%s</D:href>",
                                apr_xml_quote_string(p, s, 1));
         }
       break;
@@ -399,9 +562,9 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       /* ### note that a VCC (a special VCR) is defined as _PRIVATE for now */
       if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
         return DAV_PROP_INSERT_NOTSUPP;
-      value = dav_svn_build_uri(resource->info->repos, DAV_SVN_BUILD_URI_VCC,
-                                SVN_IGNORED_REVNUM, NULL, 
-                                1 /* add_href */, p);
+      value = dav_svn__build_uri(resource->info->repos, DAV_SVN__BUILD_URI_VCC,
+                                 SVN_IGNORED_REVNUM, NULL,
+                                 1 /* add_href */, p);
       break;
 
     case DAV_PROPID_version_name:
@@ -420,13 +583,13 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       if (resource->baselined)
         {
           /* just the revision number for baselines */
-          value = apr_psprintf(p, "%" SVN_REVNUM_T_FMT,
+          value = apr_psprintf(p, "%ld",
                                resource->info->root.rev);
         }
       else
         {
           svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-          
+
           /* Get the CR field out of the node's skel.  Notice that the
              root object might be an ID root -or- a revision root. */
           serr = svn_fs_node_created_rev(&committed_rev,
@@ -435,12 +598,13 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
-          
+
           /* Convert the revision into a quoted string */
-          s = apr_psprintf(p, "%" SVN_REVNUM_T_FMT, committed_rev);
+          s = apr_psprintf(p, "%ld", committed_rev);
           value = apr_xml_quote_string(p, s, 1);
         }
       break;
@@ -472,11 +636,12 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
           if (serr != NULL)
             {
               /* ### what to do? */
+              svn_error_clear(serr);
               value = "###error###";
               break;
             }
 
-          value = svn_md5_digest_to_cstring (digest, p);
+          value = svn_md5_digest_to_cstring(digest, p);
 
           if (! value)
             return DAV_PROP_INSERT_NOTSUPP;
@@ -491,11 +656,36 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
       if (serr != NULL)
         {
           /* ### what to do? */
+          svn_error_clear(serr);
           value = "###error###";
           break;
         }
       break;
 
+    case SVN_PROPID_deadprop_count:
+      {
+        unsigned int propcount;
+        apr_hash_t *proplist;
+
+        if (resource->type != DAV_RESOURCE_TYPE_REGULAR)
+          return DAV_PROP_INSERT_NOTSUPP;
+
+        serr = svn_fs_node_proplist(&proplist,
+                                    resource->info->root.root,
+                                    resource->info->repos_path, p);
+        if (serr != NULL)
+          {
+            /* ### what to do? */
+            svn_error_clear(serr);
+            value = "###error###";
+            break;
+          }
+
+        propcount = apr_hash_count(proplist);
+        value = apr_psprintf(p, "%u", propcount);
+        break;
+      }
+
     default:
       /* ### what the heck was this property? */
       return DAV_PROP_INSERT_NOTDEF;
@@ -504,7 +694,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   /* assert: value != NULL */
 
   /* get the information and global NS index for the property */
-  global_ns = dav_get_liveprop_info(propid, &dav_svn_liveprop_group, &info);
+  global_ns = dav_get_liveprop_info(propid, &dav_svn__liveprop_group, &info);
 
   /* assert: info != NULL && info->name != NULL */
 
@@ -522,7 +712,7 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
     s = apr_psprintf(response_pool,
                      "<D:supported-live-property D:name=\"%s\" "
                      "D:namespace=\"%s\"/>" DEBUG_CR,
-                     info->name, dav_svn_namespace_uris[info->ns]);
+                     info->name, namespace_uris[info->ns]);
   }
   apr_text_append(response_pool, phdr, s);
 
@@ -530,62 +720,86 @@ static dav_prop_insert dav_svn_insert_prop(const dav_resource *resource,
   return what;
 }
 
-static int dav_svn_is_writable(const dav_resource *resource, int propid)
+
+static int
+is_writable(const dav_resource *resource, int propid)
 {
   const dav_liveprop_spec *info;
 
-  (void) dav_get_liveprop_info(propid, &dav_svn_liveprop_group, &info);
+  (void) dav_get_liveprop_info(propid, &dav_svn__liveprop_group, &info);
   return info->is_writable;
 }
 
-static dav_error * dav_svn_patch_validate(const dav_resource *resource,
-                                          const apr_xml_elem *elem,
-                                          int operation, void **context,
-                                          int *defer_to_dead)
+
+static dav_error *
+patch_validate(const dav_resource *resource,
+               const apr_xml_elem *elem,
+               int operation,
+               void **context,
+               int *defer_to_dead)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-static dav_error * dav_svn_patch_exec(const dav_resource *resource,
-                                      const apr_xml_elem *elem,
-                                      int operation, void *context,
-                                      dav_liveprop_rollback **rollback_ctx)
+
+static dav_error *
+patch_exec(const dav_resource *resource,
+           const apr_xml_elem *elem,
+           int operation,
+           void *context,
+           dav_liveprop_rollback **rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-static void dav_svn_patch_commit(const dav_resource *resource,
-                                 int operation, void *context,
-                                 dav_liveprop_rollback *rollback_ctx)
+
+static void
+patch_commit(const dav_resource *resource,
+             int operation,
+             void *context,
+             dav_liveprop_rollback *rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
 }
 
-static dav_error * dav_svn_patch_rollback(const dav_resource *resource,
-                                          int operation, void *context,
-                                          dav_liveprop_rollback *rollback_ctx)
+static dav_error *
+patch_rollback(const dav_resource *resource,
+                       int operation,
+                       void *context,
+                       dav_liveprop_rollback *rollback_ctx)
 {
   /* NOTE: this function will not be called unless/until we have
      modifiable (writable) live properties. */
   return NULL;
 }
 
-const dav_hooks_liveprop dav_svn_hooks_liveprop = {
-  dav_svn_insert_prop,
-  dav_svn_is_writable,
-  dav_svn_namespace_uris,
-  dav_svn_patch_validate,
-  dav_svn_patch_exec,
-  dav_svn_patch_commit,
-  dav_svn_patch_rollback,
+
+static const dav_hooks_liveprop hooks_liveprop = {
+  insert_prop,
+  is_writable,
+  namespace_uris,
+  patch_validate,
+  patch_exec,
+  patch_commit,
+  patch_rollback,
 };
 
-void dav_svn_gather_propsets(apr_array_header_t *uris)
+
+const dav_liveprop_group dav_svn__liveprop_group =
+{
+  props,
+  namespace_uris,
+  &hooks_liveprop
+};
+
+
+void
+dav_svn__gather_propsets(apr_array_header_t *uris)
 {
   /* ### what should we use for a URL to describe the available prop set? */
   /* ### for now... nothing. we will *only* have DAV properties */
@@ -595,149 +809,58 @@ void dav_svn_gather_propsets(apr_array_header_t *uris)
 #endif
 }
 
-int dav_svn_find_liveprop(const dav_resource *resource,
-                          const char *ns_uri, const char *name,
-                          const dav_hooks_liveprop **hooks)
+
+int
+dav_svn__find_liveprop(const dav_resource *resource,
+                       const char *ns_uri,
+                       const char *name,
+                       const dav_hooks_liveprop **hooks)
 {
   /* don't try to find any liveprops if this isn't "our" resource */
-  if (resource->hooks != &dav_svn_hooks_repos)
+  if (resource->hooks != &dav_svn__hooks_repository)
     return 0;
 
-  return dav_do_find_liveprop(ns_uri, name, &dav_svn_liveprop_group, hooks);
-}
-
-void dav_svn_insert_all_liveprops(request_rec *r, const dav_resource *resource,
-                                  dav_prop_insert what, apr_text_header *phdr)
-{
-    const dav_liveprop_spec *spec;
-    apr_pool_t *pool;
-    apr_pool_t *subpool;
-
-    /* don't insert any liveprops if this isn't "our" resource */
-    if (resource->hooks != &dav_svn_hooks_repos)
-        return;
-
-    if (!resource->exists) {
-        /* a lock-null resource */
-        /*
-        ** ### technically, we should insert empty properties. dunno offhand
-        ** ### what part of the spec said this, but it was essentially thus:
-        ** ### "the properties should be defined, but may have no value".
-        */
-        return;
-    }
-
-    pool = resource->info->pool;
-    subpool = svn_pool_create(pool);
-    resource->info->pool = subpool;
-
-    for (spec = dav_svn_props; spec->name != NULL; ++spec)
-      {
-        (void) dav_svn_insert_prop(resource, spec->propid, what, phdr);
-        svn_pool_clear(subpool);
-      }
-
-    resource->info->pool = pool;
-    svn_pool_destroy(subpool);
-
-    /* ### we know the others aren't defined as liveprops */
-}
-
-void dav_svn_register_uris(apr_pool_t *p)
-{
-    /* register the namespace URIs */
-    dav_register_liveprop_group(p, &dav_svn_liveprop_group);
+  return dav_do_find_liveprop(ns_uri, name, &dav_svn__liveprop_group, hooks);
 }
 
 
-int dav_svn_get_last_modified_time (const char **datestring,
-                                    apr_time_t *timeval,
-                                    const dav_resource *resource,
-                                    enum dav_svn_time_format format,
-                                    apr_pool_t *pool)
+void
+dav_svn__insert_all_liveprops(request_rec *r,
+                              const dav_resource *resource,
+                              dav_prop_insert what,
+                              apr_text_header *phdr)
 {
-  svn_revnum_t committed_rev = SVN_INVALID_REVNUM;
-  svn_string_t *committed_date = NULL;
-  svn_error_t *serr;
-  apr_time_t timeval_tmp;
-  dav_svn_authz_read_baton arb;
-  
-  arb.r = resource->info->r;
-  arb.repos = resource->info->repos;
+  const dav_liveprop_spec *spec;
+  apr_pool_t *pool;
+  apr_pool_t *subpool;
 
-  if ((datestring == NULL) && (timeval == NULL))
-    return 0;
+  /* don't insert any liveprops if this isn't "our" resource */
+  if (resource->hooks != &dav_svn__hooks_repository)
+    return;
 
-  if (resource->baselined && resource->type == DAV_RESOURCE_TYPE_VERSION)
+  if (!resource->exists)
     {
-      /* A baseline URI. */
-      committed_rev = resource->info->root.rev;
-    }
-  else if (resource->type == DAV_RESOURCE_TYPE_REGULAR
-           || resource->type == DAV_RESOURCE_TYPE_WORKING
-           || resource->type == DAV_RESOURCE_TYPE_VERSION)
-    {
-      serr = svn_fs_node_created_rev(&committed_rev,
-                                     resource->info->root.root,
-                                     resource->info->repos_path, pool);
-      if (serr != NULL)
-        return 1;
-    }
-  else
-    {
-      /* unsupported resource kind -- has no mod-time */
-      return 1;
+      /* a lock-null resource */
+      /*
+       ** ### technically, we should insert empty properties. dunno offhand
+       ** ### what part of the spec said this, but it was essentially thus:
+       ** ### "the properties should be defined, but may have no value".
+       */
+      return;
     }
 
-  /* Get the svn:date property of the CR */
-  serr = svn_repos__fs_revision_prop(&committed_date,
-                                     resource->info->repos->repos,
-                                     committed_rev,
-                                     SVN_PROP_REVISION_DATE,
-                                     dav_svn_authz_read, &arb,
-                                     pool);
-  if (serr != NULL)
-    return 1;
-  
-  if (committed_date == NULL)
-    return 1;
+  pool = resource->info->pool;
+  subpool = svn_pool_create(pool);
+  resource->info->pool = subpool;
 
-  /* return the ISO8601 date as an apr_time_t */
-  serr = svn_time_from_cstring(&timeval_tmp, committed_date->data, pool);
-  if (serr != NULL)
-    return 1;
-
-  if (timeval)
-    memcpy(timeval, &timeval_tmp, sizeof(*timeval));
-
-  if (! datestring)
-    return 0;
-
-  if (format == dav_svn_time_format_iso8601)
+  for (spec = props; spec->name != NULL; ++spec)
     {
-      *datestring = committed_date->data;
-    }
-  else if (format == dav_svn_time_format_rfc1123)
-    {
-      apr_time_exp_t tms;
-      apr_status_t status;
-      
-      /* convert the apr_time_t into an apr_time_exp_t */
-      status = apr_time_exp_gmt(&tms, timeval_tmp);
-      if (status != APR_SUCCESS)
-        return 1;
-              
-      /* stolen from dav/fs/repos.c   :-)  */
-      *datestring = apr_psprintf(pool, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT",
-                                 apr_day_snames[tms.tm_wday],
-                                 tms.tm_mday, apr_month_snames[tms.tm_mon],
-                                 tms.tm_year + 1900,
-                                 tms.tm_hour, tms.tm_min, tms.tm_sec);      
-    }
-  else /* unknown time format */
-    {
-      return 1;
+      svn_pool_clear(subpool);
+      (void) insert_prop(resource, spec->propid, what, phdr);
     }
 
-  return 0;
-}                                              
+  resource->info->pool = pool;
+  svn_pool_destroy(subpool);
+
+  /* ### we know the others aren't defined as liveprops */
+}
