@@ -2,7 +2,7 @@
  * copy-cmd.c -- Subversion copy command
  *
  * ====================================================================
- * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -40,42 +40,62 @@ svn_cl__copy(apr_getopt_t *os,
 {
   svn_cl__opt_state_t *opt_state = ((svn_cl__cmd_baton_t *) baton)->opt_state;
   svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
-  apr_array_header_t *targets;
+  apr_array_header_t *targets, *sources;
   const char *src_path, *dst_path;
-  svn_boolean_t src_is_url, dst_is_url;
+  svn_boolean_t srcs_are_urls, dst_is_url;
   svn_commit_info_t *commit_info = NULL;
   svn_error_t *err;
+  int i;
 
-  SVN_ERR(svn_opt_args_to_target_array2(&targets, os, 
-                                        opt_state->targets, pool));
+  SVN_ERR(svn_cl__args_to_target_array_print_reserved(&targets, os,
+                                                      opt_state->targets, 
+                                                      pool));
   if (targets->nelts < 2)
     return svn_error_create(SVN_ERR_CL_INSUFFICIENT_ARGS, 0, NULL);
-  if (targets->nelts > 2)
-    return svn_error_create(SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);
 
-  src_path = ((const char **) (targets->elts))[0];
-  dst_path = ((const char **) (targets->elts))[1];
+  /* Get the src list and associated peg revs */
+  sources = apr_array_make(pool, targets->nelts - 1,
+                           sizeof(svn_client_copy_source_t *));
+  for (i = 0; i < (targets->nelts - 1); i++)
+    {
+      const char *target = APR_ARRAY_IDX(targets, i, const char *);
+      svn_client_copy_source_t *source = apr_palloc(pool, sizeof(*source));
+      const char *src;
+      svn_opt_revision_t *peg_revision = apr_palloc(pool,
+                                                    sizeof(*peg_revision));
 
-  /* Figure out which type of trace editor to use. */
-  src_is_url = svn_path_is_url(src_path);
+      SVN_ERR(svn_opt_parse_path(peg_revision, &src, target, pool));
+      source->path = src;
+      source->revision = &(opt_state->start_revision);
+      source->peg_revision = peg_revision;
+
+      APR_ARRAY_PUSH(sources, svn_client_copy_source_t *) = source;
+    }
+
+  /* Figure out which type of trace editor to use.
+     If the src_paths are not homogeneous, setup_copy will return an error. */
+  src_path = APR_ARRAY_IDX(targets, 0, const char *);
+  srcs_are_urls = svn_path_is_url(src_path);
+  dst_path = APR_ARRAY_IDX(targets, targets->nelts - 1, const char *);
+  apr_array_pop(targets);
   dst_is_url = svn_path_is_url(dst_path);
 
-  if ((! src_is_url) && (! dst_is_url))
+  if ((! srcs_are_urls) && (! dst_is_url))
     {
       /* WC->WC */
       if (! opt_state->quiet)
         svn_cl__get_notifier(&ctx->notify_func2, &ctx->notify_baton2,
                              FALSE, FALSE, FALSE, pool);
     }
-  else if ((! src_is_url) && (dst_is_url))
+  else if ((! srcs_are_urls) && (dst_is_url))
     {
       /* WC->URL : Use notification. */
       /* ### todo:
-         
+
          We'd like to use the notifier, but we MAY have a couple of
          problems with that, the same problems that used to apply to
          the old trace_editor:
-         
+
          1) We don't know where the commit editor for this case will
             be anchored with respect to the repository, so we can't
             use the DST_URL.
@@ -85,53 +105,38 @@ svn_cl__copy(apr_getopt_t *os,
             basenames will be chosen for our committed things.  So a
             copy of dir1/foo.c to http://.../dir2/foo-copy-c would
             display like: "Adding   dir1/foo-copy.c", which could be a
-            bogus path. 
+            bogus path.
       */
     }
-  else if ((src_is_url) && (! dst_is_url))
+  else if ((srcs_are_urls) && (! dst_is_url))
     {
       /* URL->WC : Use checkout-style notification. */
       if (! opt_state->quiet)
         svn_cl__get_notifier(&ctx->notify_func2, &ctx->notify_baton2, TRUE,
                              FALSE, FALSE, pool);
     }
-  else
-    /* URL->URL : No notification needed. */
-    ;
+  /* else URL -> URL, meaning that no notification is needed. */
 
   if (! dst_is_url)
     {
-      ctx->log_msg_func2 = NULL;
-      if (opt_state->message || opt_state->filedata)
+      ctx->log_msg_func3 = NULL;
+      if (opt_state->message || opt_state->filedata || opt_state->revprop_table)
         return svn_error_create
           (SVN_ERR_CL_UNNECESSARY_LOG_MESSAGE, NULL,
-           _("Local, non-commit operations do not take a log message"));
+           _("Local, non-commit operations do not take a log message "
+             "or revision properties"));
     }
 
-  if (ctx->log_msg_func2)
-    SVN_ERR(svn_cl__make_log_msg_baton(&(ctx->log_msg_baton2), opt_state,
+  if (ctx->log_msg_func3)
+    SVN_ERR(svn_cl__make_log_msg_baton(&(ctx->log_msg_baton3), opt_state,
                                        NULL, ctx->config, pool));
 
-  err = svn_client_copy3(&commit_info, src_path,
-                         &(opt_state->start_revision),
-                         dst_path, ctx, pool);
+  err = svn_client_copy4(&commit_info, sources, dst_path, TRUE,
+                         opt_state->parents, opt_state->revprop_table,
+                         ctx, pool);
 
-  /* If dst_path already exists, try to copy src_path as a child of it. */
-  if (err && (err->apr_err == SVN_ERR_ENTRY_EXISTS
-              || err->apr_err == SVN_ERR_FS_ALREADY_EXISTS))
-    {
-      const char *src_basename = svn_path_basename(src_path, pool);
-
-      svn_error_clear(err);
-      
-      err = svn_client_copy3(&commit_info, src_path,
-                             &(opt_state->start_revision),
-                             svn_path_join(dst_path, src_basename, pool),
-                             ctx, pool);
-    }
-
-  if (ctx->log_msg_func2)
-    SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton2, err));
+  if (ctx->log_msg_func3)
+    SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton3, err));
   else if (err)
     return err;
 
