@@ -42,27 +42,16 @@ svn_cl__move(apr_getopt_t *os,
   svn_cl__opt_state_t *opt_state = ((svn_cl__cmd_baton_t *) baton)->opt_state;
   svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
   apr_array_header_t *targets;
-  const char *src_path, *dst_path;
+  const char *dst_path;
   svn_commit_info_t *commit_info = NULL;
   svn_error_t *err;
 
-  SVN_ERR(svn_opt_args_to_target_array2(&targets, os, 
-                                        opt_state->targets, pool));
+  SVN_ERR(svn_cl__args_to_target_array_print_reserved(&targets, os,
+                                                      opt_state->targets, 
+                                                      pool));
 
   if (targets->nelts < 2)
     return svn_error_create(SVN_ERR_CL_INSUFFICIENT_ARGS, 0, NULL);
-  if (targets->nelts > 2)
-    return svn_error_create(SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);
-
-  src_path = ((const char **) (targets->elts))[0];
-  dst_path = ((const char **) (targets->elts))[1];
-  
-  if (! opt_state->quiet)
-    svn_cl__get_notifier(&ctx->notify_func2, &ctx->notify_baton2, FALSE,
-                         FALSE, FALSE, pool);
-
-  SVN_ERR(svn_cl__make_log_msg_baton(&(ctx->log_msg_baton2), opt_state,
-                                     NULL, ctx->config, pool));
 
   if (opt_state->start_revision.kind != svn_opt_revision_unspecified
       && opt_state->start_revision.kind != svn_opt_revision_head)
@@ -72,26 +61,38 @@ svn_cl__move(apr_getopt_t *os,
          _("Cannot specify revisions (except HEAD) with move operations"));
     }
 
-  err = svn_client_move4(&commit_info, src_path, dst_path,
-                         opt_state->force, ctx, pool);
+  if (! opt_state->quiet)
+    svn_cl__get_notifier(&ctx->notify_func2, &ctx->notify_baton2, FALSE,
+                         FALSE, FALSE, pool);
 
-  /* If dst_path already exists, try to move src_path as a child of it. */
-  if (err && (err->apr_err == SVN_ERR_ENTRY_EXISTS
-              || err->apr_err == SVN_ERR_FS_ALREADY_EXISTS))
+  dst_path = APR_ARRAY_IDX(targets, targets->nelts - 1, const char *);
+  apr_array_pop(targets);
+
+  if (! svn_path_is_url(dst_path))
     {
-      const char *src_basename = svn_path_basename(src_path, pool);
-
-      svn_error_clear(err);
-      
-      err = svn_client_move4(&commit_info, src_path,
-                             svn_path_join(dst_path, src_basename, pool),
-                             opt_state->force, ctx, pool);
-
+      ctx->log_msg_func3 = NULL;
+      if (opt_state->message || opt_state->filedata || opt_state->revprop_table)
+        return svn_error_create
+          (SVN_ERR_CL_UNNECESSARY_LOG_MESSAGE, NULL,
+           _("Local, non-commit operations do not take a log message "
+             "or revision properties"));
     }
 
+  if (ctx->log_msg_func3)
+    SVN_ERR(svn_cl__make_log_msg_baton(&(ctx->log_msg_baton3), opt_state,
+                                       NULL, ctx->config, pool));
+
+  err = svn_client_move5(&commit_info, targets, dst_path, opt_state->force,
+                         TRUE, opt_state->parents, opt_state->revprop_table,
+                         ctx, pool);
+
   if (err)
     err = svn_cl__may_need_force(err);
-  SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton2, err));
+
+  if (ctx->log_msg_func3)
+    SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton3, err));
+  else if (err)
+    return err;
 
   if (commit_info && ! opt_state->quiet)
     SVN_ERR(svn_cl__print_commit_info(commit_info, pool));
