@@ -2,7 +2,7 @@
  * main.c :  Main control function for svnserve
  *
  * ====================================================================
- * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -40,6 +40,7 @@
 #include "svn_repos.h"
 #include "svn_fs.h"
 #include "svn_version.h"
+#include "svn_io.h"
 
 #include "svn_private_config.h"
 #include "winservice.h"
@@ -134,6 +135,7 @@ void winservice_notify_stop(void)
 #define SVNSERVE_OPT_VERSION     260
 #define SVNSERVE_OPT_PID_FILE    261
 #define SVNSERVE_OPT_SERVICE     262
+#define SVNSERVE_OPT_CONFIG_FILE 263
 
 static const apr_getopt_option_t svnserve__options[] =
   {
@@ -158,8 +160,10 @@ static const apr_getopt_option_t svnserve__options[] =
     {"threads",          'T', 0, N_("use threads instead of fork")},
 #endif
     {"listen-once",      'X', 0, N_("listen once (useful for debugging)")},
+    {"config-file",      SVNSERVE_OPT_CONFIG_FILE, 1,
+     N_("read configuration from file ARG")},
     {"pid-file",         SVNSERVE_OPT_PID_FILE, 1,
-     N_("write server process ID to file arg")},
+     N_("write server process ID to file ARG")},
 #ifdef WIN32
     {"service",          SVNSERVE_OPT_SERVICE, 0,
      N_("run as a windows service (SCM only)")},
@@ -197,7 +201,7 @@ static void help(apr_pool_t *pool)
   exit(0);
 }
 
-static svn_error_t * version(apr_getopt_t *os, apr_pool_t *pool)
+static svn_error_t * version(apr_pool_t *pool)
 {
   const char *fs_desc_start
     = _("The following repository back-end (FS) modules are available:\n\n");
@@ -207,7 +211,12 @@ static svn_error_t * version(apr_getopt_t *os, apr_pool_t *pool)
   version_footer = svn_stringbuf_create(fs_desc_start, pool);
   SVN_ERR(svn_fs_print_modules(version_footer, pool));
 
-  return svn_opt_print_help(os, "svnserve", TRUE, FALSE, version_footer->data,
+#ifdef SVN_HAVE_SASL
+  svn_stringbuf_appendcstr(version_footer,
+                           _("\nCyrus SASL authentication is available.\n"));
+#endif
+
+  return svn_opt_print_help(NULL, "svnserve", TRUE, FALSE, version_footer->data,
                             NULL, NULL, NULL, NULL, pool);
 }
 
@@ -304,6 +313,7 @@ int main(int argc, const char *argv[])
   apr_sockaddr_t *sa;
   apr_pool_t *pool;
   apr_pool_t *connection_pool;
+  apr_allocator_t *allocator;
   svn_error_t *err;
   apr_getopt_t *os;
   int opt;
@@ -323,34 +333,30 @@ int main(int argc, const char *argv[])
   const char *host = NULL;
   int family = APR_INET;
   int mode_opt_count = 0;
+  const char *config_filename = NULL;
   const char *pid_filename = NULL;
+  svn_node_kind_t kind;
 
   /* Initialize the app. */
-  if (svn_cmdline_init("svn", stderr) != EXIT_SUCCESS)
+  if (svn_cmdline_init("svnserve", stderr) != EXIT_SUCCESS)
     return EXIT_FAILURE;
 
   /* Create our top-level pool. */
   pool = svn_pool_create(NULL);
 
+#ifdef SVN_HAVE_SASL
+  SVN_INT_ERR(cyrus_init(pool));
+#endif
+
   /* Check library versions */
   err = check_lib_versions();
   if (err)
-    {
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      svn_pool_destroy(pool);
-      return EXIT_FAILURE;
-    }
+    return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
 
   /* Initialize the FS library. */
   err = svn_fs_initialize(pool);
   if (err)
-    {
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      svn_pool_destroy(pool);
-      return EXIT_FAILURE;
-    }
+    return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
 
   err = svn_cmdline__getopt_init(&os, argc, argv, pool);
   if (err)
@@ -360,6 +366,10 @@ int main(int argc, const char *argv[])
   params.tunnel = FALSE;
   params.tunnel_user = NULL;
   params.read_only = FALSE;
+  params.cfg = NULL;
+  params.pwdb = NULL;
+  params.authzdb = NULL;
+
   while (1)
     {
       status = apr_getopt_long(os, svnserve__options, &opt, &arg);
@@ -374,13 +384,16 @@ int main(int argc, const char *argv[])
           break;
 
         case SVNSERVE_OPT_VERSION:
-          SVN_INT_ERR(version(os, pool));
+          SVN_INT_ERR(version(pool));
           exit(0);
           break;
-          
+
         case 'd':
-          run_mode = run_mode_daemon;
-          mode_opt_count++;
+          if (run_mode != run_mode_daemon)
+            {
+              run_mode = run_mode_daemon;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_FOREGROUND:
@@ -388,8 +401,11 @@ int main(int argc, const char *argv[])
           break;
 
         case 'i':
-          run_mode = run_mode_inetd;
-          mode_opt_count++;
+          if (run_mode != run_mode_inetd)
+            {
+              run_mode = run_mode_inetd;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_LISTEN_PORT:
@@ -401,8 +417,11 @@ int main(int argc, const char *argv[])
           break;
 
         case 't':
-          run_mode = run_mode_tunnel;
-          mode_opt_count++;
+          if (run_mode != run_mode_tunnel)
+            {
+              run_mode = run_mode_tunnel;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_TUNNEL_USER:
@@ -410,12 +429,29 @@ int main(int argc, const char *argv[])
           break;
 
         case 'X':
-          run_mode = run_mode_listen_once;
-          mode_opt_count++;
+          if (run_mode != run_mode_listen_once)
+            {
+              run_mode = run_mode_listen_once;
+              mode_opt_count++;
+            }
           break;
 
         case 'r':
           SVN_INT_ERR(svn_utf_cstring_to_utf8(&params.root, arg, pool));
+
+          err = svn_io_check_resolved_path(params.root, &kind, pool);
+          if (err)
+            return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
+          if (kind != svn_node_dir)
+            {
+              svn_error_clear
+                (svn_cmdline_fprintf
+                   (stderr, pool,
+                    _("svnserve: Root path '%s' does not exist "
+                      "or is not a directory.\n"), params.root));
+              return EXIT_FAILURE;
+            }
+
           params.root = svn_path_internal_style(params.root, pool);
           SVN_INT_ERR(svn_path_get_absolute(&params.root, params.root, pool));
           break;
@@ -430,11 +466,21 @@ int main(int argc, const char *argv[])
 
 #ifdef WIN32
         case SVNSERVE_OPT_SERVICE:
-          run_mode = run_mode_service;
-          mode_opt_count++;
+          if (run_mode != run_mode_service)
+            {
+              run_mode = run_mode_service;
+              mode_opt_count++;
+            }
           break;
 #endif
 
+        case SVNSERVE_OPT_CONFIG_FILE:
+          SVN_INT_ERR(svn_utf_cstring_to_utf8(&config_filename, arg, pool));
+          config_filename = svn_path_internal_style(config_filename, pool);
+          SVN_INT_ERR(svn_path_get_absolute(&config_filename, config_filename,
+                                            pool));
+          break;
+
         case SVNSERVE_OPT_PID_FILE:
           SVN_INT_ERR(svn_utf_cstring_to_utf8(&pid_filename, arg, pool));
           pid_filename = svn_path_internal_style(pid_filename, pool);
@@ -455,6 +501,14 @@ int main(int argc, const char *argv[])
       usage(argv[0], pool);
     }
 
+  /* If a configuration file is specified, load it and any referenced
+   * password and authorization files. */
+  if (config_filename)
+      SVN_INT_ERR(load_configs(&params.cfg, &params.pwdb, &params.authzdb,
+                               config_filename, TRUE,
+                               svn_path_dirname(config_filename, pool),
+                               pool));
+
   if (params.tunnel_user && run_mode != run_mode_tunnel)
     {
       svn_error_clear
@@ -473,20 +527,16 @@ int main(int argc, const char *argv[])
       if (status)
         {
           err = svn_error_wrap_apr(status, _("Can't open stdin"));
-          svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-          svn_error_clear(err);
-          exit(1);
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
         }
 
       status = apr_file_open_stdout(&out_file, pool);
       if (status)
         {
           err = svn_error_wrap_apr(status, _("Can't open stdout"));
-          svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-          svn_error_clear(err);
-          exit(1);
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
         }
-                                
+
       conn = svn_ra_svn_create_conn(NULL, in_file, out_file, pool);
       svn_error_clear(serve(conn, &params, pool));
       exit(0);
@@ -545,7 +595,7 @@ int main(int argc, const char *argv[])
 
   /* Make sure we have IPV6 support first before giving apr_sockaddr_info_get
      APR_UNSPEC, because it may give us back an IPV6 address even if we can't
-     create IPV6 sockets. */  
+     create IPV6 sockets. */
 
 #if APR_HAVE_IPV6
 #ifdef MAX_SECS_TO_LINGER
@@ -555,20 +605,18 @@ int main(int argc, const char *argv[])
   status = apr_socket_create(&sock, APR_INET6, SOCK_STREAM, APR_PROTO_TCP,
                              pool);
 #endif
-  if (status == 0)   
+  if (status == 0)
     {
       apr_socket_close(sock);
       family = APR_UNSPEC;
     }
 #endif
-  
+
   status = apr_sockaddr_info_get(&sa, host, family, port, 0, pool);
   if (status)
     {
       err = svn_error_wrap_apr(status, _("Can't get address info"));
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      exit(1);
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
 
@@ -582,9 +630,7 @@ int main(int argc, const char *argv[])
   if (status)
     {
       err = svn_error_wrap_apr(status, _("Can't create server socket"));
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      exit(1);
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
   /* Prevents "socket in use" errors when server is killed and quickly
@@ -595,9 +641,7 @@ int main(int argc, const char *argv[])
   if (status)
     {
       err = svn_error_wrap_apr(status, _("Can't bind server socket"));
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      exit(1);
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
   apr_socket_listen(sock, 7);
@@ -641,10 +685,22 @@ int main(int argc, const char *argv[])
         return ERROR_SUCCESS;
 #endif
 
+      /* If we are using fulltext caches etc., we will allocate many large
+         chunks of memory of various sizes outside the cachde for those
+         fulltexts. Make sure, we use the memory wisely: use an allocator
+         that causes memory fragments to be given back to the OS early. */
+
+      if (apr_allocator_create(&allocator))
+        return EXIT_FAILURE;
+
+      apr_allocator_max_free_set(allocator, SVN_ALLOCATOR_RECOMMENDED_MAX_FREE);
+
       /* Non-standard pool handling.  The main thread never blocks to join
          the connection threads so it cannot clean up after each one.  So
          separate pools, that can be cleared at thread exit, are used */
-      connection_pool = svn_pool_create(NULL);
+
+      connection_pool = svn_pool_create_ex(NULL, allocator);
+      apr_allocator_owner_set(allocator, connection_pool);
 
       status = apr_socket_accept(&usock, sock, connection_pool);
       if (handling_mode == connection_mode_fork)
@@ -663,9 +719,7 @@ int main(int argc, const char *argv[])
         {
           err = svn_error_wrap_apr
             (status, _("Can't accept client connection"));
-          svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-          svn_error_clear(err);
-          exit(1);
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
         }
 
       conn = svn_ra_svn_create_conn(usock, NULL, NULL, connection_pool);
