@@ -35,6 +35,12 @@
 /* Used to terminate lines in large multi-line string literals. */
 #define NL APR_EOL_STR
 
+#ifndef MAX
+#define MAX(a,b) (((a)>(b))?(a):(b))
+#endif
+#ifndef MIN
+#define MIN(a,b) (((a)<(b))?(a):(b))
+#endif
 
 
 static svn_error_t *
@@ -2186,6 +2192,105 @@ reporter_depth_exclude(const char **msg,
 }
 
 
+
+/* Tests for svn_repos_get_logsN() */
+
+/* Log receiver which simple increments a counter. */
+static svn_error_t *
+log_receiver(void *baton,
+             svn_log_entry_t *log_entry,
+             apr_pool_t *pool)
+{
+  int *count = baton;
+  (*count)++;
+  return SVN_NO_ERROR;
+}
+
+
+static svn_error_t *
+get_logs(const char **msg,
+         svn_boolean_t msg_only,
+         svn_test_opts_t *opts,
+         apr_pool_t *pool)
+{
+  svn_repos_t *repos;
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root;
+  svn_revnum_t start, end, youngest_rev = 0;
+  apr_pool_t *subpool = svn_pool_create(pool);
+  
+  *msg = "test svn_repos_get_logs ranges and limits";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* Create a filesystem and repository. */
+  SVN_ERR(svn_test__create_repos(&repos, "test-repo-get-logs",
+                                 opts->fs_type, pool));
+  fs = svn_repos_fs(repos);
+
+  /* Revision 1:  Add the Greek tree. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_test__create_greek_tree(txn_root, subpool));
+  SVN_ERR(svn_repos_fs_commit_txn(NULL, repos, &youngest_rev, txn, subpool));
+
+  /* Revision 2:  Tweak A/mu and A/B/E/alpha. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_test__set_file_contents(txn_root, "A/mu", 
+                                      "Revision 2", subpool));
+  SVN_ERR(svn_test__set_file_contents(txn_root, "A/B/E/alpha", 
+                                      "Revision 2", subpool));
+  SVN_ERR(svn_repos_fs_commit_txn(NULL, repos, &youngest_rev, txn, subpool));
+
+  /* Revision 3:  Tweak A/B/E/alpha and A/B/E/beta. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_test__set_file_contents(txn_root, "A/B/E/alpha", 
+                                      "Revision 3", subpool));
+  SVN_ERR(svn_test__set_file_contents(txn_root, "A/B/E/beta", 
+                                      "Revision 3", subpool));
+  SVN_ERR(svn_repos_fs_commit_txn(NULL, repos, &youngest_rev, txn, subpool));
+
+  
+  for (start = 0; start <= youngest_rev; start++)
+    {
+      for (end = 0; end <= youngest_rev; end++)
+        {
+          svn_revnum_t start_arg = start ? start : SVN_INVALID_REVNUM;
+          svn_revnum_t end_arg   = end ? end : SVN_INVALID_REVNUM;
+          svn_revnum_t eff_start = start ? start : youngest_rev;
+          svn_revnum_t eff_end   = end ? end : youngest_rev;
+          int limit, max_logs =
+            MAX(eff_start, eff_end) + 1 - MIN(eff_start, eff_end);
+          int num_logs;
+
+          for (limit = 0; limit <= max_logs; limit++)
+            {
+              int num_expected = limit ? limit : max_logs;
+
+              svn_pool_clear(subpool);
+              num_logs = 0;
+              SVN_ERR(svn_repos_get_logs4(repos, NULL, start_arg, end_arg, 
+                                          limit, FALSE, FALSE, FALSE, NULL,
+                                          NULL, NULL, log_receiver, &num_logs,
+                                          subpool));
+              if (num_logs != num_expected)
+                return svn_error_createf(SVN_ERR_TEST_FAILED, NULL,
+                                         "Log with start=%ld,end=%ld,limit=%d "
+                                         "returned %d entries (expected %d)", 
+                                         start_arg, end_arg, limit, 
+                                         num_logs, max_logs);
+            }
+        }
+    }
+  svn_pool_destroy(subpool);
+  return SVN_NO_ERROR;
+}
+
+
 
 /* The test table.  */
 
@@ -2203,5 +2308,6 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS(commit_continue_txn),
     SVN_TEST_PASS(node_location_segments),
     SVN_TEST_PASS(reporter_depth_exclude),
+    SVN_TEST_PASS(get_logs),
     SVN_TEST_NULL
   };
