@@ -41,7 +41,7 @@ fail(apr_pool_t *pool, const char *fmt, ...)
   return svn_error_create(SVN_ERR_TEST_FAILED, 0, msg);
 }
 
-#define MAX_NBR_RANGES 3
+#define MAX_NBR_RANGES 5
 
 /* Verify that INPUT is parsed properly, and returns an error if
    parsing fails, or incorret parsing is detected.  Assumes that INPUT
@@ -110,7 +110,8 @@ verify_mergeinfo_parse(const char *input,
    -> merge ranges. */
 static apr_hash_t *info1, *info2;
 
-#define NBR_MERGEINFO_VALS 5
+#define NBR_MERGEINFO_VALS 20
+
 /* Valid mergeinfo values. */
 static const char * const mergeinfo_vals[NBR_MERGEINFO_VALS] =
   {
@@ -118,7 +119,27 @@ static const char * const mergeinfo_vals[NBR_MERGEINFO_VALS] =
     "/trunk/foo:1-6",
     "/trunk: 5,7-9,10,11,13,14",
     "/trunk: 3-10,11*,13,14",
-    "/branch: 1,2-18*,33*"
+    "/branch: 1,2-18*,33*",
+    /* Path names containing ':'s */
+    "patch-common::netasq-bpf.c:25381",
+    "patch-common_netasq-bpf.c::25381",
+    ":patch:common:netasq:bpf.c:25381",
+    /* Unordered rangelists */
+    "/trunk:3-6,15,18,9,22",
+    "/trunk:5,3",
+    "/trunk:3-6*,15*,18*,9,22*",
+    "/trunk:5,3*",
+    "/trunk:100,3-7,50,99,1-2",
+    /* Overlapping rangelists */
+    "/gunther_branch:5-10,7-12",
+    "/gunther_branch:5-10*,7-12*",
+    "/branches/branch1:43832-45742,49990-53669,43832-49987",
+    /* Unordered and overlapping rangelists */
+    "/gunther_branch:7-12,1,5-10",
+    "/gunther_branch:7-12*,1,5-10*",
+    /* Adjacent rangelists of differing inheritability. */
+    "/b5:5-53,1-4,54-90*",
+    "/c0:1-77,12-44"
   };
 /* Paths corresponding to mergeinfo_vals. */
 static const char * const mergeinfo_paths[NBR_MERGEINFO_VALS] =
@@ -127,7 +148,22 @@ static const char * const mergeinfo_paths[NBR_MERGEINFO_VALS] =
     "/trunk/foo",
     "/trunk",
     "/trunk",
-    "/branch"
+    "/branch",
+    "patch-common::netasq-bpf.c",
+    "patch-common_netasq-bpf.c:",
+    ":patch:common:netasq:bpf.c",
+    "/trunk",
+    "/trunk",
+    "/trunk",
+    "/trunk",
+    "/trunk",
+    "/gunther_branch",
+    "/gunther_branch",
+    "/branches/branch1",
+    "/gunther_branch",
+    "/gunther_branch",
+    "/b5",
+    "/c0"
   };
 /* First ranges from the paths identified by mergeinfo_paths. */
 static svn_merge_range_t mergeinfo_ranges[NBR_MERGEINFO_VALS][MAX_NBR_RANGES] =
@@ -136,7 +172,24 @@ static svn_merge_range_t mergeinfo_ranges[NBR_MERGEINFO_VALS][MAX_NBR_RANGES] =
     { {0, 6,  TRUE} },
     { {4, 5,  TRUE}, { 6, 11, TRUE }, {12, 14, TRUE } },
     { {2, 10, TRUE}, {10, 11, FALSE}, {12, 14, TRUE } },
-    { {0, 1,  TRUE}, { 1, 18, FALSE}, {32, 33, FALSE} }
+    { {0, 1,  TRUE}, { 1, 18, FALSE}, {32, 33, FALSE} },
+    { {25380, 25381, TRUE } },
+    { {25380, 25381, TRUE } },
+    { {25380, 25381, TRUE } },
+    { {2, 6, TRUE}, {8, 9, TRUE}, {14, 15, TRUE}, {17, 18, TRUE},
+      {21, 22, TRUE} },
+    { {2, 3, TRUE}, {4, 5, TRUE} },
+    { {2, 6, FALSE}, {8, 9, TRUE}, {14, 15, FALSE}, {17, 18, FALSE},
+      {21, 22, FALSE} },
+    { {2, 3, FALSE}, {4, 5, TRUE} },
+    { {0, 7, TRUE}, {49, 50, TRUE}, {98, 100, TRUE} },
+    { {4, 12, TRUE} },
+    { {4, 12, FALSE} },
+    { {43831, 49987, TRUE}, {49989, 53669, TRUE} },
+    { {0, 1, TRUE}, {4, 12, TRUE} },
+    { {0, 1, TRUE}, {4, 12, FALSE} },
+    { {0, 53, TRUE}, {53, 90, FALSE} },
+    { {0, 77, TRUE} },
   };
 
 static svn_error_t *
@@ -246,7 +299,7 @@ test_parse_combine_rangeinfo(const char **msg,
 }
 
 
-#define NBR_BROKEN_MERGEINFO_VALS 38
+#define NBR_BROKEN_MERGEINFO_VALS 27
 /* Invalid mergeinfo values. */
 static const char * const broken_mergeinfo_vals[NBR_BROKEN_MERGEINFO_VALS] =
   {
@@ -255,11 +308,6 @@ static const char * const broken_mergeinfo_vals[NBR_BROKEN_MERGEINFO_VALS] =
     "/trunk: 5,7-9,10,11,13,14,",
     "/trunk 5,7-9,10,11,13,14",
     "/trunk:5 7--9 10 11 13 14",
-    /* Unordered revs   */
-    "/trunk:3-6,15,18,9,22",
-    "/trunk:5,3",
-    "/trunk:3-6*,15*,18*,9,22*",
-    "/trunk:5,3*",
     /* Overlapping revs differing inheritability */
     "/trunk:5-9*,9",
     "/trunk:5,5-9*",
@@ -269,15 +317,6 @@ static const char * const broken_mergeinfo_vals[NBR_BROKEN_MERGEINFO_VALS] =
     "/trunk:4*,4",
     "/trunk:3-7*,4-23",
     "/trunk:3-7,4-23*",
-    /* Overlapping revs same inheritability */
-    "/trunk:5-9*,9*",
-    "/trunk:5*,5-9*", 
-    "/trunk:5-9,9",
-    "/trunk:5,5-9",
-    "/trunk:4,4",
-    "/trunk:4*,4*",
-    "/trunk:3-7,4-23",
-    "/trunk:3-7*,4-23*",
     /* Reversed revision ranges */
     "/trunk:22-20",
     "/trunk:22-20*",
@@ -291,6 +330,7 @@ static const char * const broken_mergeinfo_vals[NBR_BROKEN_MERGEINFO_VALS] =
     /* path mapped to range with no revisions */
     "/trunk:",
     "/trunk:2-9\n/branch:",
+    "::",
     /* No path */
     ":1-3",
     /* Invalid revisions */
@@ -621,9 +661,9 @@ test_merge_mergeinfo(const char **msg,
                         {12, 13, TRUE}, {13, 22, FALSE} } } } },
 
       /* Two paths all inheritable ranges */
-      { "/trunk: 3,5,7-9,10,11,13,14\n/fred:8-10",
-        "/trunk: 1-4,6\n/fred:9-12", 2,
-        { {"/trunk", { {0, 11, TRUE}, {12, 14, TRUE} } },
+      { "/trunk::1: 3,5,7-9,10,11,13,14\n/fred:8-10",
+        "/trunk::1: 1-4,6\n/fred:9-12", 2,
+        { {"/trunk::1", { {0, 11, TRUE}, {12, 14, TRUE} } },
           {"/fred",  { {7, 12, TRUE} } } } },
 
       /* Two paths all non-inheritable ranges */
@@ -641,11 +681,11 @@ test_merge_mergeinfo(const char **msg,
 
       /* A slew of different paths but no ranges to be merged */
       { "/trunk: 3,5-9*\n/betty: 2-4",
-        "/fred: 1-18\n/barney: 1,3-43", 4,
-        { {"/trunk",  { {2,  3, TRUE}, {4,  9, FALSE} } },
-          {"/betty",  { {1,  4, TRUE} } },
-          {"/barney", { {0,  1, TRUE}, {2, 43, TRUE} } },
-          {"/fred",   { {0, 18, TRUE} } } } }
+        "/fred: 1-18\n/:barney: 1,3-43", 4,
+        { {"/trunk",   { {2,  3, TRUE}, {4,  9, FALSE} } },
+          {"/betty",   { {1,  4, TRUE} } },
+          {"/:barney", { {0,  1, TRUE}, {2, 43, TRUE} } },
+          {"/fred",    { {0, 18, TRUE} } } } }
     };
 
   *msg = "merging of mergeinfo hashs";
