@@ -2,14 +2,14 @@
 #
 # mailer.py: send email describing a commit
 #
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.3.x/tools/hook-scripts/mailer/mailer.py $
-# $LastChangedDate: 2005-12-15 01:57:49 +0100 (Thu, 15 Dec 2005) $
-# $LastChangedBy: maxb $
-# $LastChangedRevision: 17790 $
+# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.5.x/tools/hook-scripts/mailer/mailer.py $
+# $LastChangedDate: 2007-09-25 15:15:57 +0000 (Tue, 25 Sep 2007) $
+# $LastChangedBy: cmpilato $
+# $LastChangedRevision: 866854 $
 #
 # USAGE: mailer.py commit      REPOS REVISION [CONFIG-FILE]
-#        mailer.py propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
-#        mailer.py propchange2 REPOS REVISION AUTHOR PROPNAME ACTION \
+#        mailer.py propchange  REPOS REVISION AUTHOR REVPROPNAME [CONFIG-FILE]
+#        mailer.py propchange2 REPOS REVISION AUTHOR REVPROPNAME ACTION \
 #                              [CONFIG-FILE]
 #        mailer.py lock        REPOS AUTHOR [CONFIG-FILE]
 #        mailer.py unlock      REPOS AUTHOR [CONFIG-FILE]
@@ -21,9 +21,8 @@
 #   in Subversion 1.2.0.  Its value is one of 'A', 'M' or 'D' to indicate
 #   if the property was added, modified or deleted, respectively.
 #
-#   This version of mailer.py requires the python bindings from
-#   subversion 1.2.0 or later.
-#
+#   See _MIN_SVN_VERSION below for which version of Subversion's Python
+#   bindings are required by this version of mailer.py.
 
 import os
 import sys
@@ -38,17 +37,35 @@ import tempfile
 import types
 import urllib
 
-import svn.fs
-import svn.delta
-import svn.repos
-import svn.core
+# Minimal version of Subversion's bindings required
+_MIN_SVN_VERSION = [1, 5, 0]
+
+# Import the Subversion Python bindings, making sure they meet our
+# minimum version requirements.
+try:
+  import svn.fs
+  import svn.delta
+  import svn.repos
+  import svn.core
+except ImportError:
+  sys.stderr.write(
+    "You need version %s or better of the Subversion Python bindings.\n" \
+    % string.join(map(lambda x: str(x), _MIN_SVN_VERSION), '.'))
+  sys.exit(1)
+if _MIN_SVN_VERSION > [svn.core.SVN_VER_MAJOR,
+                       svn.core.SVN_VER_MINOR,
+                       svn.core.SVN_VER_PATCH]:
+  sys.stderr.write(
+    "You need version %s or better of the Subversion Python bindings.\n" \
+    % string.join(map(lambda x: str(x), _MIN_SVN_VERSION), '.'))
+  sys.exit(1)
+
 
 SEPARATOR = '=' * 78
 
-
 def main(pool, cmd, config_fname, repos_dir, cmd_args):
   ### TODO:  Sanity check the incoming args
-  
+
   if cmd == 'commit':
     revision = int(cmd_args[0])
     repos = Repository(repos_dir, revision, pool)
@@ -112,6 +129,11 @@ except AttributeError:
       rv = self.tochild.close() or rv
       return rv
 
+def remove_leading_slashes(path):
+  while path and path[0] == '/':
+    path = path[1:]
+  return path
+
 
 class OutputBase:
   "Abstract base class to formalize the inteface of output methods"
@@ -183,12 +205,31 @@ class MailedOutput(OutputBase):
     OutputBase.__init__(self, cfg, repos, prefix_param)
 
   def start(self, group, params):
-    # whitespace-separated list of addresses; split into a clean list:
-    self.to_addrs = \
-        filter(None, string.split(self.cfg.get('to_addr', group, params)))
+    # whitespace (or another character) separated list of addresses
+    # which must be split into a clean list
+    to_addr_in = self.cfg.get('to_addr', group, params)
+    # if list of addresses starts with '[.]'
+    # use the character between the square brackets as split char
+    # else use whitespaces
+    if len(to_addr_in) >= 3 and to_addr_in[0] == '[' \
+                            and to_addr_in[2] == ']':
+      self.to_addrs = \
+        filter(None, string.split(to_addr_in[3:], to_addr_in[1]))
+    else:
+      self.to_addrs = filter(None, string.split(to_addr_in))
     self.from_addr = self.cfg.get('from_addr', group, params) \
                      or self.repos.author or 'no_author'
+    # if the from_addr (also) starts with '[.]' (may happen if one
+    # map is used for both to_addr and from_addr) remove '[.]'
+    if len(self.from_addr) >= 3 and self.from_addr[0] == '[' \
+                                and self.from_addr[2] == ']':
+      self.from_addr = self.from_addr[3:]
     self.reply_to = self.cfg.get('reply_to', group, params)
+    # if the reply_to (also) starts with '[.]' (may happen if one
+    # map is used for both to_addr and reply_to) remove '[.]'
+    if len(self.reply_to) >= 3 and self.reply_to[0] == '[' \
+                               and self.reply_to[2] == ']':
+      self.reply_to = self.reply_to[3:]
 
   def mail_headers(self, group, params):
     subject = self.make_subject(group, params)
@@ -202,6 +243,7 @@ class MailedOutput(OutputBase):
            'Subject: %s\n' \
            'MIME-Version: 1.0\n' \
            'Content-Type: text/plain; charset=UTF-8\n' \
+           'Content-Transfer-Encoding: 8bit\n' \
            % (self.from_addr, string.join(self.to_addrs, ', '), subject)
     if self.reply_to:
       hdrs = '%sReply-To: %s\n' % (hdrs, self.reply_to)
@@ -244,7 +286,7 @@ class StandardOutput(OutputBase):
 
 
 class PipeOutput(MailedOutput):
-  "Deliver a mail message to an MDA via a pipe."
+  "Deliver a mail message to an MTA via a pipe."
 
   def __init__(self, cfg, repos, prefix_param):
     MailedOutput.__init__(self, cfg, repos, prefix_param)
@@ -298,7 +340,8 @@ class Commit(Messenger):
     Messenger.__init__(self, pool, cfg, repos, 'commit_subject_prefix')
 
     # get all the changes and sort by path
-    editor = svn.repos.ChangeCollector(repos.fs_ptr, repos.root_this, self.pool)
+    editor = svn.repos.ChangeCollector(repos.fs_ptr, repos.root_this, \
+                                       self.pool)
     e_ptr, e_baton = svn.delta.make_editor(editor, self.pool)
     svn.repos.replay(repos.root_this, e_ptr, e_baton, self.pool)
 
@@ -444,7 +487,7 @@ def get_commondir(dirlist):
   a commondir is found, the dirlist returned is rooted in that
   commondir.  If no commondir is found, dirlist is returned unchanged,
   and commondir is the empty string."""
-  if len(dirlist) == 1 or '/' in dirlist:
+  if len(dirlist) < 2 or '/' in dirlist:
     commondir = ''
     newdirs = dirlist
   else:
@@ -571,12 +614,35 @@ class DiffSelections:
 
 
 class DiffURLSelections:
-  def __init__(self, cfg, group):
-    self.add = cfg.get('diff_add_url', group, None)
-    self.copy = cfg.get('diff_copy_url', group, None)
-    self.delete = cfg.get('diff_delete_url', group, None)
-    self.modify = cfg.get('diff_modify_url', group, None)
+  def __init__(self, cfg, group, params):
+    self.cfg = cfg
+    self.group = group
+    self.params = params
 
+  def _get_url(self, action, repos_rev, change):
+    # The parameters for the URLs generation need to be placed in the
+    # parameters for the configuration module, otherwise we may get
+    # KeyError exceptions.
+    params = self.params.copy()
+    params['path'] = change.path and urllib.quote(change.path) or None
+    params['base_path'] = change.base_path and urllib.quote(change.base_path) \
+                          or None
+    params['rev'] = repos_rev
+    params['base_rev'] = change.base_rev
+
+    return self.cfg.get("diff_%s_url" % action, self.group, params)
+
+  def get_add_url(self, repos_rev, change):
+    return self._get_url('add', repos_rev, change)
+
+  def get_copy_url(self, repos_rev, change):
+    return self._get_url('copy', repos_rev, change)
+
+  def get_delete_url(self, repos_rev, change):
+    return self._get_url('delete', repos_rev, change)
+
+  def get_modify_url(self, repos_rev, change):
+    return self._get_url('modify', repos_rev, change)
 
 def generate_content(renderer, cfg, repos, changelist, group, params, paths,
                      pool):
@@ -586,35 +652,44 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
   date = time.ctime(svn.core.secs_from_timestr(svndate, pool))
 
   diffsels = DiffSelections(cfg, group, params)
-  diffurls = DiffURLSelections(cfg, group)
+  diffurls = DiffURLSelections(cfg, group, params)
 
   show_nonmatching_paths = cfg.get('show_nonmatching_paths', group, params) \
       or 'yes'
 
+  params_with_rev = params.copy()
+  params_with_rev['rev'] = repos.rev
+  commit_url = cfg.get('commit_url', group, params_with_rev)
+
   # figure out the lists of changes outside the selected path-space
-  other_added_data = other_removed_data = other_modified_data = [ ]
+  other_added_data = other_replaced_data = other_deleted_data = \
+      other_modified_data = [ ]
   if len(paths) != len(changelist) and show_nonmatching_paths != 'no':
     other_added_data = generate_list('A', changelist, paths, False)
-    other_removed_data = generate_list('R', changelist, paths, False)
+    other_replaced_data = generate_list('R', changelist, paths, False)
+    other_deleted_data = generate_list('D', changelist, paths, False)
     other_modified_data = generate_list('M', changelist, paths, False)
 
   if len(paths) != len(changelist) and show_nonmatching_paths == 'yes':
     other_diffs = DiffGenerator(changelist, paths, False, cfg, repos, date,
-                                group, params, diffsels, diffurls, pool),
+                                group, params, diffsels, diffurls, pool)
   else:
-    other_diffs = [ ]
+    other_diffs = None
 
   data = _data(
     author=repos.author,
     date=date,
     rev=repos.rev,
     log=repos.get_rev_prop(svn.core.SVN_PROP_REVISION_LOG) or '',
+    commit_url=commit_url,
     added_data=generate_list('A', changelist, paths, True),
-    removed_data=generate_list('R', changelist, paths, True),
+    replaced_data=generate_list('R', changelist, paths, True),
+    deleted_data=generate_list('D', changelist, paths, True),
     modified_data=generate_list('M', changelist, paths, True),
     show_nonmatching_paths=show_nonmatching_paths,
     other_added_data=other_added_data,
-    other_removed_data=other_removed_data,
+    other_replaced_data=other_replaced_data,
+    other_deleted_data=other_deleted_data,
     other_modified_data=other_modified_data,
     diffs=DiffGenerator(changelist, paths, True, cfg, repos, date, group,
                         params, diffsels, diffurls, pool),
@@ -625,11 +700,13 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
 
 def generate_list(changekind, changelist, paths, in_paths):
   if changekind == 'A':
-    selection = lambda change: change.added
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_ADD
   elif changekind == 'R':
-    selection = lambda change: change.path is None
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_REPLACE
+  elif changekind == 'D':
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_DELETE
   elif changekind == 'M':
-    selection = lambda change: not change.added and change.path is not None
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_MODIFY
 
   items = [ ]
   for path, change in changelist:
@@ -639,8 +716,10 @@ def generate_list(changekind, changelist, paths, in_paths):
         is_dir=change.item_kind == svn.core.svn_node_dir,
         props_changed=change.prop_changes,
         text_changed=change.text_changed,
-        copied=change.added and change.base_path,
-        base_path=change.base_path,
+        copied=(change.action == svn.repos.CHANGE_ACTION_ADD \
+                or change.action == svn.repos.CHANGE_ACTION_REPLACE) \
+               and change.base_path,
+        base_path=remove_leading_slashes(change.base_path),
         base_rev=change.base_rev,
         )
       items.append(item)
@@ -665,23 +744,14 @@ class DiffGenerator:
     self.diffurls = diffurls
     self.pool = pool
 
+    self.diff = self.diff_url = None
+
     self.idx = 0
 
   def __nonzero__(self):
     # we always have some items
     return True
 
-  def _gen_url(self, urlstr, repos_rev, change):
-    if not len(urlstr):
-      return None
-    args = {
-      'path' : change.path and urllib.quote(change.path) or None,
-      'base_path' : change.base_path and urllib.quote(change.base_path) or None,
-      'rev' : repos_rev,
-      'base_rev' : change.base_rev,
-      }
-    return urlstr % args
-    
   def __getitem__(self, idx):
     while 1:
       if self.idx == len(self.changelist):
@@ -699,7 +769,7 @@ class DiffGenerator:
       binary = None
       singular = None
       content = None
-      
+
       # just skip directories. they have no diffs.
       if change.item_kind == svn.core.svn_node_dir:
         continue
@@ -708,63 +778,83 @@ class DiffGenerator:
       if self.paths.has_key(path) != self.in_paths:
         continue
 
+      if change.base_rev != -1:
+        svndate = self.repos.get_rev_prop(svn.core.SVN_PROP_REVISION_DATE,
+                                          change.base_rev)
+        ### pick a different date format?
+        base_date = time.ctime(svn.core.secs_from_timestr(svndate, self.pool))
+      else:
+        base_date = ''
+
       # figure out if/how to generate a diff
 
-      if not change.path:
+      base_path = remove_leading_slashes(change.base_path)
+      if change.action == svn.repos.CHANGE_ACTION_DELETE:
         # it was delete.
         kind = 'D'
 
-        # show the diff url?
-        if self.diffurls.delete:
-          diff_url = self._gen_url(self.diffurls.delete,
-                                   self.repos.rev, change)
+        # get the diff url, if any is specified
+        diff_url = self.diffurls.get_delete_url(self.repos.rev, change)
 
         # show the diff?
         if self.diffsels.delete:
           diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                 change.base_path, None, None, self.pool)
+                                 base_path, None, None, self.pool)
 
-          label1 = '%s\t%s' % (change.base_path, self.date)
-          label2 = '(empty file)'
+          label1 = '%s\t%s\t(r%s)' % (base_path, self.date, change.base_rev)
+          label2 = '/dev/null\t00:00:00 1970\t(deleted)'
           singular = True
-          
-      elif change.added:
-        if change.base_path and (change.base_rev != -1):
-          # this file was copied.
-          kind = 'C'
+
+      elif change.action == svn.repos.CHANGE_ACTION_ADD \
+           or change.action == svn.repos.CHANGE_ACTION_REPLACE:
+        if base_path and (change.base_rev != -1):
 
           # any diff of interest?
           if change.text_changed:
+            # this file was copied and modified.
+            kind = 'W'
 
-            # show the diff url?
-            if self.diffurls.copy:
-              diff_url = self._gen_url(self.diffurls.copy,
-                                       self.repos.rev, change)
+            # get the diff url, if any is specified
+            diff_url = self.diffurls.get_copy_url(self.repos.rev, change)
 
             # show the diff?
-            if self.diffsels.copy:
+            if self.diffsels.modify:
               diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                     change.base_path,
+                                     base_path,
                                      self.repos.root_this, change.path,
                                      self.pool)
-              label1 = change.base_path + '\t(original)'
-              label2 = '%s\t%s' % (change.path, self.date)
+              label1 = '%s\t%s\t(r%s, copy source)' \
+                       % (base_path, base_date, change.base_rev)
+              label2 = '%s\t%s\t(r%s)' \
+                       % (change.path, self.date, self.repos.rev)
+              singular = False
+          else:
+            # this file was copied.
+            kind = 'C'
+            if self.diffsels.copy:
+              diff = svn.fs.FileDiff(None, None, self.repos.root_this,
+                                     change.path, self.pool)
+              label1 = '/dev/null\t00:00:00 1970\t' \
+                       '(empty, because file is newly added)'
+              label2 = '%s\t%s\t(r%s, copy of r%s, %s)' \
+                       % (change.path, self.date, self.repos.rev, \
+                          change.base_rev, base_path)
               singular = False
         else:
           # the file was added.
           kind = 'A'
 
-          # show the diff url?
-          if self.diffurls.add:
-            diff_url = self._gen_url(self.diffurls.add,
-                                     self.repos.rev, change)
+          # get the diff url, if any is specified
+          diff_url = self.diffurls.get_add_url(self.repos.rev, change)
 
           # show the diff?
           if self.diffsels.add:
             diff = svn.fs.FileDiff(None, None, self.repos.root_this,
                                    change.path, self.pool)
-            label1 = '(empty file)'
-            label2 = '%s\t%s' % (change.path, self.date)
+            label1 = '/dev/null\t00:00:00 1970\t' \
+                     '(empty, because file is newly added)'
+            label2 = '%s\t%s\t(r%s)' \
+                     % (change.path, self.date, self.repos.rev)
             singular = True
 
       elif not change.text_changed:
@@ -774,19 +864,19 @@ class DiffGenerator:
         # a simple modification.
         kind = 'M'
 
-        # show the diff url?
-        if self.diffurls.modify:
-          diff_url = self._gen_url(self.diffurls.modify,
-                                   self.repos.rev, change)
+        # get the diff url, if any is specified
+        diff_url = self.diffurls.get_modify_url(self.repos.rev, change)
 
         # show the diff?
         if self.diffsels.modify:
           diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                 change.base_path,
+                                 base_path,
                                  self.repos.root_this, change.path,
                                  self.pool)
-          label1 = change.base_path + '\t(original)'
-          label2 = '%s\t%s' % (change.path, self.date)
+          label1 = '%s\t%s\t(r%s)' \
+                   % (base_path, base_date, change.base_rev)
+          label2 = '%s\t%s\t(r%s)' \
+                   % (change.path, self.date, self.repos.rev)
           singular = False
 
       if diff:
@@ -805,7 +895,7 @@ class DiffGenerator:
       # return a data item for this diff
       return _data(
         path=change.path,
-        base_path=change.base_path,
+        base_path=base_path,
         base_rev=change.base_rev,
         diff=diff,
         diff_url=diff_url,
@@ -864,6 +954,9 @@ class DiffContent:
     else:
       ltype = 'U'
 
+    if line[-2] == '\r':
+      line=line[0:-2] + '\n' # remove carriage return
+
     return _data(
       raw=line,
       text=line[1:-1],  # remove indicator and newline
@@ -882,30 +975,39 @@ class TextCommitRenderer:
 
     w = self.output.write
 
-    w('Author: %s\nDate: %s\nNew Revision: %s\n\n'
-      % (data.author, data.date, data.rev))
+    w('Author: %s\nDate: %s\nNew Revision: %s\n' % (data.author,
+                                                      data.date,
+                                                      data.rev))
+
+    if data.commit_url:
+      w('URL: %s\n\n' % data.commit_url)
+    else:
+      w('\n')
+
+    w('Log:\n%s\n\n' % data.log.strip())
 
     # print summary sections
     self._render_list('Added', data.added_data)
-    self._render_list('Removed', data.removed_data)
+    self._render_list('Replaced', data.replaced_data)
+    self._render_list('Deleted', data.deleted_data)
     self._render_list('Modified', data.modified_data)
 
-    if data.other_added_data or data.other_removed_data \
-           or data.other_modified_data:
+    if data.other_added_data or data.other_replaced_data \
+           or data.other_deleted_data or data.other_modified_data:
       if data.show_nonmatching_paths:
         w('\nChanges in other areas also in this revision:\n')
         self._render_list('Added', data.other_added_data)
-        self._render_list('Removed', data.other_removed_data)
+        self._render_list('Replaced', data.other_replaced_data)
+        self._render_list('Deleted', data.other_deleted_data)
         self._render_list('Modified', data.other_modified_data)
       else:
         w('and changes in other areas\n')
 
-    w('\nLog:\n%s\n' % data.log)
-
-    self._render_diffs(data.diffs)
+    self._render_diffs(data.diffs, '')
     if data.other_diffs:
-      w('\nDiffs of changes in other areas also in this revision:\n')
-      self._render_diffs(data.other_diffs)
+      self._render_diffs(data.other_diffs,
+                         '\nDiffs of changes in other areas also'
+                         ' in this revision:\n')
 
   def _render_list(self, header, data_list):
     if not data_list:
@@ -936,41 +1038,47 @@ class TextCommitRenderer:
         w('      - copied%s from r%d, %s%s\n'
           % (text, d.base_rev, d.base_path, is_dir))
 
-  def _render_diffs(self, diffs):
+  def _render_diffs(self, diffs, section_header):
+    """Render diffs. Write the SECTION_HEADER if there are actually
+    any diffs to render."""
+    if not diffs:
+      return
     w = self.output.write
+    section_header_printed = False
 
     for diff in diffs:
       if not diff.diff and not diff.diff_url:
         continue
+      if not section_header_printed:
+        w(section_header)
+        section_header_printed = True
       if diff.kind == 'D':
         w('\nDeleted: %s\n' % diff.base_path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
+      elif diff.kind == 'A':
+        w('\nAdded: %s\n' % diff.path)
       elif diff.kind == 'C':
         w('\nCopied: %s (from r%d, %s)\n'
           % (diff.path, diff.base_rev, diff.base_path))
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
-      elif diff.kind == 'A':
-        w('\nAdded: %s\n' % diff.path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
+      elif diff.kind == 'W':
+        w('\nCopied and modified: %s (from r%d, %s)\n'
+          % (diff.path, diff.base_rev, diff.base_path))
       else:
         # kind == 'M'
         w('\nModified: %s\n' % diff.path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
+
+      if diff.diff_url:
+        w('URL: %s\n' % diff.diff_url)
 
       if not diff.diff:
         continue
-      
+
       w(SEPARATOR + '\n')
 
       if diff.binary:
         if diff.singular:
           w('Binary file. No diff available.\n')
         else:
-          w('Binary files. No diff available.\n')
+          w('Binary file (source and/or target). No diff available.\n')
         continue
 
       for line in diff.content:
@@ -994,8 +1102,10 @@ class Repository:
 
     self.author = self.get_rev_prop(svn.core.SVN_PROP_REVISION_AUTHOR)
 
-  def get_rev_prop(self, propname):
-    return svn.fs.revision_prop(self.fs_ptr, self.rev, propname, self.pool)
+  def get_rev_prop(self, propname, rev = None):
+    if not rev:
+      rev = self.rev
+    return svn.fs.revision_prop(self.fs_ptr, rev, propname, self.pool)
 
   def get_root(self, rev):
     try:
@@ -1069,7 +1179,7 @@ class Config:
       value = getattr(sub, option, None)
     if value is None:
       value = getattr(self.defaults, option, '')
-    
+
     # parameterize it
     if params is not None:
       value = value % params
@@ -1079,6 +1189,11 @@ class Config:
     if mapper is not None:
       value = mapper(value)
 
+      # Apply any parameters that may now be available for
+      # substitution that were not before the mapping.
+      if value is not None and params is not None:
+        value = value % params
+
     return value
 
   def get_diff_cmd(self, group, args):
@@ -1107,7 +1222,8 @@ class Config:
         # then just return the value unchanged.
         setattr(self.maps, optname,
                 lambda value,
-                       sect=getattr(self, sectname): getattr(sect, value,
+                       sect=getattr(self, sectname): getattr(sect,
+                                                             value.lower(),
                                                              value))
         # mark for removal when all optnames are done
         if sectname not in mapsections:
@@ -1134,12 +1250,13 @@ class Config:
 
     # compute the default repository-based parameters. start with some
     # basic parameters, then bring in the regex-based params.
-    default_params = self._global_params.copy()
+    self._default_params = self._global_params
 
     try:
       match = re.match(self.defaults.for_repos, repos_dir)
       if match:
-        default_params.update(match.groupdict())
+        self._default_params = self._default_params.copy()
+        self._default_params.update(match.groupdict())
     except AttributeError:
       # there is no self.defaults.for_repos
       pass
@@ -1147,12 +1264,12 @@ class Config:
     # select the groups that apply to this repository
     for group in self._groups:
       sub = getattr(self, group)
-      params = default_params
+      params = self._default_params
       if hasattr(sub, 'for_repos'):
         match = re.match(sub.for_repos, repos_dir)
         if not match:
           continue
-        params = self._global_params.copy()
+        params = params.copy()
         params.update(match.groupdict())
 
       # if a matching rule hasn't been given, then use the empty string
@@ -1172,7 +1289,7 @@ class Config:
       self._group_re.append((None,
                              re.compile(self.defaults.for_paths),
                              None,
-                             default_params))
+                             self._default_params))
     except AttributeError:
       # there is no self.defaults.for_paths
       pass
@@ -1189,7 +1306,7 @@ class Config:
         params.update(match.groupdict())
         groups.append((group, params))
     if not groups:
-      groups.append((None, self._global_params))
+      groups.append((None, self._default_params))
     return groups
 
 
@@ -1224,8 +1341,8 @@ if __name__ == '__main__':
     scriptname = os.path.basename(sys.argv[0])
     sys.stderr.write(
 """USAGE: %s commit      REPOS REVISION [CONFIG-FILE]
-       %s propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
-       %s propchange2 REPOS REVISION AUTHOR PROPNAME ACTION [CONFIG-FILE]
+       %s propchange  REPOS REVISION AUTHOR REVPROPNAME [CONFIG-FILE]
+       %s propchange2 REPOS REVISION AUTHOR REVPROPNAME ACTION [CONFIG-FILE]
        %s lock        REPOS AUTHOR [CONFIG-FILE]
        %s unlock      REPOS AUTHOR [CONFIG-FILE]
 
@@ -1255,7 +1372,7 @@ if the property was added, modified or deleted, respectively.
     usage()
 
   cmd = sys.argv[1]
-  repos_dir = sys.argv[2]
+  repos_dir = svn.core.svn_path_canonicalize(sys.argv[2])
   try:
     expected_args = cmd_list[cmd]
   except KeyError:
