@@ -1,13 +1,17 @@
 #!/usr/bin/env python
 
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.3.x/tools/hook-scripts/svnperms.py $
-# $LastChangedDate: 2005-09-23 20:19:04 +0200 (Fri, 23 Sep 2005) $
-# $LastChangedBy: niemeyer $
-# $LastChangedRevision: 16232 $
+# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.5.x/tools/hook-scripts/svnperms.py $
+# $LastChangedDate: 2007-08-26 01:04:48 +0000 (Sun, 26 Aug 2007) $
+# $LastChangedBy: hwright $
+# $LastChangedRevision: 866391 $
 
 import commands
 import sys, os
 import getopt
+try:
+    my_getopt = getopt.gnu_getopt
+except AttributeError:
+    my_getopt = getopt.getopt
 import re
 
 __author__ = "Gustavo Niemeyer <gustavo@niemeyer.net>"
@@ -95,11 +99,11 @@ class Permission:
     def __init__(self):
         self._group = {}
         self._permlist = []
-    
+
     def parse_groups(self, groupsiter):
         for option, value in groupsiter:
             self._group[option] = value.split()
-            
+
     def parse_perms(self, permsiter):
         for option, value in permsiter:
             # Paths never start with /, so remove it if provided
@@ -253,7 +257,7 @@ class MissingArgumentsException(Exception):
 
 def parse_options():
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "f:s:r:t:R:A:h", ["help"])
+        opts, args = my_getopt(sys.argv[1:], "f:s:r:t:R:A:h", ["help"])
     except getopt.GetoptError, e:
         raise Error, e.msg
     class Options: pass
@@ -299,7 +303,7 @@ def parse_options():
             os.path.isfile(os.path.join(obj.repository, "format"))):
         raise Error, "path '%s' doesn't look like a repository" % \
                      obj.repository
-        
+
     return obj
 
 def main():
